/*
Macro Name	: Get_Chi_Square_Table
Function	: Computes 
			  
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	:This macro is called while loading the star-schema for calculating Chi-Sq p-value.
Calls		:convert_to_perstring and Chi_Sq_Table

Parameters	: 

input_ds		: Name of the input data set 
output_ds		: Name of the data set which stores which stores chi-sq p-value for variable .
	
Input Dataset(s) Format:
input_ds:
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
Attr_Name          	Character  	$5. 
No_of_ACC_Dev       Numeric		Default Format
No_of_ACC_actual    Numeric		Default Format
No_of_events_Dev    Numeric		Default Format
No_of_events_actual Numeric		Default Format
Var_Name			Character  	$4.
------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds:
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
Var_name		Character	Default Format		
p_value			Numeric		p-value format
------------------------------------------------------------------------

Logic:
	1. Computes Percentage of bads per attribute in development sample.
	2. Call macro to convert percentage of bads sequence as string
	3. Calls the macro Chi-Square Table to compute chi-square p-vale.
Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.
*/
%macro get_chi_square_table(input_ds=,output_ds=);
%let err_code=;
/*****************************************************************/
%check_alldefaults_nodefaults(&input_ds,No_Of_ACC_Actual,No_of_events_Dev, err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
proc sql noprint;
create table work.sum as 
	select var_name, attr_name, No_of_events_actual, 100*No_of_events_Dev/sum(No_of_events_Dev) as Pbads_Dev 
    from &input_ds group by var_Name ;
quit;
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
%convert_to_perstring(input_ds=sum,percentage = Pbads_Dev,output_ds=temp1);
%chi_sq_table(input_ds=sum, input_pct_ds=temp1,Freq=No_of_events_actual,output_ds= &output_ds);
%return;
/*****************************************************************/
%Error_Handler:
%put ERROR: &err_code;
%defdata(input_ds=&input_ds,stat='Getchi',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);
%mend get_chi_square_table; 


/*
Macro Name	: convert_to_perstring
Function	: Converts it any column in to string separated by space.

Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	: Get_Chi_Square_Table
Calls		: None

Parameters	: 

input_ds		: Name of the input data set.
percentage		: Name of the column which stores percentage values in the input data set.
output_ds		: Name of the data set which stores percentage screen.
	
Input Dataset(s) Format:
input_ds:
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
Attr_Name          	Character  	$5. 
Pbads_Dev		    Numeric		Default Format
No_of_events_actual Numeric		Default Format
Var_Name			Character  	$4.
------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds:
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
Var_nm			Character	Default Format		
testp			Character	Default Format
------------------------------------------------------------------------

Logic:
1. For a given column (in percentage) agument it to form string for given variable.
2. When variable changes...
	set the string and variable in the output data set
	reinitialize the string and repeat step 1.

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
%macro convert_to_perstring(input_ds=,percentage =,output_ds=);
data &output_ds(keep= Var_nm testp);
set &input_ds end=last;
length testp $ 100.;
retain testp _CHAR_;
retain Var_nm _character_;

if var_name NE Var_nm then
do;
	if Var_nm NE "" then
		output ;
	Var_nm = var_name;/*Assign New Variable Name*/
	testp ="";
end;
testp=ktrim(testp) || "" || Put(&percentage, 8.4);
testp=ktrim(kleft(testp));
if last = 1 then
	output ;
run;
%mend convert_to_perstring; 


/*
Macro Name	: Chi_Sq_Table
Function	: Calls the chi_sq_cal
			  
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	: Get_Chi_Square_Table
Calls		: Chi_Sq_Calc

Parameters	: 

input_ds		: Name of the input data set 
input_pct_ds	: Name of the dat set which stores percentage string of each variable
Freq			: Name of the variable which strore observed frequency.
output_ds		: Name of the data set which stores p-values.
	
Input Dataset(s) Format:
input_ds:
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
Attr_Name          	Character  	$5. 
Pbads_Dev		    Numeric		Default Format
No_of_events_actual Numeric		Default Format
Var_Name			Character  	$4.
------------------------------------------------------------------------

input_pct_ds:
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
Var_nm			Character	Default Format		
testp			Character	Default Format
------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds:
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
Var_name		Character	Default Format		
p_value			Numeric		p-value format
------------------------------------------------------------------------

Logic:
1. Reads percent strings for all variables from input_pct_ds and stores them into array
2. For every variable
	a. selects appropriate frequecy from given frquency column
	b. selects appropriate string 
	c. Calls macro to calculate chi-sq p-value between observed and expected frequency.

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
%macro chi_sq_table(input_ds=, input_pct_ds=,Freq=,output_ds=);
/*CS4.3 HF2 */
/*The value of i has been declared local as this is used in above parent macro also.*/
%local i;
proc sql noprint ;
      select count(*) into :num_obs
      from &input_pct_ds;

	  %let num_obs=%sysfunc(ktrim(%sysfunc(kleft(&num_obs))));

      select testp into :_testp_1 - :_testp_&num_obs
      from &input_pct_ds;

      select var_Nm into :_VarNm_1 - :_VarNm_&num_obs
      from &input_pct_ds;
quit;

data &output_ds;
      length Var_name $15.;
	  format p_value &CS_PVAL_FMT.;
      stop;
run;

%do i = 1 %to &num_obs; 
	%let var =%sysfunc(ktrim(&&_VarNm_&i));
	data temp;
	set &input_ds;
	where Var_Name= "&var";
	run;
	%let pct =&&_testp_&i;
	%chi_sq_calc(input_ds=temp,freq=&Freq,pct=&pct,output_ds=Chisq);
	proc sql noprint ;
	    select P_PCHI into :P_PCHI from Chisq; 
	quit;
	proc sql noprint;
	    insert into &output_ds values("&var",&P_PCHI);
	quit;
%end;
%mend chi_sq_table;


/*
Macro Name	: Chi_Sq_Calc
Function	: Computes chi square p-value for given observed frequency and given percent string.
			  
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	:Chi_Sq_Table 
Calls		:

Parameters	: 

input_ds		: Name of the input data set 
output_ds		: Name of the data set which stores brier score.
	
Input Dataset(s) Format:
input_ds:
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
Attr_Name          	Character  	$5. 
Pbads_Dev		    Numeric		Default Format
No_of_events_actual Numeric		Default Format
Var_Name			Character  	$4.
------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds:
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
P_PCHI			Numeric		p-value format
Test_stats		Numeric		Default Format
------------------------------------------------------------------------

Logic:
	1. Calculates chi-sq p-value and test statistic using proc freq.
Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.
*/

%macro chi_sq_calc(input_ds=,freq=,pct=,output_ds=);
data &output_ds;
N =.;
_PCHI_ =.;
DF_PCHI=.;
P_PCHI=.;
run;
proc sql noprint ;
select min(No_of_events_actual), min(Pbads_Dev)into :min_act ,:min_Dev from &input_ds;
quit; 

%if  (%sysevalf(&min_act * &min_Dev))  ne 0 %then
%do; 
	proc freq data=&input_ds order=data noprint;
      weight &freq;
      tables Attr_Name / nocum testp=(&pct);
	  output out=&output_ds chiSq; 
	run;
%end;

%mend chi_sq_calc;

