/*
Macro Name	: Brier
Function	: Computes Brier score.
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	:Brier Score Screen , Stored proc Name(s) : Check with Deepa.
Calls		:None

Parameters	: 

input_ds		: Name of the input data set 
output_ds		: Name of the data set which stores brier score.
 
Input Dataset(s) Format:
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq	 		Number		Default Formatting
No_Of_ACC_Actual	Number		Default Formatting
No_of_Acc_Dev	 	Number 		Default Formatting
PD	 				Number		Default Formatting
Score_Points	 	Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------

Output Dataset(s) Format:

output_ds
-------------------------------------------------------------------------
Name	Type		Format
------------------------------------------------------------------------
Brier	Numeric		Default Format
------------------------------------------------------------------------

Logic:
1. Nomial Brier score is calculated using formula SUM(# acc_i *PD_i*PD_i+# acc_i*(1-2*PD_i)/Total Accounts;
2. Trivial Brie score is calculated using formula Default Rate * (1 -Default Rate), 
   where Default Rate = Total Number of Bads/ Total Number of Accounts.
3. Brier Score is given ratio Nominal brier Score and Trivial Brier score. 
   When Nominal brier Score = Trivial Brier score => Brier Score = 1 => Model as good as Random Model
   When Nominal brier Score > Trivial Brier score => Brier Score > 1 => Model as better than as Random Model
   When Nominal brier Score < Trivial Brier score => Brier Score < 1 => Model as worse than as Random Model	
Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
%macro brier(input_ds=,output_ds=);
%let err_code=;
/*****************************************************************/
%check_alldefaults_nodefaults(&input_ds,No_Of_ACC_Actual,No_Of_Actual_Bads, err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
proc sql noprint;
	select sum (No_Of_Actual_Bads) into:Bads from &input_ds;
	select sum (No_Of_ACC_Actual) into:Total_acc from &input_ds;
quit;

data Brier1;
	set &input_ds;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_brrier.1, noquote))= No_of_ACC_Actual*PD*PD+No_Of_Actual_Bads*(1-2*PD);
	Trivial_Brier = (&Bads/&Total_acc)*(1-&Bads/&Total_acc);
run;

proc sql noprint;
	create table &output_ds as 
	select 1-(sum(Brier)/(&Total_acc)/max(Trivial_Brier)) as %sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_brrier.1, noquote)) 
	from Brier1;
quit;
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/

%return;
/*****************************************************************/
%Error_Handler:
%put ERROR: &err_code;
%defdata(input_ds=&input_ds,stat='Brier',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);

%mend brier;

