/*
Macro Name	: Conf_Matrix
Function	: Generates confusion matrix from given input and calculates various measures. 
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	:Confusion Matrix screen , Stored proc Name(s) : Check with Deepa.
Calls		:None

Parameters	: 

input_ds			: Name of the input data set 
output_ConfMat_ds	: Name of the data set stores Confusion matrix
output_Msr_ds		: Nane fo the date set which stores various measures calculated based on confustion matrix.

 
Input Dataset(s) Format:
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
ACCOUNT_RK          Numeric		Default Format
ACTUAL_GOOD_BAD	    Numeric		Default Format
PD                  Numeric		Default Format
PREDICTED_GOOD_BAD  Numeric		Default Format
scorecard_points    Numeric		Default Format
------------------------------------------------------------------------
Output Dataset(s) Format:

output_ConfMat_ds
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
PREDICTED_GOOD_BAD	Numeric		Default Format
ACTUAL_GOOD_BAD		Numeric		Default Format
count				Numeric		Default Format
------------------------------------------------------------------------

output_Msr_ds
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
Measure_nm		Character	Default Format
Measure_value	Numeric		Default Format
------------------------------------------------------------------------

Logic:

1. Generate 2X2 table based on variables which stores Actual Good&Bad and Predicted Good&Bad.
2. Calculate various measures like Accuracy, Sensetivity, Specificity and Precision based on Confusion Matrix
   using following formulae	

	Accuracy    =Correct predictions/Total Accounts;
	Error_rate   =1.0 -Accuracy;
	Sensitivity  = True Negatives/Total Negativs;
	Specificity  = True Positives/ Total Positives;
	Precision = Rate of correctly predicted bads (True Negatives/ Total Actual Negatives;
 
Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/

/*Prints Confusion matrix and related measures */
/*CS4.3 HF2 */
%macro conf_matrix(input_ds=,output_ConfMat_ds=, output_Msr_ds=);
%Let GOOD=0;
%Let BAD=1;
%Let TP=0;
%Let FN=0;
%Let FP=0;
%Let TN=0;

proc freq data=&input_ds noprint;
tables PREDICTED_GOOD_BAD*ACTUAL_GOOD_BAD /out=&output_ConfMat_ds(drop=PERCENT) norow nocol nopercent;
run;

data _NULL_;
set &output_ConfMat_ds;
if PREDICTED_GOOD_BAD=&GOOD and ACTUAL_GOOD_BAD=&GOOD then
	call symput('TN',count);
else if PREDICTED_GOOD_BAD=&GOOD and ACTUAL_GOOD_BAD=&BAD then
	call symput('FN',count);
else if PREDICTED_GOOD_BAD=&BAD and ACTUAL_GOOD_BAD=&GOOD then
	call symput('FP',count);
else if PREDICTED_GOOD_BAD=&BAD and ACTUAL_GOOD_BAD=&BAD then
	call symput('TP',count);
run;

/*Prints Accuracy and Sensetivity Measures*/
data work.measure;
%if %eval(&tp + &fp + &tn + &fn) ne 0 %then  %do;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_accuracy.1, noquote))=(&TP+&TN)/(&tp + &fp + &tn + &fn);
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_error_rate.1, noquote))=1.0 -%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_accuracy.1, noquote));
%end;
%else %do;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_accuracy.1, noquote))=.;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_error_rate.1, noquote))=1.0 -.;
%end;
%if  %eval(&FN+&TP) ne 0 %then %do;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_sensitivity.1, noquote))= &TP/(&FN+&TP);
%end;
%else 
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_sensitivity.1, noquote))=.;
%if  %eval(&TN+&FP) ne 0 %then %do;
 	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_specificity.1, noquote))= &TN/(&TN+&FP);
%end;
%else 
  	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_specificity.1, noquote))=.;
%if %eval(&TP+&FP) ne 0 %then %do;
 	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_precision.1, noquote))= &TP/ (&TP+&FP);
%end;
%else %do;
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_precision.1, noquote))=.;
%end;
run;

 

proc transpose data=work.measure out=&output_Msr_ds NAME=Measure_nm;
run;

data &output_Msr_ds;
set &output_Msr_ds(rename=(COL1=Measure_value));
label Measure_nm=;
/*if Measure_nm="Error_rate" then*/
/*	Measure_nm="Error Rate";*/
run;

%mend conf_matrix;
