/*
Macro Name	: cs_rep_data_prep
Function	: Generates Confusion Matrix for every cut-off for the given input.
			  The data-set generated will be input to calculation various statistics 
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	: 9.1

Called-by	:Data preparation stored proc. This macro is called before calculation of various statistic
			 like ROC, AUC, Pietra, BER etc. Applicable for PD based pooling scheme.
Calls		:None

Parameters	: 

input_ds		: Name of the input data set
output_ds		: Name of the data set which stores confusion matrix.
dist_type		: Type of Variable (Actual /Estimated)
				  Possible Values ACT/EST	
Input Dataset(s) Format:
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq	 		Number		Default Formatting
No_Of_ACC_Actual	Number		Default Formatting
No_of_Acc_Dev	 	Number 		Default Formatting
PD	 				Number		Default Formatting
Score_Points	 	Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------

Output Dataset(s) Format:
output_ds
-------------------------------------------------------------------------
Name	Type		Format
------------------------------------------------------------------------
score	Numeric 	Default Format			
FN		Numeric 	Default Format
TP		Numeric 	Default Format
TN		Numeric 	Default Format
FP		Numeric 	Default Format
PD		Numeric 	Default Format
------------------------------------------------------------------------
Logic:
1. Depending upon Flag Actual or Expected bads are considered in confusion matrix calculation.
2. Every score point(can be extended to PD) for a pool is considered as as the cut off. All accounts
   in earlier pools are considered to be bad and all the accounts in later pools are considered to be good. 
3. If this cut-off is true cut-off then we will find maximum actual bads in earlier pools and actual goods in later pools.
4. There will be some goods in the earlier pools and some bads in the earlier pools. This will lead to the confusion matrix.
		Actual 
	---------------
 	|Good	| Bad	|Total	|
--------------------|-------|
Bad	|BGood  |BBad	|Bads	|
-------------------	|-------|
Good|GGood	|GBad 	|Goods  |
-------------------	|-------|
5. Using this logic confusion matrix is calculated using per pool score-point.

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.


*/

%macro cs_rep_data_prep(input_ds=,output_ds=,dist_type=);
/*
data _NULL_;
	name=put(time(),mmss8.1);
	name1=kscan(name,1,":.");
	name2=kscan(name,2,":.");
	name3=kscan(name,3,":."); 
	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	call symput('log_name',"&LOG_PATH"||"cs_rep_data_prep_"||ktrim(kleft(name4))||".log");
run;
proc printto log="&log_name";
run;
*/
/* Considers each pool_maximum as cutoff and computes the confusion matrix for each cut-off 
For all cut-offs in a table we will generate following table
		Actual 
	---------------
 	|Good	| Bad	|Total	|
--------------------|-------|
Bad	|BGood  |BBad	|Bads	|
-------------------	|-------|
Good|GGood	|GBad 	|Goods  |
-------------------	|-------|
*/
/*In few macro the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
%if &dist_type="ACT" %then %do;
	data input;
		set &input_ds(rename=(No_of_Actual_bads=No_of_bads));
	run;
%end;
%else %if &dist_type="EST" %then %do;
	data input;
		set &input_ds(drop=No_of_Actual_bads);
		No_of_bads = PD* No_Of_ACC_Actual;
	run;
%end;

data temp_score;
set input(keep=Score_Points);
rename Score_Points=score;
run;

proc sql;
	/*creating the temporary tables and  taking values above score points*/
	/*------- For creating First Row of the table for each cut-off 
	Bad	|BGood  |BBad	|Bads|
	-------------------	|-------|
	*/
	create table temp1 as
	select No_Of_ACC_Actual , No_Of_Bads,Score_Points,score
	from input,temp_score
	where Score_Points <= score;

	/*creating the tables for above score points required for joining*/
	create table Si_table as 
	select score, sum(No_Of_ACC_Actual )as Total_Accounts, sum(No_Of_Bads ) as TN,
		              sum(No_Of_ACC_Actual )- sum(No_Of_Bads ) as FP
	from temp1 group by score;


	/*creating the temporary tables and  taking values below score */
	/*------- For creating Second Row of the table for each cut-off ------------------------
	Good|GGood	|GBad 	|Goods  |
	-------------------	|-------|
	*/
	create table temp2 as
	select No_Of_ACC_Actual , No_Of_Bads,Score_Points,score
	from input,temp_score
	where Score_Points > score;

	/*creating the tables for below score points required for joining*/
	create table Si_table1 as
	select score, sum(No_Of_ACC_Actual )as Total_Accounts, sum(No_Of_Bads ) as FN,
	              sum(No_Of_ACC_Actual )- sum(No_Of_Bads ) as TP
	from temp2 group by score;


	/*For combining first and second row of confusion matrix use join*/
	create table conf_mat as 
	select si_table.score,Si_table1.FN,Si_table1.TP,Si_table.TN,Si_table.FP
	from Si_table left outer join Si_table1
	on si_table1.score= si_table.score;

	/*droping temporary tables which not used further*/	
	drop table temp1;
	drop table temp2;
	drop table temp_score;
	drop table si_table;
	drop table si_table1;

	/*updating the missing values */
	UPDATE conf_mat SET FN=0 WHERE FN is missing; 
	UPDATE conf_mat SET TP=0 WHERE TP is missing;
	UPDATE conf_mat SET FP=0 WHERE FP is missing; 
	UPDATE conf_mat SET TN=0 WHERE TN is missing;
quit;

data &output_ds; 
merge conf_mat input(keep= PD);
run;

/*
proc printto;
run;
*/
%mend cs_rep_data_prep;

