/*
Macro Name	: roc_stat
Function	: Generate data for ROC curve and computes AUC (Area under curve) for the actual model.
Created by	: BIS Team
Date		: Apr 07, 2006
SAS Version	:  9.1

Called-by	:AUC screen for PD 
Calls		:None

Parameters	: 

input_ds		: Name of the input data set (Input data set is the Confusion matrix with each cut-off)
output_graph_ds	: Name of the data set which stores data generated for CAP curve.
output_stat_ds	: Name of the data set which stores value of AR statistic
 
Input Dataset(s) Format:
-------------------------------------------------------------------------
Name	Type		Format
------------------------------------------------------------------------
score	Numeric 	Default Format			
FN		Numeric 	Default Format
TP		Numeric 	Default Format
TN		Numeric 	Default Format
FP		Numeric 	Default Format
PD		Numeric 	Default Format
------------------------------------------------------------------------

Output Dataset(s) Format:
output_graph_ds
-------------------------------------------------------------------------
Name				Type		Format
------------------------------------------------------------------------
score				Numeric 	Default Format			
FN					Numeric 	Default Format
TP					Numeric 	Default Format
TN					Numeric 	Default Format
FP					Numeric 	Default Format
PD					Numeric 	Default Format
Dist_good			Numeric		Default Formatting
Dist_bad			Numeric		Default Formatting
ks					Numeric		Default Formatting
Dist_all			Numeric		Default Formatting
xdecile				Numeric		Default Formatting
Accurancy			Numeric		Default Formatting
Error_rate			Numeric		Default Formatting
Sensitivity			Numeric		Default Formatting
Specificity			Numeric		Default Formatting
Specificity1		Numeric		Default Formatting
Pietra				Numeric		Default Formatting
Depth				Numeric		Default Formatting
BER					Numeric		Default Formatting
last_specificity1	Numeric		Default Formatting
roc					Numeric		Default Formatting
------------------------------------------------------------------------
output_stat_ds
-------------------------------------------------------------------------
Name	Type		Format
------------------------------------------------------------------------
_Name_	Character	Default Formatting
Value* 	Numeric		Default Formatting
------------------------------------------------------------------------
Logic:
ROC curve is calculated as follows

1. Input is confusion matrix for each cut-off at the pool level.
2. Using this input compute sensetivity and specificity using following formulae
	Sensitivity = True Negatives/Total Negatives
    Specificity = True Positives/Total Positives
3. Calculate Complement of Specificity as Specificity1 = 1-Specificity 
4. With Specificity1 on X axis and Sensetivity on Y axis draw ROC chart

value of AUC is calculated as follows
1. Use of trpizoidal rule to calculate Area under the curve.
2. Use proc expand for this purpose.
3. Formula is as follows Sum ( Sensetivity_i *(Specificity1_i - Specificity1_i-1))

	
Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_graph_ds,output_stat_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
%macro roc_stat(input_ds=,output_graph_ds=,output_stat_ds=);
%let err_code=;

 

data temp;
set &input_ds;
var_tot_acc = TP+FP+TN+FN;
var_no_bads = FN+TN;
run;

/*CS4.3 HF2 */
/*PD of portfoilio is caculated and used to caculated BER*/
proc sql noprint;
  Select (sum(var_no_bads)/sum(var_tot_acc)) into: port_pd  from temp;
quit;

/***************************************************************/
%check_alldefaults_nodefaults(temp,var_tot_acc,var_no_bads, err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/***************************************************************/
data CS_ROC_Reports;
	set &input_ds;

    Dist_good   = 1.0 -(tp/(tp + fp));
    Dist_bad 	=tn/(tn + fn);
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_ks.1, noquote))= Dist_bad  - Dist_good ;

    Dist_all=(tn+fn)/(tp + fp + tn + fn);
	xdecile = 1.0 - Dist_all;  

    Accurancy    =(TP+TN)/(tp + fp + tn + fn);
    %sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_error_rate.1, noquote))=1.0 -(TP+TN)/(tp + fp + tn + fn);

    Sensitivity  = TN/(FN+TN);
    Specificity  = TP/(TP+FP);
    Specificity1 = 1.0 - TP/(TP+FP);
	%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_pietra.1, noquote))= Sensitivity - Specificity1; /* Changed as per discussions with Bharat--- Ajit*/
	/*CS4.3 HF2 */
    Depth =(TN+FP)/(tp + fp + tn + fn);
	%sysfunc(sasmsg(smd_ds.bismsg,CRS.MEASURE_NM.MEASURE_NM_BASIAN_ERROR_RATE.1, noquote))= &port_pd*(1-Sensitivity)+ (1-&port_pd)*Specificity1;
run;
 
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
proc expand data=CS_ROC_Reports out=CS_ROC_Reports; 
      id score;
	  convert specificity1 = last_specificity1 / transform=( lead 1 );
	   convert Sensitivity = last_Sensitivity / transform=( lead 1 );
run;

 
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
data &output_graph_ds;
	retain  
			roc 0 
			areaTr 0;	
	set CS_ROC_Reports;
	roc + ((sensitivity)*(last_specificity1-specificity1));
	areaTr+(last_Sensitivity-Sensitivity)*(last_specificity1-specificity1)*0.5;
	if _N_ =1 then
	area1 = (Sensitivity*specificity1)/2;
run; 
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
/*CS4.3 HF2 */
/*Corrected basion erorr rate.*/
proc sql noprint ;
	create table Stats as select 
	(sqrt(2)/4.0)*max(%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_pietra.1, noquote))) 
    as %sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_pietra.1, noquote)) , 
	min(%sysfunc(sasmsg(smd_ds.bismsg,CRS.MEASURE_NM.MEASURE_NM_BASIAN_ERROR_RATE.1, noquote))) 
	as %sysfunc(sasmsg(smd_ds.bismsg,CRS.MEASURE_NM.MEASURE_NM_BASIAN_ERROR_RATE.1, noquote)), 
	max(%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_roc.1, noquote)))+max(areaTr)+max(area1) 
    as %sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_auc.1, noquote)) 
	from &output_graph_ds;
	/*select AUC into :AUC from stats;*/
quit;
/*****************************************************************/
%errorhandler(err_code);
%if (&err_code ne 'NONE')%then %goto Error_Handler;
/*****************************************************************/
proc Transpose data = stats out = &output_stat_ds;
run; 


%return;
/*****************************************************************/
%Error_Handler:
%put ERROR: &err_code;

%defdata(input_ds=&input_ds,stat='ROC',
		out_graph_ds =&output_graph_ds,output_ds = ,out_stat_ds=&output_stat_ds);

%mend roc_stat;

