/******************************************************************************************************
       Module:  draw_table

     Function:  This stored process creates the a table based on the dataset passed into the macro 
				and the selected columns. If labels are passed into the macro, then they are used. 
				Otherwise, the labels are picked up from the input dataset.

      Authors:  BIS Team
         Date:  05 April 2006
          SAS:  9.1
    Called-by:	All reports which need a table on the screen
        Calls:  NA

Datasets used:  Supplied as the parameter
                The input dataset may have any structure. Only the columns passed into the macro
				should exist in the input dataset and these columns will be shown on screen.
                
         Logic: 1. Copy the metadata of the input dataset into the work folder.
				2. From this table, get the column types for the selected columns into a delimited string
				3. If the value of the input variable col_labels is empty or null, then get the 
					column labels from the copied table for selected column names into another delimited
					string.
				4. Display the table on screen using the column labels as the table headers and the
					data from the selected columns as the data in the table.
				
   Parameters:  INPUT (From Report Screen when link is used for navigation) 
				ds			:   Input dataset to be considered when displaying on screen.
                col_names	:   Names of the columns whose data need to be displayed on screen.
                col_labels	:   Names of the columns labels which should be considered when drawing
								the table on screen. If this value is empty, then the labels will be 
								picked up from the input dataset for the selected columns.

	NOTE: The HTML and BODY opening and closing tags and any other required tags like the form tags, 
		should be part of the calling program. This macro only starts from the TABLE tag and closes 
		the TABLE tag before ending the macro.

******************************************************************************************************/

%macro draw_table(ds=,col_names=,col_labels=);
/* CS4.3 HF2 */
options MVARSIZE=1000;
%let lib_nm=%upcase(%scan(&ds,1,"."));
%let ds_nm=%upcase(%scan(&ds,2,"."));

/* Copy the metadata of the input dataset into the work folder. */
proc sql;
create table work.tab_metadata as
select libname, memname, name, label, type 
from sashelp.vcolumn 
where libname="&lib_nm" and memname="&ds_nm";
quit;



/* From this table, get the column types for the selected columns into a delimited string.
If the value of the input variable col_labels is empty or null, then get the column labels from 
the copied table for selected column names into another delimited string. */
%let col_types=%str();
%let col_label=%str();
%let cnt=1;
%let col_type=%str();
%let col_label=%str();
%if &col_labels=%str() %then %let lbl=Y;
%else %let lbl=N;
%let token=%qscan(&col_names,&cnt);
%do %while(&token ne %str());
	%if &lbl=%str(Y) %then %do;
		proc sql noprint;
			select type, label into :col_type, :col_label
			from work.tab_metadata 
			where libname="&lib_nm" and memname="&ds_nm" and upcase(name)=upcase("&token");
		quit;
		/* CS4.3 HF2 */
		/*Removal of the %trim and %left as %let by default removes leading and trailing blanks*/
/*		%let col_label=%nrquote(%trim(%left(&col_label)));*/
		%let col_label=%nrquote(&col_label);
		%if &col_labels=%str() %then %let col_labels=&col_label;
		/* CS4.3 HF2 */
		/*Commented the one with the comma seperator and added a line with # as a seperator*/
/*		%else %let col_labels=&col_labels,&col_label;*/
		%else %let col_labels=&col_labels#&col_label;
		%let col_label=%str();
	%end;
	%else %do;
		proc sql noprint;
			select type into :col_type
			from work.tab_metadata 
			where libname="&lib_nm" and memname="&ds_nm" and upcase(name)=upcase("&token");
		quit;
	%end;
	/* CS4.3 HF2 */
	/*Removal of the %trim and %left as %let by default removes leading and trailing blanks*/
/*	%let col_type=%nrquote(%trim(%left(&col_type)));*/
	%let col_type=%nrquote(&col_type);
	%if &col_types=%str() %then %let col_types=&col_type;
	%else %let col_types=&col_types,&col_type;
	%let col_type=%str();
	%let col_label=%str();
	%let cnt=%eval(&cnt+1);
	%let token=%qscan(&col_names,&cnt);
%end;
%let no_of_cols=%eval(&cnt-1);
%let col_types=%nrquote(&col_types);
%let col_labels=%nrquote(&col_labels);
%put A=&col_types;
%put B=&col_labels;

/* Start display of the table on screen */
data _null_;
/*I18N-scnhuh-20080422-1655*/

	file _webout &webout_options;
/*I18N-scnhuh-20080422-1655*/
	put '<table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#FFFFFF" class="tableborder"> ';
	put '<tr> ';
	put '<td valign="top"><table width="100%" border="0" cellpadding="3" cellspacing="0"> ';
	put '<tr> ';
	/* Display column labels as table headers */
	do cnt=1 to &no_of_cols;
		/* Start of CS4.3 HF2 */
		length token1 $1000.;
/*		token1=scan("&col_labels",cnt,",");*/
		if "&lbl" eq "Y" then token1=kscan("&col_labels",cnt,"#");
		else token1=kscan("&col_labels",cnt,",");
		/* End of CS4.3 HF2 */
		token2=kscan("&col_types",cnt,",");
		if token2="num" then put '<td align="right"' @;
		else if token2="char" then put '<td align="left"' @;
		if cnt ne &no_of_cols then put ' class="cornerborderTop">' @;
		else put ' class="HrzBorderTop">' @;
		put token1 @;
		put '</td> ';
	end;
	put '</tr> ';
run;

/* Display the data in the selected columns in the input table as the data on screen */
data _null_;
/*I18N-scnhuh-20080422-1655*/

	file _webout &webout_options;
/*I18N-scnhuh-20080422-1655*/
	set &ds end=last;
	put '<tr> ';
%do cnt=1 %to &no_of_cols;
	%let token1=%qscan(&col_names,&cnt);
	%let token2=%qscan(&col_types,&cnt);
		if "&token2" eq "num" then put '<td align="right"' @;
		else if "&token2" eq "char" then put '<td align="left"' @;
		if last then do;
			if &cnt ne &no_of_cols then put ' class="VrtBorder">' @;
			else put ' class="Font11px">' @;
		end;
		else do;
			if &cnt ne &no_of_cols then put ' class="cornerborder">' @;
			else put ' class="HrzBorder">' @;
		end;
		put &token1 @;
		put '</td> ';
%end;
	put '</tr> ';
run;

/* Close table tag */
data _null_;
/*I18N-scnhuh-20080422-1655*/

	file _webout &webout_options;
/*I18N-scnhuh-20080422-1655*/
	put '</table></td> ';
	put '</tr> ';
	put '</table> ';
run;

%mend draw_table;
