/******************************************************************************************************
       Module:  build_all_abt

     Function:  Builds and scores all the ABTs listed in the dataset Build_abt_status.

      Authors:  BIS Team
         Date:  14 May 2007
          SAS:  9.1
    Called-by:	stored process build_pool_abt
        Calls:  macro get_model_vars_code
				macro build_sel_abt

Datasets used:  pool.build_abt_status
 
        Logic:  1. Retrieves the model code and variables to be built from the metadata
					by passing the model information to the macro get_model_vars_code
				2. Loops through the list of ABTs to be built and builds each 
					by passing variables information to the macro build_sel_abt
				3. Scores every ABT which has been built successfully.
				4. If an ABt has been built and scored successfully, set status as Complete else
					Failed in build_abt_status
				
   Parameters:  repository	: The metadata repository to get model information from
				scheme_no	: The scheme identifier for which the ABTs have to be built
				in_ds		: The dataset containing the list of ABTs to be built
				work_lib	: The library which should be used for temporary calculations
				action		: B - Build Data
							  R - Restart from falied ABT

******************************************************************************************************/
%macro build_all_abt(repository=, scheme_no=, in_ds=, work_lib=, action=);
options mlogic mprint;
options obs=max no$syntaxcheck;
/*In few macro the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
	/* Handle missing values */
	%if %str(&repository) = %str() %then %do;
		%put WARNING: Metadata Repository should be provided. Defaulting to Foundation Repository.;
		%let repository = Foundation;
	%end;

	%if %str(&scheme_no) = %str() %then %do;
		%put ERROR: The pool scheme key is not provided. Exiting;
		%abort;
	%end;

	%if %str(&in_ds) = %str() %then %do;
		%put ERROR: The dataset containing the list of ABTs to be built and scored is not provided. Exiting;
		%abort;
	%end;

	%if %str(&work_lib) = %str() %then %do;
		%put WARNING: Work library not provided. Defaulting to work.;
		%let work_lib = work;
	%end;

	%if %sysfunc(exist(&in_ds.)) eq 0 %then %do;
		%put ERROR: The dataset containing the list of ABTs to be built and scored is not available. Exiting;
		%abort;
	%end;

	%if %str(&action) = %str() %then %do;
		%put ERROR: The action to be performed is not available. Defaulting to B - Building ABT;
		%let action = B;
	%end;

	proc sort data=&in_ds. out=&in_ds.;
		format abt_dt date9.;
		by pool_scheme_no model_key;
	quit;

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	/* Get the variables to be built for the ABT and the model code */
	%if %str(&action) eq %str(B) %then %do;
		/* If action = B, then build all ABTs in &in_ds. */
		proc sql noprint;
			select count(distinct model_key), count(*) into :unq_model_cnt, :model_cnt 
			from &in_ds. 
			where pool_scheme_no=&scheme_no.;

			%let unq_model_cnt=%trim(%left(&unq_model_cnt));
			%let model_cnt=%trim(%left(&model_cnt));
			%if &unq_model_cnt > 1 %then %do;
				select distinct model_key, model_nm 
					into :mdl1-:mdl&unq_model_cnt., :nm1-:nm&unq_model_cnt.
				from &in_ds. 
				where pool_scheme_no=&scheme_no.;
			%end;
			%else %do;
				select distinct model_key, model_nm into :mdl1, :nm1
				from &in_ds. 
				where pool_scheme_no=&scheme_no.;
			%end;
		quit;

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	%end;
	%if %str(&action) eq %str(R) %then %do;
		/* If action = R, then build only those ABTs in &in_ds. which do not have status as Complete. */
		proc sql noprint;
			select count(distinct model_key), count(*) into :unq_model_cnt, :model_cnt 
			from &in_ds. 
					 /*I18N-scnjih-20080430-1315*/
			/*where pool_scheme_no=&scheme_no. and status_desc ne 'Complete';*/
				where pool_scheme_no=&scheme_no. and status_desc ne  "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
					 /*I18N-scnjih-20080430-1315*/

			%let unq_model_cnt=%trim(%left(&unq_model_cnt));
			%let model_cnt=%trim(%left(&model_cnt));
			%if &unq_model_cnt > 1 %then %do;
				select distinct model_key, model_nm 
					into :mdl1-:mdl&unq_model_cnt., :nm1-:nm&unq_model_cnt.
				from &in_ds. 
				 /*I18N-scnjih-20080430-1315*/
			/*where pool_scheme_no=&scheme_no. and status_desc ne 'Complete';*/
				where pool_scheme_no=&scheme_no. and status_desc ne  "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
				/*I18N-scnjih-20080430-1315*/
				
			%end;
			%else %do;
				select distinct model_key, model_nm into :mdl1, :nm1
				from &in_ds. 
					 /*I18N-scnjih-20080430-1315*/
			/*where pool_scheme_no=&scheme_no. and status_desc ne 'Complete';*/
				where pool_scheme_no=&scheme_no. and status_desc ne  "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
					 /*I18N-scnjih-20080430-1315*/
			%end;
		quit;

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	%end;

	/* Determine the variables to build from the metadata of the model. 
		Consider the input variables for the model */
	%do model_no=1 %to &unq_model_cnt;
		%let mdl_no=&&mdl&model_no.;
		%let mdl_nm=%nrquote(%'%trim(%left(&&nm&model_no.))%');
		%let out_file=Data/bisdata/scratch/abt/_&mdl_no._code.sas;
		%let cols_ds=abtscr._&mdl_no._columns;

		%if %str(&action) eq %str(B) %then %do;
			proc sql;
				update &in_ds. 
				set cols_ds="&cols_ds", model_code="&out_file"
				where pool_scheme_no=&scheme_no. and model_key=&mdl_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
		%else %if %str(&action) eq %str(R) %then %do;
			proc sql;
				update &in_ds. 
				set cols_ds="&cols_ds", model_code="&out_file"
				/*I18N-scnjih-20080430-1315*/
				/*where pool_scheme_no=&scheme_no. and model_key=&mdl_no and status_desc ne 'Complete';*/
					where pool_scheme_no=&scheme_no. and model_key=&mdl_no and status_desc ne "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
				/*I18N-scnjih-20080430-1315*/
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;

		/* Builds and scores all the ABTs listed in the dataset Build_abt_status.
		   Parameters:  repository	: The metadata repository to get model information from
						model_nm	: The name of the model for which information is to be retrieved
						model_rk	: The identifier of the model
						work_lib	: The library which should be used for temporary calculations
						code_file	: The path and file in which the model code retrieved from the metadata
										is to be stored
						out_cols_ds	: The dataset in which the required variables for the model 
										retrieved from the metadata is to be stored
		*/
		%get_model_vars_code(repository=&repository, model_nm=&mdl_nm, model_rk=&mdl_no, 
			work_lib=&work_lib, code_file="&out_file", out_cols_ds=&cols_ds);
	%end;

	/* Build & score the ABTs */
	%if %str(&action) eq %str(B) %then %do;
		/* If action = B, then build all ABTs in &in_ds. */
		%if &model_cnt > 1 %then %do;
			proc sql noprint;
				select abt_no, model_key, abt_phy_ds, abt_dt ,scr_abt_phy_ds, cols_ds, model_code
					into :abt1-:abt&model_cnt., :mdl1-:mdl&model_cnt., 
					:phy1-:phy&model_cnt., :dt1-:dt&model_cnt., :scr_phy1-:scr_phy&model_cnt.,
					:cols_ds1-:cols_ds&model_cnt., :m_code1-:m_code&model_cnt.
				from &in_ds. 
				where pool_scheme_no=&scheme_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
		%else %do;
			proc sql noprint;
				select abt_no, model_key, abt_phy_ds, abt_dt ,scr_abt_phy_ds, cols_ds, model_code
					into :abt1, :mdl1, :phy1, :dt1, :scr_phy1, :cols_ds1, :m_code1
				from &in_ds. 
				where pool_scheme_no=&scheme_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
	%end;
	%if %str(&action) eq %str(R) %then %do;
		/* If action = R, then build only those ABTs in &in_ds. which do not have status as Complete. */
		%if &model_cnt > 1 %then %do;
			proc sql noprint;
				select abt_no, model_key, abt_phy_ds, abt_dt ,scr_abt_phy_ds, cols_ds, model_code,
					model_type_cd, method_type_cd
					into :abt1-:abt&model_cnt., :mdl1-:mdl&model_cnt., 
					:phy1-:phy&model_cnt., :dt1-:dt&model_cnt., :scr_phy1-:scr_phy&model_cnt.,
					:cols_ds1-:cols_ds&model_cnt., :m_code1-:m_code&model_cnt., 
					:m_type_code1-:m_type_code&model_cnt., :method_code1-:method_code&model_cnt.
				from &in_ds. 
				/*I18N-scnjih-20080430-1315*/
				/*where pool_scheme_no=&scheme_no. and status_desc ne 'Complete';*/
					where pool_scheme_no=&scheme_no. and status_desc ne "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
				/*I18N-scnjih-20080430-1315*/
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
		%else %do;
			proc sql noprint;
				select abt_no, model_key, abt_phy_ds, abt_dt ,scr_abt_phy_ds, cols_ds, model_code,
					model_type_cd, method_type_cd
					into :abt1, :mdl1, :phy1, :dt1, :scr_phy1, :cols_ds1, :m_code1, 
					:m_type_code1, :method_code1
				from &in_ds. 				
				/*I18N-scnjih-20080430-1315*/
				/*where pool_scheme_no=&scheme_no. and status_desc ne 'Complete';*/
					where pool_scheme_no=&scheme_no. and status_desc ne "%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))";
				/*I18N-scnjih-20080430-1315*/
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
	%end;

	%do model_no=1 %to &model_cnt;
		%let abt_no=&&abt&model_no.;
		%let mdl_no=&&mdl&model_no.;
		%let phy_ds=&&phy&model_no.;
		%let abt_dt=&&dt&model_no.;
		%let scr_phy_ds=&&scr_phy&model_no.;
		%let cols_ds=&&cols_ds&model_no.;
		%let out_file=&&m_code&model_no.;

		%if %str(&action) eq %str(R) %then %do;
			%let mod_type=&&m_type_code&model_no.;
			%let method_type=&&method_code&model_no.;

			%if %str(&method_type) = %str(SCR) %then %do;
				%if %str(&mod_type) = %str(PD) %then %let create_scr_pd=Y;
				%if %str(&mod_type) = %str(LGD) %then %let create_scr_lgd=Y;
				%if %str(&mod_type) = %str(CCF) %then %let create_scr_ccf=Y;
			%end;
			%if %str(&method_type) = %str(LNG) %then %do;
				%if %str(&mod_type) = %str(PD) %then %let create_lng_pd=Y;
				%if %str(&mod_type) = %str(LGD) %then %let create_lng_lgd=Y;
				%if %str(&mod_type) = %str(CCF) %then %let create_lng_ccf=Y;
			%end;
		%end;

		/* Build the ABT */
		%let error_code=;
		%let col_ds=&work_lib..&mdl_no._columns;

		/* Builds the selected ABT
		   Parameters:  cols_ds		: The dataset with the list of variables to be built for the ABT
						model_key	: The model identifier for the model to name the physical dataset
						obs_dt		: The date of data capture for the ABT
						work_lib	: The library which should be used for temporary calculations
						phy_nm		: The physical name of the ABT to be built
		*/
		%build_sel_abt(cols_ds=&cols_ds., model_key=&mdl_no., 
			obs_dt=&abt_dt., work_lib=&work_lib., phy_nm=&phy_ds.);

		/* Score the ABT */
		data &scr_phy_ds.;
			set &phy_ds.;
			%include "&out_file."; 
		run;

		%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

		%if %str(error_code) eq %str(ERROR) or &syserr. > 4 %then %do;	
			proc sql;
				/*I18N-scnjih-20080430-1315*/
				/*update pool.build_abt_status set status_desc='Failed' */
					update pool.build_abt_status set status_desc="%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.2, noquote))"
				/*I18N-scnjih-20080430-1315*/
					where pool_scheme_no=&scheme_no and abt_no=&abt_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
		%else %do;
			proc sql;
			
					/*I18N-scnjih-20080430-1315*/
				/*update pool.build_abt_status set status_desc='Complete' */
					update pool.build_abt_status set status_desc="%sysfunc(sasmsg(smd_ds.bismsg,build_all_abt.sas.1, noquote))"
				/*I18N-scnjih-20080430-1315*/
				
				where pool_scheme_no=&scheme_no and abt_no=&abt_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
	%end;
proc printto ;
run;
options nomlogic nomprint;
%mend build_all_abt;
