/******************************************************************************************************
       Module:  build_sel_abt

     Function:  Builds the selected ABT

      Authors:  BIS Team
         Date:  14 May 2007
          SAS:  9.1
    Called-by:	stored process build_all_abt
        Calls:  macro create_pool_abt
				deployed code MasterLoopPoolingABTJob

Datasets used:  control.parameter_list_pool_abt
 
        Logic:  1. Get the aggregate tables to be built to get the required variables.
				2. Get the list of jobs to be run to build the aggregate tables
				3. Get the order in which the jobs should be run.
				4. Load this list of jobs into the dataset control.parameter_list_pool_abt
				5. Set the date of data capture to be the value of obs_dt
				6. Call the deployed job MasterLoopPoolingABTJob which in turn will
					call the jobs in control.parameter_list_pool_abt
				7. If all jobs are successful, call the macro create_pool_abt to build the ABT
				
   Parameters:  cols_ds		: The dataset with the list of variables to be built for the ABT
				model_key	: The model identifier for the model to name the physical dataset
				obs_dt		: The date of data capture for the ABT
				work_lib	: The library which should be used for temporary calculations
				phy_nm		: The physical name of the ABT to be built

******************************************************************************************************/
%macro build_sel_abt(cols_ds=, model_key=, obs_dt=, work_lib=, phy_nm=);
	/* Handle missing values */
/*In few macro the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
	%if %str(&cols_ds) = %str() %then %do;
		%put ERROR: The list of columns required for the ABT is not available. Exiting;
		%abort;
	%end;

	%if %str(&model_key) = %str() %then %do;
		%put ERROR: MODEL Key for the model to be scored is not available. Exiting;
		%abort;
	%end;

	%if %str(&obs_dt) = %str() %then %do;
		%put ERROR: Date as on which to build the ABT is not available. Exiting;
		%abort;
	%end;

	%if %str(&work_lib) = %str() %then %do;
		%put WARNING: Work library not provided. Defaulting to work.;
		%let work_lib = work;
	%end;

	%if %str(&phy_nm) = %str() %then %do;
		%put WARNING: Physical name for the ABT not provided. Defaulting to _<model_key>.;
		%let phy_nm = _&model_key.;
	%end;

	%if %sysfunc(exist(control.parameter_list_pool_abt)) %then %do;
		/* Backup the existing Parameter List table as _&abt_key._<date>_<time> and 
			create a new one for use */
		%let tb_nm=%qscan("&phy_nm", 2, ".");
		%global backup_table_name;
		%let backup_table_name=;
		data _NULL_;
/*			name=put(date(), NLDATE10.)||"_"||strip(tranwrd(put(time(), time6.), ":", "_"));*/
			name=put(date(), date9.)||"_"||strip(tranwrd(put(time(), time6.), ":", "_"));
			call symput('backup_table_name', "_param"||strip("&tb_nm")||"_"||trim(left(name)));
		run;

		%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

		data control.&backup_table_name.;
			set control.parameter_list_pool_abt;
			output control.&backup_table_name.;
		run;

		%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

		proc sql;
			drop table control.parameter_list_pool_abt;
		quit;

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	%end;

	data control.parameter_list_pool_abt;
		length JOB_NM $40.;
		length LEVEL_NO 8.;
		length RETURN_CD 8.;
		stop;
	run;

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	/* Determine the aggregate tables to build and
	   determine the jobs to build */
	proc sql;
		create table &work_lib..abt_var as
		select columnname as var_nm 
		from &cols_ds;

		/* Manual insertion of key variables which are required to build the ABT */
		insert into &work_lib..abt_var
		(VAR_NM) 
		values ('CUSTOMER_RK')
		values ('ACCOUNT_RK')
		values ('APPLICATION_RK')

/*Change for product filter*/
		values ('ACCOUNT_TYPE_CD')
		values ('ACC_PRD_TYP_CD')
		;	
		/* End of manual insertion */

		create table &work_lib..job_levels as
		select distinct Table_x_job.job_nm, job_level_no as level_no, . as return_cd,
			child_jobs
		from &work_lib..abt_var inner join analytic.Var_master
		on (Abt_var.var_nm = Var_master.var_nm)
		inner join analytic.Table_x_job
		on (Var_master.agg_table_nm = Table_x_job.var_table_nm)
		inner join analytic.Job_x_level
		on (Table_x_job.job_nm = Job_x_level.job_nm)
		order by job_level_no;
	quit;

	%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

	%global child_jobs;
	%let child_jobs='';
	proc sql;
		select distinct child_jobs into :child_jobs separated by ',' 
		from &work_lib..job_levels 
		where child_jobs NE '';

		delete from &work_lib..job_levels where job_nm in (&child_jobs.);

		alter table &work_lib..job_levels drop child_jobs;
	quit;

	%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

	proc append data=&work_lib..job_levels base=control.parameter_list_pool_abt force;	
	quit;

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	/* Set the data capture date in macro variable LOAD_ABT_USER_INPUT_DTTM
	1. Only global definition of LOAD_ABT_USER_INPUT_DTTM will be available in autoexec.
	2. Pass this macro thro the loop jobs to the final ETL jobs.
	3. When the autoexec for the ETL jobs run, the other variables dependent on 
		LOAD_ABT_USER_INPUT_DTTM will get set.
	*/
	data _null_;
		ld_dt="&obs_dt"d;
		ld_dttm=dhms(ld_dt, 0,0,0);
		call symput('LOAD_ABT_USER_INPUT_DTTM', ld_dttm);
		call symput('LOAD_ABT_USER_INPUT_DATE', ld_dt); /* CS4.3 HF2 */
	run;

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	/* Call the MasterLoopJob to run the jobs to build the Modeling ABT */
	%include jobs(MasterLoopPoolingABTJob.sas);

	/************** End of Build the required aggregate tables **************/

	proc printto log="&log_name";
	quit;

	/************** Create the ABT **************/

	proc sql;
		create table &work_lib..var_list
		as select distinct a.v_no, a.var_nm, a.agg_table_nm, a.agg_var_nm, a.agg_table_key 
		from analytic.var_master a inner join &cols_ds b
		on (a.var_nm = b.columnname);
	quit;

	%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	
	/* Call macro to create the abt 
		   Parameters:  abt_type		: SCR for a scoring ABT
									  		MOD for a modeling ABT
						abt_cat			: Account for account level ABT
									 		Customer for customer level ABT
									  		Application for application level ABT
						abt_phy_table	: Name of the physical table to be created
						abt_vars		: Dataset with the list of variables to be created in the ABT
	*/

	%let abt_pool_cat=Account;/*Account for account level ABT
								Customer for customer level ABT
								Application for application level ABT*/

/*	%create_pool_abt(abt_type=SCR, abt_cat=Account, abt_phy_table=&phy_nm, abt_vars=&work_lib..var_list);*/
	%create_pool_abt(abt_type=SCR, abt_cat=&abt_pool_cat., abt_phy_table=&phy_nm, abt_vars=&work_lib..var_list);

/*Filter for a product type for Account or Application level ABT.*/
%if %str(&abt_pool_cat) eq %str(Account) %then %do;
		proc sql noprint;
	      create table &phy_nm as select  * from &phy_nm where 
		       ACCOUNT_TYPE_CD in (select MODEL_PRODUCT_TYPE_CD 
									from dim.Analytical_model_dim
									where MODEL_RK=&model_key.);
		quit;
%end;
%else %if %str(&abt_pool_cat) eq %str(Application) %then %do;
		proc sql noprint;
	      create table &phy_nm as select  * from &phy_nm where 
		       ACC_PRD_TYP_CD in (select MODEL_PRODUCT_TYPE_CD 
									from dim.Analytical_model_dim
									where MODEL_RK=&model_key.);
		quit;
%end;
/* */
 

	/************** End of Create the ABT **************/
%mend build_sel_abt;
