/********************************************************************************************
       Module:  test_levels (Measure level Dashboard Screen for model monitoring reports)

     Function:  This stored process creates the HTML screen which is used to view the
				all the statistics for the MODELS selected in the pdmodels stored process

				In this the user can select a statistic from one of the two MODELS
				selected, by clicking the link provided on the traffic lighting,
				for generating the respective model monitoring report screen.
				 
      Authors:  BIS Team
         Date:  05 April 2006
          SAS:  9.1
    Called-by:	1. select_models stored process
				2. From any model monitoring report screen as part of the links provided on
					the page for navigation.
					
        Calls:  Calls the following Stored Process
				1. Any of the reports	(when a traffic lighting for that particular
											report is clicked)
				2. Reporting  			(when Reporting link is clicked)
				3. mon_models     		(when Monitoring Models link is clicked)
				4. select_models		(when Select Models link is clicked)

				Calls the following Macros
				1. model_traffic macro for generating the traffic lighting for all the
					measures.

Datasets used:  No input datasets required for processing of the stored process
 
        Logic:  1. All inputs to the stored process are accepted as global parameters. 
				2. Setting default value for mdl_dep (Deployment Status Code).
				3. Access the Star Schema to get the required data for the selected product
					and model type.
				4. Call the model_traffic macro for generating the traffic lighting
					for the given measure.
				5. Create a dataset containing data for both the models selected with all
					the measures and the required information(measure_value, traffic lighting,
					level_desc, etc.)
				6. Generate the HTML using streaming webout.
					a. Provide hidden textboxes for all output parameters so that the values
						are easily submitted to the next page. 
					b. Provide javascript functions for the links on top of the page
					c. Provide javascript function for passing the model(model for which
						the measure is selected) as model1 and the other model as model2
						to the respective model monitoring report screen called.
				
   Parameters: 
			INPUT 
				prod_type_desc		:   Product Type Description (e.g., Credit Card,
										Mortgage, Loan, Core)
		        prod_type_code		:   Product Type Code (e.g., CCD, MTG, LON, COR)
				model_type_desc		:	Analytical Model Type Description 
										(e.g., PD Approvals, PD Ongoing, LGD, CCF)
		        model_type_code   	:   Analytical Model Type Code (e.g., PD Approvals,
										PD Ongoing, LGD, CCF)
		        include_exp_mdl		:   Information to include the expired models for 
										selection of model from the traffic light dashboard.
				model1				:	The first model if selected (Eg. PD MODEL 1).		
				model1_key			:	The surrogate key for the first selected model.
				model2				:	The  second model if  selected. (Eg. PD MODEL 2)
				model2_key			:	The surrogate key for the second selected model.
				sel_model			: 	Model selected on the Measure level dashboard screen.
				sel_model_key		:	Key of the selected model.
				period				:	The selected period (month and year)(Eg DEC05 JAN06).
				time_key			:	The surrogate key for selected period.
				
			OUTPUT
				prod_type_desc		:   Product Type Description (e.g., Credit Card,
										Mortgage, Loan, Core)
		        prod_type_code		:   Product Type Code (e.g., CCD, MTG, LON, COR)
				model_type_desc		:	Analytical Model Type Description 
										(e.g., PD Approvals, PD Ongoing, LGD, CCF)
		        model_type_code   	:   Analytical Model Type Code (e.g., PD Approvals,
										PD Ongoing, LGD, CCF)
		        include_exp_mdl		:   Information to include the expired models for 
										selection of model from the traffic light dashboard.
				model1				:	The first model if selected (Eg. PD MODEL 1).		
				model1_key			:	The surrogate key for the first selected model.
				model2				:	The  second model if  selected. (Eg. PD MODEL 2)
				model2_key			:	The surrogate key for the second selected model.
				sel_model			: 	Model selected on the Measure level dashboard screen.
				sel_model_key		:	Key of the selected model.
				period				:	The selected period (month and year)(Eg DEC05 JAN06).
				time_key			:	The surrogate key for selected period.
				statistic			:	The name of the measure selected.
				prg_name			:	The program name, for calling the report, which is
										picked from program_nm column of dim.measure_dim.
				outcome_period		:	The target period in months.
				elapsed_time		:	The time elapsed.

*********************************************************************************************/
/*This code is Internationalized*/
/* Global input parameters that are passed from the calling program*/
%global prod_type_desc prod_type_code model_type_desc model_type_code include_exp_mdl 
		model1 model1_key model2 model2_key sel_model sel_model_key period time_key format_name;
 
option mlogic mprint;
 
%macro check_macro_value;
/* Trimming extra spaces from all the input parameters*/
%if "&prod_type_desc" ne "" %then
	%let prod_type_desc=%str(%sysfunc(ktrim(%sysfunc(kleft(&prod_type_desc)))));
%if "&prod_type_code" ne "" %then
	%let prod_type_code=%str(%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code)))));
%if "&model_type_desc" ne "" %then
	%let model_type_desc=%str(%sysfunc(ktrim(%sysfunc(kleft(&model_type_desc)))));
%if "&model_type_code" ne "" %then
	%let model_type_code=%str(%sysfunc(ktrim(%sysfunc(kleft(&model_type_code)))));
%if "&include_exp_mdl" ne "" %then
	%let include_exp_mdl=%str(%sysfunc(ktrim(%sysfunc(kleft(&include_exp_mdl)))));
%if "&model1" ne "" %then
	%let model1=%str(%sysfunc(ktrim(%sysfunc(kleft(&model1)))));
%if "&model1_key" ne "" %then
	%let model1_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&model1_key)))));
%let model2=&model2;
%if "&model2" ne "" %then
	%let model2=%str(%sysfunc(ktrim(%sysfunc(kleft(&model2)))));
%if "&model2_key" ne "" %then
	%let model2_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&model2_key)))));
%if "&sel_model" ne "" %then
	%let sel_model=%str(%sysfunc(ktrim(%sysfunc(kleft(&sel_model)))));
%if "&sel_model_key" ne "" %then
	%let sel_model_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&sel_model_key)))));
%if "&period" ne "" %then
	%let period=%str(%sysfunc(ktrim(%sysfunc(kleft(&period)))));
%if "&time_key" ne "" %then
	%let time_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&time_key)))));
/*******************/
%mend;
%check_macro_value;
/*This macro outputs the format for the stattistics for combination of 
  product type code and the model type code.*/
%format_for_model_product(model_type_code_fm=&model_type_code,prod_type_code_fm=&prod_type_code)

/* If model1 or model2 is not selected then, a missing value is assigned to the respective
		model_key */
data _NULL_;
	temp=.;
	if "&model1_key"="" then
		call symput('model1_key',temp);
	if "&model2_key"="" then
		call symput('model2_key',temp);
run;

/* Setting value for mdl_dep (Deployment status code) depending whether the user has included
	expired models or not */
%let mdl_dep=;
data _NULL_;
	if kupcase("&include_exp_mdl")="Y" then
		call symput('mdl_dep','DEPLOYMENT_STATUS_CD in (&PRD_MODEL_DEPLOYMENT,&DEV_MODEL_DEPLOYMENT,&EXP_MODEL_DEPLOYMENT)');
	else
		call symput('mdl_dep','DEPLOYMENT_STATUS_CD in (&PRD_MODEL_DEPLOYMENT,&DEV_MODEL_DEPLOYMENT)');
run;
/*******************/

/* Subsetting the measure_dim by the model_type_code */
proc sql noprint;
	create table work.measure_rep as
	select * from dim.measure_dim
	where kupcase(level_desc) ne kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))")
		and kupcase(model_type_cd)=kupcase("&model_type_code")
	order by measure_nm;
quit;
/*******************/

/* Creating dataset containing all the required data depending on the product and model_type
	selected */
proc sql noprint;
	create table work.measure_out AS 
	SELECT measure_rep.MEASURE_NM, measure_rep.SEQ_NO, measure_rep.PROGRAM_NM,
		measure_rep.LEVEL_DESC, MODEL_MEASURE_FACT.SCORE_TIME_SK, 
		MODEL_MEASURE_FACT.MEASURE_SK, MODEL_MEASURE_FACT.MEASURE_VALUE, 
		MODEL_MEASURE_FACT.MODEL_SK, ANALYTICAL_MODEL_DIM.MODEL_NM,
		ANALYTICAL_MODEL_DIM.TARGET_PERIOD_MTHS_CNT
	FROM FACT.MODEL_MEASURE_FACT 	
		INNER JOIN work.measure_rep 
		ON (MODEL_MEASURE_FACT.MEASURE_SK = measure_rep.MEASURE_SK)  	
		INNER JOIN DIM.ANALYTICAL_MODEL_DIM 
		ON (MODEL_MEASURE_FACT.MODEL_SK = ANALYTICAL_MODEL_DIM.MODEL_SK)  
		INNER JOIN DIM.TIME_DIM 
		ON (MODEL_MEASURE_FACT.SCORE_TIME_SK = TIME_DIM.TIME_SK) 
		where kupcase(ANALYTICAL_MODEL_DIM.MODEL_PRODUCT_TYPE_CD)=kupcase("&prod_type_code") 
			and kupcase(MODEL_MEASURE_FACT.DEV_FLG)="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.dev_flag_n.1, noquote))" 
			and kupcase(ANALYTICAL_MODEL_DIM.MODEL_TYPE_CD)=kupcase("&model_type_code")
			and &mdl_dep; 
quit;
/*******************/

/* Calls the model_traffic macro to get the traffic lighting for the data provided
	The following parameters are passed to this macro
			1. data1	: input dataset containing the columns measure_value and measure_nm
			2. model_cd	: MODEL TYPE (PD Approvals, PD Ongoing, LGD, CCF)
	This macro adds one more column named color to the dataset passed which contains
	values ( Green, Red, Yellow, White) depending on the bounds(for red and green) provided
	in the measure_dim dataset */
%model_traffic(data1=work.measure_out,model_cd=&model_type_code,product_type_code=&prod_type_code);
/*******************/
/* Splitting the dataset created above into two dataset, depending on the model_keys */
data work.model1 work.model2;
	set work.measure_out;
	format Measure_value &CS_DEC_FMT.;
	if score_time_sk=&time_key;
	if model_sk=&model1_key then output work.model1;
	if model_sk=&model2_key then output work.model2;
run;
/*******************/

%let target_period1=.;
%let target_period2=.;
proc sql noprint;
	select target_period_mths_cnt into :target_period1
	from work.model1;

	select target_period_mths_cnt into :target_period2
	from work.model2;
quit;
%let target_period1=%sysfunc(ktrim(%sysfunc(kleft(&target_period1))));
%let target_period2=%sysfunc(ktrim(%sysfunc(kleft(&target_period2))));

%let elapsed_time1=.;
%let elapsed_time2=.;
data _null_;
	dt = input ("&period",&CS_TIME_PER_FMT.);
	diff =  intck('month',dt,today());
	if diff >= &target_period1 then
		call symput('elapsed_time1',&target_period1);
	else
		call symput('elapsed_time1',diff);
	if diff >= &target_period2 then
		call symput('elapsed_time2',&target_period2);
	else
		call symput('elapsed_time2',diff);
run;
%let elapsed_time1=%sysfunc(ktrim(%sysfunc(kleft(&elapsed_time1))));
%let elapsed_time2=%sysfunc(ktrim(%sysfunc(kleft(&elapsed_time2))));

proc sort data=work.model1 ;
	by Measure_nm;
run;
/*******************/

proc sort data=dim.measure_dim out=work.measure_dim ;
	by measure_nm;
run;
/*******************/

/* Creating dataset for the first model selected. For measures (except those having
	level_desc as 'Other Reports') which are not there for this model but are there
	 in the measure_dim, White color is assigned */
data work.model1_rep;
	merge work.model1(rename=(model_nm=model_nm1 model_sk=model_sk1 measure_value=measure_value1
							color=color1) in=inmod1)
		work.measure_dim(in=inms);
	by measure_nm;
	if kupcase(model_type_cd)=kupcase("&model_type_code") 
		and kupcase(PRODUCT_TYPE_CD)=kupcase("&prod_type_code")
		and kupcase(level_desc) ne kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))");
	if inms and not inmod1 then do;
		color1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
		model_nm1="&model1";
		model_sk1=&model1_key;
	end;
run;
/*******************/

proc sort data=work.model2 ;
	by Measure_nm;
run;
/*******************/

/* Creating dataset for the second model selected. For measures (except those having
	level_desc as 'Other Reports') which are not there for this model but are there
	 in the measure_dim, White color is assigned */
data work.model2_rep;
	merge work.model2(rename=(model_nm=model_nm2 model_sk=model_sk2 measure_value=measure_value2
							color=color2) in=inmod2)
		work.measure_dim(in=inms);
	by measure_nm;
	if kupcase(model_type_cd)=kupcase("&model_type_code") 
		and kupcase(PRODUCT_TYPE_CD)=kupcase("&prod_type_code")
		and kupcase(level_desc) ne kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))");
	if inms and not inmod2 then do;
		color2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
		model_nm2="&model2";
		model_sk2=&model2_key;
	end;
run;
/*******************/

/* Creating dataset for the first model for measures having level_desc as 'Other Reports' */
proc sql noprint;
	create table work.other1_gif as
	select measure_dim.measure_nm, measure_dim.level_desc, measure_dim.measure_sk, 
		measure_dim.program_nm, measure_dim.seq_no, model_measure_fact.measure_value
	from work.measure_dim left outer join fact.model_measure_fact
	on measure_dim.measure_sk = model_measure_fact.measure_sk
	where kupcase(measure_dim.level_desc) = kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))")
		and kupcase(measure_dim.model_type_cd) = kupcase("&model_type_code")
		and kupcase(measure_dim.PRODUCT_TYPE_CD)=kupcase("&prod_type_code")
		and model_measure_fact.model_sk=&model1_key
		and model_measure_fact.score_time_sk=&time_key;
quit;
/*******************/

proc sort data=work.other1_gif ;
	by measure_nm;
run;
/*******************/

/* For measures which are not there in the above created dataset(for model1), but are there in
	measure_dim, if the measure_value is 1 in the model_measure_fact then no color is assigned
	else White color is assigned to this measure */
data work.model1_otrep;
	merge work.other1_gif(rename=(Measure_value=measure_value1) in=inot)
		work.measure_dim(in=inms);
	by measure_nm;
	length color1 $ 5.;
	if kupcase(level_desc) = kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))")
		and kupcase(model_type_cd)=kupcase("&model_type_code")
		and kupcase(PRODUCT_TYPE_CD)=kupcase("&prod_type_code");
	if measure_value1=1 then
			color1="";
	else color1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
run;
/*******************/

/* Creating dataset for the second model for measures having level_desc as 'Other Reports' */
proc sql noprint;
	create table work.other2_gif as
	select measure_dim.measure_nm, measure_dim.level_desc, measure_dim.measure_sk, 
		measure_dim.program_nm, measure_dim.seq_no, model_measure_fact.measure_value
	from work.measure_dim left outer join fact.model_measure_fact
	on measure_dim.measure_sk = model_measure_fact.measure_sk
	where kupcase(measure_dim.level_desc) = kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))")
		and kupcase(measure_dim.model_type_cd) = kupcase("&model_type_code")
		and kupcase(measure_dim.PRODUCT_TYPE_CD)=kupcase("&prod_type_code")
		and model_measure_fact.model_sk=&model2_key
		and model_measure_fact.score_time_sk=&time_key;
quit;
/*******************/

proc sort data=work.other2_gif ;
	by measure_nm;
run;
/*******************/

/* For measures which are not there in the above created dataset(for model2), but are there in
	measure_dim, if the measure_value is 1 in the model_measure_fact then no color is assigned
	else White color is assigned to this measure */
data work.model2_otrep;
	merge work.other2_gif(rename=(Measure_value=measure_value2) in=inot)
		work.measure_dim(in=inms);
	by measure_nm;
	length color2 $ 5.;
	if kupcase(level_desc) = kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))")
		and kupcase(model_type_cd)=kupcase("&model_type_code") 
		and kupcase(PRODUCT_TYPE_CD)=kupcase("&prod_type_code");
	if measure_value2=1 then
			color2="";
	else color2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
run;
/*******************/

proc sort data=work.model1_rep ;
	by measure_nm;
run;
/*******************/

proc sort data=work.model2_rep ;
	by measure_nm;
run;
/*******************/
proc sort data=work.model1_otrep ;
	by measure_nm;
run;
/*******************/

proc sort data=work.model2_otrep ;
	by measure_nm;
run;
/*******************/

/* Creating dataset containing data for both the models with all the measures */
data work.reports;
	merge work.model1_rep work.model2_rep work.model1_otrep work.model2_otrep;
	by measure_nm;
run;
/*******************/

/*traffic_light_for_level_desc*/
%macro traffic_light_for_level_desc(msr_nm=, red_for_red=., yellow_for_red=., red_for_yellow=.,
		 						  yellow_for_yellow=.);
	%if &msr_nm ne %then %do;
		proc sql noprint;
			create table work.Period_color as
			select * from work.measure_out
			where kupcase(measure_nm) = kupcase(&msr_nm);
		quit;
	%end;

	%else %do;
		proc sort data=work.reports out=work.reports1 ;
			by LEVEL_DESC;
		run;

		data work.reports1(keep= LEVEL_DESC green_cnt_measure1 
										red_cnt_measure1 yellow_cnt_measure1 
										green_cnt_measure2 red_cnt_measure2 
										yellow_cnt_measure2);
		set work.reports1;
		retain green_cnt_measure1 red_cnt_measure1 yellow_cnt_measure1 
			   green_cnt_measure2 red_cnt_measure2 yellow_cnt_measure2 0;
		by LEVEL_DESC;
			if first.LEVEL_DESC then 
			do; 
		/*count of colours for measure 1 values.*/
			     if color1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_green.1, noquote))"  then green_cnt_measure1=green_cnt_measure1+1;
			     if color1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_red.1, noquote))"    then red_cnt_measure1=red_cnt_measure1+1;
			     if color1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_yellow.1, noquote))" then yellow_cnt_measure1=yellow_cnt_measure1+1;
		/*count of colours for measure 2 values.*/
				 if color2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_green.1, noquote))"  then green_cnt_measure2=green_cnt_measure2+1;
				 if color2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_red.1, noquote))"   then red_cnt_measure2=red_cnt_measure2+1;
				 if color2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_yellow.1, noquote))" then yellow_cnt_measure2=yellow_cnt_measure2+1;
			end;
			if last.LEVEL_DESC then output work.reports1;
		run;
		data work.Period_color(drop=Green_cnt_measure1 Yellow_cnt_measure1 Red_cnt_measure1
								       Green_cnt_measure2 Yellow_cnt_measure2 Red_cnt_measure2);
		set work.reports1;
		length color_measure1 color_measure2 $ 6.;
			%if &model1. ne  %then 
			%do;
				if Red_cnt_measure1=0 and Yellow_cnt_measure1=0 and Green_cnt_measure1=0 then
					color_measure1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
				else if Red_cnt_measure1 >= &red_for_red. or Yellow_cnt_measure1 >= &yellow_for_red. then
					color_measure1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_red.1, noquote))";
				else if Red_cnt_measure1 >= &red_for_yellow. or Yellow_cnt_measure1 >= &yellow_for_yellow. then
					color_measure1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_yellow.1, noquote))";
				else color_measure1="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_green.1, noquote))";
			%end;
			%if &model2. ne  %then 
			%do; 
				if Red_cnt_measure2=0 and Yellow_cnt_measure2=0 and Green_cnt_measure2=0 then
					color_measure2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))";
				else if Red_cnt_measure2 >= &red_for_red. or Yellow_cnt_measure2 >= &yellow_for_red. then
					color_measure2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_red.1, noquote))";
				else if Red_cnt_measure2 >= &red_for_yellow. or Yellow_cnt_measure2 >= &yellow_for_yellow. then
					color_measure2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_yellow.1, noquote))";
				else color_measure2="%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_green.1, noquote))";
			%end;
		run;
	%end;
proc sql;
	create table work.test as select * from 
	work.reports Left join work.Period_color
	on reports.LEVEL_DESC=Period_color.LEVEL_DESC;
quit;

%mend traffic_light_for_level_desc;

%macro select_traffic_for_level_desc();
/*Stability: for PD*/
		%if %sysfunc(kupcase(&CS_STAB_MDL_DB_SEL_MSR_PD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_STAB_MSR_NAME_PD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_STAB_NO_RED_FOR_RED_PD.,
						yellow_for_red=&CS_STAB_NO_YELLOW_FOR_RED_PD.,
						red_for_yellow=&CS_STAB_NO_RED_FOR_YELLOW_PD.,
						yellow_for_yellow=&CS_STAB_NO_YELLOW_FOR_YELLOW_PD.);
		%end;
/*Performance: for PD*/
				%if %sysfunc(kupcase(&CS_PERF_MDL_DB_SEL_MSR_PD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_PERF_MSR_NAME_PD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_PERF_NO_RED_FOR_RED_PD.,
						yellow_for_red=&CS_PERF_NO_YELLOW_FOR_RED_PD.,
						red_for_yellow=&CS_PERF_NO_RED_FOR_YELLOW_PD.,
						yellow_for_yellow=&CS_PERF_NO_YELLOW_FOR_YELLOW_PD.);
		%end;
/*Calibration: for PD*/
     	%if %sysfunc(kupcase(&CS_CLIB_MDL_DB_SEL_MSR_PD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_CLIB_MSR_NAME_PD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_CLIB_NO_RED_FOR_RED_PD.,
						yellow_for_red=&CS_CLIB_NO_YELLOW_FOR_RED_PD.,
						red_for_yellow=&CS_CLIB_NO_RED_FOR_YELLOW_PD.,
						yellow_for_yellow=&CS_CLIB_NO_YELLOW_FOR_YELLOW_PD.);
		%end;

/**/
/*Stability: for LGD*/
		%if %sysfunc(kupcase(&CS_STAB_MDL_DB_SEL_MSR_LGD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_STAB_MSR_NAME_LGD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_STAB_NO_RED_FOR_RED_LGD.,
						yellow_for_red=&CS_STAB_NO_YELLOW_FOR_RED_LGD.,
						red_for_yellow=&CS_STAB_NO_RED_FOR_YELLOW_LGD.,
						yellow_for_yellow=&CS_STAB_NO_YELLOW_FOR_YELLOW_LGD.);
		%end;
/*Performance: for LGD*/
				%if %sysfunc(kupcase(&CS_PERF_MDL_DB_SEL_MSR_LGD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_PERF_MSR_NAME_LGD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_PERF_NO_RED_FOR_RED_LGD.,
						yellow_for_red=&CS_PERF_NO_YELLOW_FOR_RED_LGD.,
						red_for_yellow=&CS_PERF_NO_RED_FOR_YELLOW_LGD.,
						yellow_for_yellow=&CS_PERF_NO_YELLOW_FOR_YELLOW_LGD.);
		%end;
/*Calibration: for LGD*/
     	%if %sysfunc(kupcase(&CS_CLIB_MDL_DB_SEL_MSR_LGD.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_CLIB_MSR_NAME_LGD.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_CLIB_NO_RED_FOR_RED_LGD.,
						yellow_for_red=&CS_CLIB_NO_YELLOW_FOR_RED_LGD.,
						red_for_yellow=&CS_CLIB_NO_RED_FOR_YELLOW_LGD.,
						yellow_for_yellow=&CS_CLIB_NO_YELLOW_FOR_YELLOW_LGD.);
		%end;

/**/
/*Stability: for CCF*/
		%if %sysfunc(kupcase(&CS_STAB_MDL_DB_SEL_MSR_CCF.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_STAB_MSR_NAME_CCF.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_STAB_NO_RED_FOR_RED_CCF.,
						yellow_for_red=&CS_STAB_NO_YELLOW_FOR_RED_CCF.,
						red_for_yellow=&CS_STAB_NO_RED_FOR_YELLOW_CCF.,
						yellow_for_yellow=&CS_STAB_NO_YELLOW_FOR_YELLOW_CCF.);
		%end;
/*Performance: for CCF*/
				%if %sysfunc(kupcase(&CS_PERF_MDL_DB_SEL_MSR_CCF.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_PERF_MSR_NAME_CCF.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_PERF_NO_RED_FOR_RED_CCF.,
						yellow_for_red=&CS_PERF_NO_YELLOW_FOR_RED_CCF.,
						red_for_yellow=&CS_PERF_NO_RED_FOR_YELLOW_CCF.,
						yellow_for_yellow=&CS_PERF_NO_YELLOW_FOR_YELLOW_CCF.);
		%end;
/*Calibration: for CCF*/
     	%if %sysfunc(kupcase(&CS_CLIB_MDL_DB_SEL_MSR_CCF.)) eq %sysfunc(kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.mesure_flag_y.1, noquote))")) %then %do;
			%traffic_light_for_level_desc(msr_nm=&CS_CLIB_MSR_NAME_CCF.);
		%end;
		%else %do;
			%traffic_light_for_level_desc(red_for_red=&CS_CLIB_NO_RED_FOR_RED_CCF.,
						yellow_for_red=&CS_CLIB_NO_YELLOW_FOR_RED_CCF.,
						red_for_yellow=&CS_CLIB_NO_RED_FOR_YELLOW_CCF.,
						yellow_for_yellow=&CS_CLIB_NO_YELLOW_FOR_YELLOW_CCF.);
		%end;
/**/
%mend select_traffic_for_level_desc;
%select_traffic_for_level_desc();
/**/
proc sort data=work.test out=work.reports ;
	by level_desc seq_no;
run;

/* Start HTML Screen*/
data _NULL_;
	file _webout&webout_options;
	set work.reports end=last;
	by level_desc;
	if _N_=1 then do;
		put '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" '@;
		put '"http://www.w3.org/TR/html4/loose.dtd"> ';
		put '<html> ';
		put '<head> ';
/*I18N-scnhuh-20080418-1023*/
/*from iso-8859-1 to utf-8*/
		put '<meta http-equiv="Content-Type" content="text/html; charset=utf-8"> ';
/*I18N-scnhuh-20080418-1023*/		
		put '<title>'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_monitoring.1, noquote))"@;
		if "&model_type_code"=&PD_APP_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_appl_pd.1, noquote))"@;
		else if "&model_type_code"=&PD_ONG_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_ong_pd.1, noquote))"@;
		else if "&model_type_code"=&LGD_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_lgd.1, noquote))"@;
		else if "&model_type_code"=&CCF_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_ccf.1, noquote))"@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_model.1, noquote))"@;
		put '</title> ';
		put '<link href="SASTheme_default/themes/default/styles/default.css" rel="stylesheet" type="text/css"> ';
		put '<link href="/SASCSReports/styles/default.css" rel="stylesheet" type="text/css"> ';
		put '<style type="text/css"> ';
		put '<!-- ';
		put '.style1 {color: #003399} ';
		put '--> ';
		put '</style> ';
/*		put '<link href="/SASCSReports/styles/default.css" rel="stylesheet" type="text/css"> ';*/
		
		put '<script>';
		put '<!--';

/* Calls the Reporting stored process if link is clicked */
		put 'function Link1() {';
			put "form3._program.value=&CS_REPORTING";
			put 'form3.submit()';
		put '}';

/* Calls the mon_models stored process if link is clicked */
		put 'function Link2() {';
			put "form3._program.value=&CS_MON_MDL";
			put 'form3.submit()';
		put '}';

/* Calls the select_models stored process if link is clicked */
		put 'function Link3() {';
			put "form3._program.value=&CS_MDL_LVL_DB";
			put 'form3.submit()';
		put '}';

/* When a measure is selected from the first model then that model is passed as model1
	and the second model is passed as model2 to the respective model monitoring report
	screen called */
		put 'function Reports1(prg_nm,ms_nm){';
			put 'form3._program.value=prg_nm';
			put 'form3.prg_name.value=prg_nm';
			put 'form3.statistic.value=ms_nm';
			put 'form3.outcome_period.value="'@;
			put "&target_period1"@;
			put '"';
			put 'form3.elapsed_time.value="'@;
			put "&elapsed_time1" @;
			put '"';
		
			if "&model1" eq %str("") then do;
				put 'form3.model1.value=""';
				put 'form3.model1_key.value=""';
				put 'form3.sel_model.value=""';
				put 'form3.sel_model_key.value=""';
			end;
/*			else do;*/
/*				put 'form3.model1.value="'@;*/
/*				put "&model1"@;*/
/*				put '"';*/
/*				put 'form3.model1_key.value="'@;*/
/*				put "&model1_key"@;*/
/*				put '"';*/
/*			end;*/
/*			if "&model2" eq %str("") then do;*/
/*				put 'form3.model2.value=""';*/
/*				put 'form3.model2_key.value=""';*/
/*			end;*/
/*			else do;*/
/*				put 'form3.model2.value="'@;*/
/*				put "&model2"@;*/
/*				put '"';*/
/*				put 'form3.model2_key.value="'@;*/
/*				put "&model2_key"@;*/
/*				put '"';*/
/*			end;*/
			else do;
				put 'form3.sel_model.value="'@;
				put "&model1"@;
				put '"';
				put 'form3.sel_model_key.value="'@;
				put "&model1_key"@;
				put '"';
			end;
			put 'form3.submit()';
		put '}';

/* When a measure is selected from the second model then that model is passed as model1
	and the first model is passed as model2 to the respective model monitoring report
	screen called */
		put 'function Reports2(prg_nm,ms_nm){';
			put 'form3._program.value=prg_nm';
			put 'form3.prg_name.value=prg_nm';
			put 'form3.statistic.value=ms_nm';
			put 'form3.outcome_period.value="'@;
			put "&target_period2"@;
			put '"';
			put 'form3.elapsed_time.value="'@;
			put "&elapsed_time2" @;
			put '"';

/*			if "&model2" eq %str("") then do;*/
/*				put 'form3.model1.value=""';*/
/*				put 'form3.model1_key.value=""';*/
/*			end;*/
/*			else do;*/
/*				put 'form3.model1.value="'@;*/
/*				put "&model2"@;*/
/*				put '"';*/
/*				put 'form3.model1_key.value="'@;*/
/*				put "&model2_key"@;*/
/*				put '"';*/
/*			end;*/
/*			if "&model1" eq %str("") then do;*/
/*				put 'form3.model2.value=""';*/
/*				put 'form3.model2_key.value=""';*/
/*			end;*/
/*			else do;*/
/*				put 'form3.model2.value="'@;*/
/*				put "&model1"@;*/
/*				put '"';*/
/*				put 'form3.model2_key.value="'@;*/
/*				put "&model1_key"@;*/
/*				put '"';*/
/*			end;*/
			if "&model1" eq %str("") then do;
				put 'form3.model1.value=""';
				put 'form3.model1_key.value=""';
			end;
			if "&model2" eq %str("") then do;
				put 'form3.model2.value=""';
				put 'form3.model2_key.value=""';
				put 'form3.sel_model.value=""';
				put 'form3.sel_model_key.value=""';
			end;
			else do;
				put 'form3.sel_model.value="'@;
				put "&model2"@;
				put '"';
				put 'form3.sel_model_key.value="'@;
				put "&model2_key"@;
				put '"';
			end;
			put 'form3.submit()';
		put '}';

		put '//-->';
		put '</script>';

		put '</head> ';
		put '<body>';
		put '<form name="form3" method="post" action="/SASStoredProcess/do">';

/* Hidden textboxes to hold the values for the program name and the output paremeters 
	to pass the values to the next Stored Process called */
		put '<input type="hidden" name="_program">';
		prod_type_desc=symget("prod_type_desc");
		put '<input type="hidden" name="prod_type_desc" value="' @;
		put prod_type_desc @;
		put '">';
		prod_type_code=symget("prod_type_code");
		put '<input type="hidden" name="prod_type_code" value="' @;
		put prod_type_code @;
		put '">';
		model_type_desc=symget("model_type_desc");
		put '<input type="hidden" name="model_type_desc" value="' @;
		put model_type_desc;
		put '">';
		model_type_code=symget("model_type_code");
		put '<input type="hidden" name="model_type_code" value="' @;
		put model_type_code @;
		put '">';
		include_exp_mdl=symget("include_exp_mdl");
		put '<input type="hidden" name="include_exp_mdl" value="' @;
		put include_exp_mdl @;
		put '">';
		model1=symget("model1");
		put '<input type="hidden" name="model1" value="' @;
		put model1 @;
		put '">';
		model1_key=symget("model1_key");
		put '<input type="hidden" name="model1_key" value="' @;
		put model1_key @;
		put '">';
		model2=symget("model2");
		put '<input type="hidden" name="model2" value="' @;
		put model2 @;
		put '">';
		model2_key=symget("model2_key");
		put '<input type="hidden" name="model2_key" value="' @;
		put model2_key @;
		put '">';
		sel_model=symget("model");
		put '<input type="hidden" name="sel_model" value="' @;
		put sel_model @;
		put '">';
		sel_model_key=symget("sel_model_key");
		put '<input type="hidden" name="sel_model_key" value="' @;
		put sel_model_key @;
		put '">';
		period=symget("period");
		put '<input type="hidden" name="period" value="' @;
		put period @;
		put '">';
		time_key=symget("time_key");
		put '<input type="hidden" name="time_key" value="' @;
		put time_key @;
		put '">';
		put '<input type="hidden" name="elapsed_time">';
		put '<input type="hidden" name="outcome_period">';
		put '<input type="hidden" name="statistic">';
		put '<input type="hidden" name="prg_name">';
/*********/

		put '<table width="100%"  border="0" cellpadding="0" cellspacing="0" bgcolor="#FAF8ED"> ';
		put '<tr> ';
		put '<td align="left"> ';
		put '<table width="100%"  border="0" align="left" cellpadding="3" cellspacing="0"> ';
		put '<tr> ';

/*Links on Top of the HTML Page*/
		put '<td><table width="100%"  border="0" cellspacing="0" cellpadding="0"> ';
		put '<tr> ';
		put '<td><span class="BreadCrumb">';
		put '<a href="javascript:Link1()">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.link_reporting.1, noquote))"@;
		put '</a> &gt; ';
		put '<a href="javascript:Link2()">'@;
/*I18N-scnhuh-20080420-1109*/
/*navigation menu link error, fixing: change key value of sasmsg() */		
		/*put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.link_monitoring_model_input.1, noquote))"@;*/
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.select_models.link_monitoring_model.1, noquote))"@;
/*I18N-scnhuh-20080420-1109*/		
		put '</a> &gt; ';
		put '<a href="javascript:Link3()">'@;
/*I18N-scnhuh-20080420-1109*/
/*navigation menu link error, fixing: change key value of sasmsg() */		
		/*put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.link_select_variable.1, noquote))"@;*/
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.select_models.link_select_model.1, noquote))"@;
/*I18N-scnhuh-20080420-1109*/		
		put '</a> &gt; ';
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.link_select_statistic.1, noquote))"@;
		put '</span></td> ';
/*********/

/*SAS Banner*/
		put '<td width="215"><div align="right">';
		put '<img src="/SASCSReports/images/CrBanner.gif" width="259" height="29"></div></td> ';
/*********/

		put '</tr> ';
		put '</table><H2 align="left">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_monitoring.1, noquote))"@;
		if "&model_type_code"=&PD_APP_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_appl_pd.1, noquote))"@;
		else if "&model_type_code"=&PD_ONG_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_ong_pd.1, noquote))"@;
		else if "&model_type_code"=&LGD_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_lgd.1, noquote))"@;
		else if "&model_type_code"=&CCF_MODEL then
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.model_ccf.1, noquote))"@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_model.1, noquote))"@;
		put'</H2></td> ';
		put '</tr> ';

/*Values of Product_type and Period*/
		put '<tr> ';
		put '<td align="left"> ';
		put '<p class="LabelFont">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_model_type.1, noquote))"@;
		put "&model_type_desc" @;
		put '</p> ';
		put '<p class="LabelFont">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_product_type.1, noquote))"@;
		put "&prod_type_desc" @;
		put '</p> ';
		put '<p class="LabelFont">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_period.1, noquote))"@;
		
/*I18N-scnhuh-20080425-1959*/	
/* del one line, add two lines. For displaying in NLS format */	
/*	put "&period" @;*/
		Period_WEBUI=put(input("&period",&CS_TIME_PER_FMT.),&NLDATE_FMT.);
		put Period_WEBUI @;
/*I18N-scnhuh-20080425-1959*/		
		put '</p> ';
/*********/

	/*Note for selecting report*/
		put '<p>&nbsp;</p><p class=NoteFont>'@;
		put '<strong>'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.Note_Note.1, noquote))"@;
		put'</strong>';
		put '<p class=NoteFont>';
		put "1.%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.Note_click_trafic_light.1, noquote))";
		put'</p>';
		/*CS4.3 HF2 */
		put '<p class=NoteFont>';
		put "2.%sysfunc(sasmsg(smd_ds.bismsg,ks.sas.1, noquote))";
		put'</p>';
	/*********/

/* HTML code for creating a table containing the information for all the measures for the
	selected models */
		put '<table width="80%" border="0" cellpadding="0" cellspacing="0" bgcolor="#7F9DB9" class="tableborder"> ';
		put '<tr> ';
		put '<td valign="top">'@;
		put '<div class="tableviewcomposite">'@;
		put '<table width="100%" border="0" cellpadding="0" cellspacing="0" class="HZtableborder"> ';
		put '<tr valign="top" bgcolor="#FFFFFF"> ';
		put '<th class="HZtableborder" rowspan="2">';
	/* Column names for the table */
		put '<a >'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_test_level.1, noquote))"@;
		put '</a></th>';
		put '<th class="HZtableborder" rowspan="2">';
		put '<a >'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_test_name.1, noquote))"@;
		put '</a></th>'; 
		if "&model1" ne "" then do;
			put '<th class="HZtableborder" colspan="2">';
			put'<a valign="center" class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_model.1, noquote))"@;
			put '-'@;
			put "&model1" @;
			put '</a></th>';
		end;
		if "&model2" ne "" then do;
			put '<th class="HZtableborder" colspan="2">'; 
			put '<a valign="center" class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.title_model.1, noquote))"@;
			put '-'@;
			put "&model2" @;
			put '</a></th>';
		end;
			put '</tr><tr>';
		if "&model1" ne "" then do;
			put '<th class="HZtableborder"><a class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_level.1, noquote))"@;
			put '</a></th>';
			put '<th class="HZtableborder"><a class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_statistics.1, noquote))"@;
			put '</a></th>';
		end;
		if "&model2" ne "" then do;
			put '<th class="tablenumcol"><a class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_level.1, noquote))"@;
			put '</a></th>';
			put '<th class="tablenumcol"><a class="HZtableborder">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.column_statistics.1, noquote))"@;
			put '</a></th>';
		end;
			put ' </tr>';
		put '<tr valign="top" class="tableoddrow"><td class="tableimgcol"></td></tr>';
 	/*********/
	end;

	/* Code for populating the table created on the html screen */
	if measure_nm ne '' then do;
		if mod(_N_,2)=0 then do;
 			put '<tr valign="top" class="tableevenrow" bgcolor="#F4F4F2">';
 			put '<td class="Font12px">';
		end;
		else do;
			put '<tr valign="top" class="tableevenrow" bgcolor="#FFFFFF">';
 			put '<td class="LabelFont">'@;
		end;
		if first.level_desc then do;
			put level_desc @;
			put '</td> ';
		end;
		else do;
			put '</td>';
		end;
		val=PRXMATCH(kupcase('/p-value/'),kupcase(measure_nm));
 		put '<td class="verttebleborder">'@;
		put measure_nm @;
		put '</td> ';
		if "&model1" ne "" then do;
 			if kupcase(Model_nm1)=kupcase("&model1") then do;
 				put '<td class="verttebleborder">';
  			  	if first.level_desc then 
				do;
					put '<img src="/SASCSReports/images/' @;
					put 'Sphere'@;
					put color_measure1;
					put '.gif" width="16" height="16" border="0">';
					put '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
					put '</td>';
				end;
				else do;
 				    put '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
 				end;
 			/* If the measure has pvalue then pvalue6.4 format is applied else 7.3 format
				is applied */
 				if kupcase(color1)=kupcase(%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))) then do;
					put '<td class="verttebleborder">';
					put '<img src="/SASCSReports/images/spacer.gif" width="16" height="16" border="0"></td>' @;
				end;
				else do;
					put'<td class="verttebleborder">';
					put '<a id="';
					put program_nm;
					put '" href="javascript:Reports1('''@;
					put program_nm;
					put ''','''@;
					put measure_nm;
					put ''')">'@;
					put '<img src="/SASCSReports/images/' @;
					put 'Sphere'@;
					put color1;
					put '.gif" width="16" height="16" border="0"></a>';
				end;
				if val = 0 then do;
					put measure_value1 &CS_DEC_FMT. @;
				end;
				else do;
					put measure_value1 &CS_PVAL_FMT. @;
				end;
				put '</td>';
			end;
			else do;
				if kupcase(level_desc)=kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))") then do;
 					put '<td class="verttebleborder">';
					if kupcase(color1)=kupcase(%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))) then  
						put '<img src="/SASCSReports/images/spacer.gif" width="16" height="16" border="0"></td>';
					else do;
						put '&nbsp;</td>';
 						put '<td class="verttebleborder">';
						put '<a id="';
						put program_nm @;
						put '" href="javascript:Reports1('''@;
						put program_nm;
						put ''','''@;
						put measure_nm @;
						put ''')">'@;
						put '<img src="/SASCSReports/images/Report2.gif" width="16" height="16" border="0"></a></td> ';
					end;
				end;
				else
					put '<td>&nbsp;</td>';
			end;
 		end;
		if "&model2" ne "" then do;
 			if kupcase(Model_nm2)=kupcase("&model2") then do;
 				put '<td class="verttebleborder">';
  			  	if first.level_desc then 
				do;
					put '<img src="/SASCSReports/images/' @;
					put 'Sphere'@;
					put color_measure2;
					put '.gif" width="16" height="16" border="0">';
					put '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
					put '</td>';
				end;
				else do;
 				    put '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
 				end;
 			/* If the measure has pvalue then pvalue6.4 format is applied else 7.3 format
				is applied */
 				if kupcase(color2)=kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))") then do;
					put '<td class="verttebleborder">';
					put '<img src="/SASCSReports/images/spacer.gif" width="16" height="16" border="0"></td>' @;
				end;
				else do;
					put'<td class="verttebleborder">';
					put '<a id="';
					put program_nm;
					put '" href="javascript:Reports2('''@;
					put program_nm;
					put ''','''@;
					put measure_nm;
					put ''')">'@;
					put '<img src="/SASCSReports/images/' @;
					put 'Sphere'@;
					put color2;
					put '.gif" width="16" height="16" border="0"></a>';
				end;
				if val = 0 then do;
					put measure_value2 &CS_DEC_FMT. @;
				end;
				else do;
					put measure_value2 &CS_PVAL_FMT. @;
				end;
				put '</td>';
			end;
			else do;
				if kupcase(level_desc)=kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.level_desc_other_reports.1, noquote))") then do;
 					put '<td class="verttebleborder">';
					if kupcase(color2)=kupcase("%sysfunc(sasmsg(smd_ds.bismsg,crs.test_levels.color_white.1, noquote))") then
						put '<img src="/SASCSReports/images/spacer.gif" width="16" height="16" border="0"></td> ';
					else do;
						put '&nbsp;</td>';
						put '<td class="verttebleborder">';
						put '<a id="';
						put program_nm @;
						put '" href="javascript:Reports2('''@;
						put program_nm;
						put ''','''@;
						put measure_nm @;
						put ''')">'@;
						put '<img src="/SASCSReports/images/Report2.gif" width="16" height="16" border="0"></a></td> ';
					end;
				end;
				else
					put '<td>&nbsp;</td>';
			end;
 		end;
 	end;
	if last then do;
	/* Code for table containing the information for all the measures for the
		selected models ends here */
		put '</tr> ';
		put '</table></td> ';
		put '</table> ';
		put '</td> ';
		put '</tr> ';
		put '<tr> ';
		put '<td align="left"> ';
		put '<p>&nbsp;</p> ';
		put '</td> ';
		put '</tr> ';
		put '</table> ';
		put '</td> ';
		put '</tr> ';
		put '<tr> ';
		put '<td height="32" bgcolor="#F1EACB">&nbsp; ';
		put '</td> ';
		put '</tr> ';
		put '</table> ';
		put '</form> ';
		put '</body> ';
		put '</html> ';
	end;
run;
