/***********************************************************************************************
Module:  shift_index

Function:  This stored process generates an HTML report displaying Shift Index 
   statistics, the Score shift Index and Event shift Index table
   and their overtime graphs.

Authors:  BIS Team
Date:  17 April 2006
SAS:  9.1
Called-by:	Stored Process test_levels
	
Calls:  Calls the following Stored Process    
		1. Reporting	  	 (when Reporting link is clicked)
		2. mon_model_inputs	 (when Monitoring Model Inputs link is clicked)
		3. model_in_var    	 (when Select Variable link is clicked)

	Calls the following Macros
		1. graph_options macro sets all the graph options and selects the colors
			to be used when rendering the graph.
		2. generate_feed_mip macro for obtaining the required feed.
		3. feed_measure_mip macro which makes data for the overtime graphs
		4. drawtable macro.					

Datasets used:  No input datasets required for processing of the stored process

Logic:  1. All inputs to the stored process are accepted as global parameters. 
	2. Standards
		a. 	Naming of data set which are input to and output from macro using
				<prod_type_code><model1_key><time_key> Extension as per requirement  
			i.e.		LON           100          1036     pool
				as LON1001036pool.
		b. Setting the default values for feed_pd and where_feed to be passed as
			parameter(infilter) to the generating feed macro.
		c.	Naming the graphs using timestamp and two letters for identifying 
			the graph.
	3. Set all the graph options and select the colors to be used when rendering
		the graph.
	4. Call the macro generate_feed_mip for obtaining the required feed.
	5. Check whether the generate_feed_mip macro generated any error.
		If any error is generated then the same report(shift_index) is called
		with the default criteria.
	6. If there no data, in the case of business filter or dynamic pooling,
		is generated by the generate_feed macro then the same report(shift_index)
		is called with the default criteria.
	7. Get the pooling technique.
    8. Generate the HTML using streaming webout.
		a. Provide hidden textboxes for all output parameters so that the values 
			are easily submitted to the next page. 
		b. Provide javascript functions for the links on top of the page
			and for edit and reset buttons.

Parameters: 
INPUT 
	prod_type_desc		:   Product Type Description (e.g., Credit Card,
							Mortgage, Loan, Core)
    prod_type_code		:   Product Type Code (e.g., CCD, MTG, LON, COR)
	model_type_desc		:	Analytical Model Type Description 
							(e.g., PD Approvals, PD Ongoing, LGD, CCF)
    model_type_code   	:   Analytical Model Type Code (e.g., PD Approvals,
							PD Ongoing, LGD, CCF)
    statistic           :   Statistic name (Score shift Index).
	model				:	The selected model (Eg. PD MODEL 1).		
	model_key			:	The surrogate key for the selected model.
	period				:	The selected period (month and year)(Eg DEC05 JAN06).
	time_key			:	The surrogate key for selected period.
	model_measure_type	:   Analytical Model Type Code (e.g., PD Approvals, PD Ongoing,
							LGD, CCF)
	variable			:   The value of variable (e.g Variable 21).
	variable_key		:   The value of variable_key (e.g 21).
	prg_name			:	The program name, for calling the report, which is
							picked from program_nm column of dim.measure_dim.
	outcome_period		:	The target period in months.
	elapsed_time		:	The time elapsed.
				
OUTPUT
	prod_type_desc		:   Product Type Description (e.g., Credit Card,
							Mortgage, Loan, Core)
    prod_type_code		:   Product Type Code (e.g., CCD, MTG, LON, COR)
	model_type_desc		:	Analytical Model Type Description 
							(e.g., PD Approvals, PD Ongoing, LGD, CCF)
    model_type_code   	:   Analytical Model Type Code (e.g., PD Approvals,
							PD Ongoing, LGD, CCF)
    statistic           :   Statistic name (Score shift Index).
	model				:	The selected model (Eg. PD MODEL 1).		
	model_key			:	The surrogate key for the selected model.
	period				:	The selected period (month and year)(Eg DEC05 JAN06).
	time_key			:	The surrogate key for selected period.
	model_measure_type	:   Analytical Model Type Code (e.g., PD Approvals, PD Ongoing,
							LGD, CCF)
	variable			:   The value of variable (e.g Variable 21).
	variable_key		:   The value of variable_key (e.g 21).
	called_program		:	Passes the path of the current program(shift_index)
	prg_name			:	The program name, for calling the report, which is
							picked from program_nm column of dim.measure_dim.
	outcome_period		:	The target period in months.
	elapsed_time		:	The time elapsed.

***********************************************************************************************/
goptions reset=all;
options nomprint;

/* Global variable declaration */
%global prod_type_desc prod_type_code model_type_code model model_key 
		period time_key variable variable_key flag model_measure_type statistic prg_name
		model_type_desc outcome_period elapsed_time format_name;
 
/* Trimming extra spaces from all the input parameters*/
%macro macro_val_check;
%if "&prod_type_desc" ne "" %then
%let prod_type_desc=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&prod_type_desc))))));
%if "&prod_type_code" ne "" %then
%let prod_type_code=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code))));
%if "&model_type_desc" ne "" %then
%let model_type_desc=%str(%sysfunc(ktrim(%sysfunc(kleft(&model_type_desc)))));
%if "&model_type_code" ne "" %then
%let model_type_code=%sysfunc(ktrim(%sysfunc(kleft(&model_type_code))));
%if "&model_measure_type" ne "" %then
%let model_measure_type=%sysfunc(ktrim(%sysfunc(kleft(&model_measure_type))));
%if "&model" ne "" %then
%let model=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&model))))));
%if "&model_key" ne "" %then
%let model_key=%sysfunc(ktrim(%sysfunc(kleft(&model_key))));
%if "&period" ne "" %then
%let period=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&period))))));
%if "&time_key" ne "" %then
%let time_key=%sysfunc(ktrim(%sysfunc(kleft(&time_key))));
%if "&variable" ne "" %then
%let variable=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&variable))))));
%if "&variable_key" ne "" %then
%let variable_key=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&variable_key))))));
%if "&statistic" ne "" %then
%let statistic=%str(%bquote(%sysfunc(ktrim(%sysfunc(kleft(&statistic))))));
%if "&prg_name" ne "" %then
%let prg_name=%str(%sysfunc(ktrim(%sysfunc(kleft(&prg_name)))));
%if "&outcome_period" ne "" %then
%let outcome_period=%str(%sysfunc(ktrim(%sysfunc(kleft(&outcome_period)))));
%if "&elapsed_time" ne "" %then
%let elapsed_time=%str(%sysfunc(ktrim(%sysfunc(kleft(&elapsed_time)))));
/*************/
%mend;
%macro_val_check

/*Column Names and labels which are passed to the drawtable macro*/
%let colnames=%str(Attr_Name,Seq_No,scorecard_points,prop_actual,prop_dev,prop_evt_act,prop_evt_dev,score_shift_index,event_shift_index);
%let collabels=%str();

/*Assigning standard names for datasets which are input to and ouput from macro using
		<prod_type_code><model1_key><time_key> Extension as per requrment  
	i.e.		COR           106          1030     pool
		as COR1061030pool.*/
%let out_lib=;
%debug_lib_path;
/*This macro outputs the format for the stattistics for combination of 
  product type code and the model type code.*/
%format_for_model_product(model_type_code_fm=&model_type_code,prod_type_code_fm=&prod_type_code)
/*This macro chnage the dataset name and hardcode it on the product type*/
%change_dataset_name
%let out_simip=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code.&model_key.&time_key.si_mp))));
%let out_simip_account=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code.&model_key.&time_key.&variable_key.si_ac_mp))));

/*Used in calls to feed measure, data preparation and PD_Vs_Defaultrate macro*/
%let out_stat_ds=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code.&model_key.&time_key.&variable_key.sei_stat))));
%let out_stat_ot=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code.&model_key.&time_key.&variable_key.sei_ot))));
%let in_stat_ot=%sysfunc(ktrim(%sysfunc(kleft(&prod_type_code.&model_key.&time_key.&variable_key.sei_in))));

/*Used to name graphs*/
%let gname=;
%let otn=;

/* Setting values for the feed_for and where_clause*/
%let where_feed=;
%let where_measure=;

%let feed_for=;

/*The data _null_ step below is to perform checking and initialize the required variables*/
data _null_;
/* Setting the feed_for depending on its model type.
	This is passed as parameter to generate_feed */
	if trim(left("&model_type_code"))=&PD_APP_MODEL then
	            call symput('feed_for',"&FEED_FOR_APPLPD");
	else if trim(left("&model_type_code"))=&PD_ONG_MODEL then
	            call symput('feed_for',"&FEED_FOR_PD");
	else if trim(left("&model_type_code"))=&LGD_MODEL then
	            call symput('feed_for',"&FEED_FOR_LGD");
	else if trim(left("&model_type_code"))=&LGD_APP_MODEL then
	            call symput('feed_for',"&FEED_FOR_APPLLGD");
	else if trim(left("&model_type_code"))=&CCF_MODEL then
	            call symput('feed_for',"&FEED_FOR_CCF");

	if trim(left("&statistic"))="" then
	   call symput('statistic',"%sysfunc(sasmsg(smd_ds.bismsg,crs.measure_nm.measure_nm_score_shift_index.1, noquote))");

	if "&variable_key" = "" then
	 	call symput('variable_key','1');

	call symput('where_measure','where ANALYTICAL_MODEL_DIM.MODEL_PRODUCT_TYPE_CD="&prod_type_code" and ANALYTICAL_MODEL_DIM.MODEL_SK=&model_key and SCORE_CARD_VARIABLE_FACT.VARIABLE_SK=&variable_key');
	call symput('where_feed','1');
run;

/* All graph options */
    /* Setting the graph name */
    /* Calling graph_options macro */
data _NULL_;
	name=put(time(),mmss8.1);
	name1=kscan(name,1,":.");
	name2=kscan(name,2,":.");
	name3=kscan(name,3,":.");
	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	name5="i1"||ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	name6="i2"||ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	call symput('sscc',name5);
	call symput('ssnc',name6);
	call symput('gname1',name4);
run;
%let s_i1=%sysfunc(ktrim(%sysfunc(kleft(&sscc))));
%let s_i2=%sysfunc(ktrim(%sysfunc(kleft(&ssnc))));
%let graph_name=%sysfunc(ktrim(%sysfunc(kleft(&gname1))));

/* The graph_options macro sets all the graph options and selects the colors to be used when
	rendering the graph. To access the colors being set in this macro, the respective macro
	variables for the color (color_symbol1, color_symbol2, color_symbol3) is created.
	This macro only sets the values in the macro variables.

	This macro sets the out path of the gifs to the folder created in the environment which
	will be shared between the server layer and web layer. This is picked up from the macro
	variable CS_REPORT_PATH in the bis_customize_para.sas. */
%let color_symbol1=;
%let color_symbol2=;
%graph_options;

/*The data _NULL_ step below is to produce the first part of the HTML which also includes 
the javascript blocks*/

data _NULL_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
	file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/
	rep="&statistic";
	format rep &format_name..;
	put '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" '@;
	put '"http://www.w3.org/TR/html4/loose.dtd"> ';
	put '<html> ';
	put '<head> ';
/*I18N-scnhuh-20080418-1023*/
/*from iso-8859-1 to utf-8*/
	put '<meta http-equiv="Content-Type" content="text/html; charset=utf-8"> ';
/*I18N-scnhuh-20080418-1023*/	
	put '<title>'@;
	put rep @;
	put '</title>';
	put '<link href="SASTheme_default/themes/default/styles/default.css" rel="stylesheet" type="text/css"> ';
	put '<link href="/SASCSReports/styles/default.css" rel="stylesheet" type="text/css"> ';
	put '<style type="text/css"> ';
	put '<!-- ';
	put '.style2 {font-size: small} ';
	put '--> ';
	put '</style> ';

/**************************Javascript Starts here ****************************************/
	put '<script>';
	put '<!--';

	/*Checking for missing global parameters*/
	if "&prod_type_desc" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_prod_type_desc.1, noquote))"@;
		put'");';
	end;
	else if "&prod_type_code" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_prod_type_desc.1, noquote))"@;
		put '");';
	end;
	else if "&model_type_code" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_model_type_cd.1, noquote))"@;
		put '");';
	end;
	/* Start of CS4.3 HF2 */
	/* The lines below are removed/commented as this check is not required	*/
	/*else if "&sel_model" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_select_model.1, noquote))"@;
		put '");';
	end;
	else if "&sel_model_key" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_select_model_key.1, noquote))"@;
		put '");';
	end;*/
	/* End of CS4.3 HF2 */
	else if "&period" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_period.1, noquote))"@;
		put '");';
	end;
	else if "&time_key" eq %str("") then 
	do;
		put 'alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.alert_error_put_time_key.1, noquote))"@;
		put '");';
	end;

	/******************/

	/*Functions for the links on top of the html page*/

	put 'function Link1() {';
		put "form1._program.value=&CS_REPORTING";
		put 'form1.submit()';
	put '}';

	put 'function Link2() {';
		put "form1._program.value=&CS_MON_MDL_IP";
		put 'form1.submit()';
	put '}';

	put 'function Link3() {';
		put "form1._program.value=&CS_VAR_LVL_DB";
		put 'form1.submit()';
	put '}';

 /*Calls this stored process again after assigning statistic the value selected from the
		combo box and hence displaying the overtime graph for the selected statistic */
	put 'function display(){';
		put 'var indx=form1.measure_name.selectedIndex';
		put 'var choice=form1.measure_name.options[indx].value';
		put	'if(document.images) {';
		put	'document.form1.my_img.src=choice';
		put	'}';
	put '}';

/*********JAVASCRIPT FOR SELECTING RADIO BUTTON***********/
	put	'function swapIt(choice) {';
	put	'	if(document.images) {';
	put	'		document.form1.Cumulative.src=choice';/**CHANGE THE SOURCE DYNAMICALY AS USER CLICK THE RADIO BUTTON**/
	put	'	}';
	put	'}';

	/*Function savepage() for saving the page to disk */
	%savepage;
	/*********savepage() ends here*************/

	/*Function printpage() for printing the page to a printer or to a pdf file*/
	%printpage;
	/*********printpage() ends here*************/

	put '//-->';
	put '</script>';

/**************************Javascript Ends here ****************************************/

	put '</head> ';
	put '<body>';

	put '<form name="form1" method="post" action="/SASStoredProcess/do">';
	/* Stores the values of all the global variables in hidden textboxes
	This is done to pass the values to the next Stored Process */
	put '<input type="hidden" name="_program" VALUE="'@;
	put "&prg_name"@;
	put '">';
	prod_type_desc=symget("prod_type_desc");
	put '<input type="hidden" name="prod_type_desc" value="' @;
	put prod_type_desc;
	put '">';
	prod_type_code=symget("prod_type_code");
	put '<input type="hidden" name="prod_type_code" value="' @;
	put prod_type_code;
	put '">';
	model_type_code=symget("model_type_code");
	put '<input type="hidden" name="model_type_code" value="' @;
	put model_type_code;
	put '">';
	include_exp_mdl=symget("include_exp_mdl");
	put '<input type="hidden" name="include_exp_mdl" value="' @;
	put include_exp_mdl;
	put '">';
	model=symget("model");
	put '<input type="hidden" name="model" value="' @;
	put model;
	put '">';
	model_key=symget("model_key");
	put '<input type="hidden" name="model_key" value="' @;
	put model_key;
	put '">';
	variable=symget("variable");
	put '<input type="hidden" name="variable" value="' @;
	put variable;
	put '">';
	variable_key=symget("variable_key");
	put '<input type="hidden" name="variable_key" value="' @;
	put variable_key;
	put '">';
	period=symget("period");
	put '<input type="hidden" name="period" value="' @;
	put period;
	put '">';
	time_key=symget("time_key");
	put '<input type="hidden" name="time_key" value="' @;
	put time_key;
	put '">';
	statistic=symget("statistic");
	put '<input type="hidden" name="statistic" value="'@;
	put statistic;
	put '">';
	model_measure_type=symget("model_measure_type");
	put '<input type="hidden" name="model_measure_type" value="'@;
	put model_measure_type;
	put '">';
	pd_score=symget("pd_score");
	put '<input type="hidden" name="pd_score" value="' @;
	put pd_score;
	put '">';
	range_interval=symget("range_interval");
	put '<input type="hidden" name="range_interval" value="' @;
	put range_interval;
	put '">';
	no_of_quantile=symget("no_of_quantile");
	put '<input type="hidden" name="no_of_quantile" value="' @;
	put no_of_quantile;
	put '">';

	prg_name=symget("prg_name");
	put '<input type="hidden" name="prg_name" value="' @;
	put prg_name;
	put '">';
	
	model_type_desc=symget("model_type_desc");
	put '<input type="hidden" name="model_type_desc" value="' @;
	put model_type_desc;
	put '">';
		
	outcome_period=symget("outcome_period");
	put '<input type="hidden" name="outcome_period" value="' @;
	put outcome_period;
	put '">';

	elapsed_time=symget("elapsed_time");
	put '<input type="hidden" name="elapsed_time" value="' @;
	put  elapsed_time;
	put '">';

	put '<input type="hidden" name="called_program" value="';
	put "&prg_name";
	put '">';
	/**********************/
run;

/*Calls the macro generate_feed_mip for obtaining the required feed by passing
	the following parameters
	1. in_filter		: Where clause (on the basis of the filter clause)
	2. out_lib_nm		: Library name where the output feed will be created. 
	3. out_table_nm_pool: The table name for pool level feed
	4. out_table_nm_acct: The table name for account/application level feed
	5. feed_for			: Possible values are PD, LGD, CCF, APPLPD, APPLLGD. This macro parameter
							decides whether estimated pd/lgd/ccf has to calculated at account 
							level or application level 
	6. var				: The value of the variable_key
	7. model_time_key  	: Contains the concatenated values of model and time in the
							format _<model_sk>_<time_sk>. This is used to identify
							which fact table should be considered while generating 
							the feed.
	8. product_type_code	: This is the selected product type code (E.G. MTG, LON).
*/
%let model_time=_%sysfunc(ktrim(%sysfunc(kleft(&model_key.))))_%sysfunc(ktrim(%sysfunc(kleft(&time_key.))));
%generate_feed_mip(in_filter=&where_feed,out_lib_nm=&out_lib,out_table_nm_pool=&out_simip, 
					out_table_nm_acct=&out_simip_account,feed_for=&feed_for,var=&variable_key,
					model_time_key=&model_time, product_type_code=&prod_type_code);

%let log_name=;
data _NULL_;
	call symput('log_name',"&CS_DEBUG_PATH./&out_simip."||".log");
run;

%debug_log_file(log_file_nm=&log_name);

%let name_html=;
/* Data creation for measure_nm in a dataset and call for feed_measure */
proc sql noprint;
	create table &out_lib..&in_stat_ot as
	select measure_nm, help_file_nm, monotonic() as tot_count 
	from dim.measure_dim
	where kupcase(program_nm)=kupcase("&prg_name") 
	and model_type_cd="&model_measure_type"
	and kupcase(PRODUCT_TYPE_CD)=kupcase("&prod_type_code");
/*	order by measure_nm;*/
quit;

data _null_;
	set &out_lib..&in_stat_ot;
	call symput('name_html',help_file_nm);
run;

/******feed_measure_mip requires 
	   1. where condition on basis of filter clause
       2. out_lib=output liberary  
	   3. Input dataset  
	   4. output dataset(required for statistic overtime graph)**********************/
%feed_measure_mip(in_filter=&where_measure, out_lib_nm=&out_lib., in_table_nm=&in_stat_ot, out_table_nm_measure=&out_stat_ot);

%shift_indices(input_ds=&out_lib..&out_simip,output_ds=&out_lib..score_shift_ds);

/* All other data and macro variable creations (as required in the report) */
%let no_seq=.;
proc sql;
create table &out_lib..stats as select Measure_nm, Measure_value
	from &out_lib..&out_stat_ot
	where upcase(dev_flg)="%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.dev_flag_n.1, noquote))"
	and score_time_sk=&time_key
	order by Measure_nm;
select count(seq_no) into: no_seq from &out_lib..score_shift_ds;
quit;

/***********Attribute proportion  GRAPH PLOT FOR Proportion  *********************/
 axis1 label=(a=90 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_axis1.1, noquote))"); 
 footnote c=black "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote1.1, noquote))" 
	c=&color_symbol1 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote2.1, noquote))"
	c=&color_symbol2 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote3.1, noquote))"; 
 axis2 order=1 to &no_seq. by 1 
label=("%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote4.1, noquote))"); 

proc gplot data=&out_lib..score_shift_ds;
	symbol2 color=&color_symbol2 width=.15 height=2 value=dot interpol=join;
	symbol1 color=&color_symbol1 width=.15 height=2 value=dot interpol=join;
	plot prop_actual*SEQ_NO	
	     	prop_dev*SEQ_NO/ haxis=axis2 vaxis=axis1  overlay 
	  name="&s_i1";
run;
quit;

/********************Event proportion  GRAPH PLOT FOR NON Proportion ******************/
 axis1 label=(a=90 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_axis1.1, noquote))"); 
footnote 
	c=black "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote1.1, noquote))" 
	c=&color_symbol1 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote2.1, noquote))"  
	c=&color_symbol2 "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote3.1, noquote))"; 
 axis2 order=1 to &no_seq. by 1 
	label=("%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_footnote4.1, noquote))");     	
proc gplot data=&out_lib..score_shift_ds;
	plot prop_evt_act *SEQ_NO
     prop_evt_dev*SEQ_NO	/ haxis=axis2  vaxis=axis1  overlay 
	name="&s_i2";
run;
quit;

/*****Input having statistic name  
	1.data set as(work.datset)   
	2.Model type code for 
		eg.PDA which will be passed from calling stored process
*****output will be the color 
   		for that value and statistic we will collect that in 'color' parameter***************/
%let color=;
%model_traffic (data1=&out_lib..stats,model_cd=&model_measure_type, product_type_code=&prod_type_code);

/* Creation of all graphs and call to overtime macro */
/*******Macro statistic_overtime parameter passed
      	1.Dataset this dataset will be ouput dataset of Macro feed_measure
	  	2.statistic (For Eg. D Statistic etc.)
		3.graph name (generated from timestamp)******/
/*%statistic_overtime(&out_lib..&out_stat_ot,&statistic,&otname);*/
%let fst_graph=;
%let cnt_stat=.;
%macro name();
	proc sql noprint;
		select tot_count into :fst_cnt
		from &out_lib..&in_stat_ot
		where measure_nm=put("&statistic",&format_name..);
	quit;

	proc sql noprint;
		select count(*) into :cnt_stat from &out_lib..&in_stat_ot;
		%let cnt_stat=%sysfunc(ktrim(%sysfunc(kleft(&cnt_stat))));
		select measure_nm into :statistic1 - :statistic&cnt_stat
		from &out_lib..&in_stat_ot;
	quit;
	%do indx=1 %to &cnt_stat;
		data _NULL_;
			call execute('%global ' || ktrim("graph_nm&indx"));
		run;
		%let graph_nm&indx =%sysfunc(ktrim(s&indx&graph_name));
		%statistic_overtime(&out_lib..&out_stat_ot,&&statistic&indx,&&graph_nm&indx);
	%end;
	%let fst_cnt=%sysfunc(ktrim(%sysfunc(kleft(&fst_cnt))));
	%let fst_graph = &&graph_nm&fst_cnt;
	%put &fst_graph;
%mend name;
%name();

%let fst_graph=%sysfunc(ktrim(%sysfunc(kleft(&fst_graph))));

%debug_log_web;

/* All data _null_ statements to create the html */
%let view_feed=&out_simip..html;
data _null_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
	file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/
	rep="&statistic";
	format rep &format_name..;
	name_html=symget("name_html");
	put'<script>';
		/**************Help File *********************/
	%helpfile(filename=&name_html);

		/*************feed Table*********************/
	put 'function feed_table(){';
		put'window.open("/SASCSReports/'@; 
		put "&view_feed"@; 
	put '")};';
			
		/*********************************************/
	put'</script>';

	put '  <table width="100%"  border="0" cellpadding="0" cellspacing="0" bgcolor="#FAF8ED">';
	/* To display SAS credit scoring as heading */
	put '<tr>';
	put '<td align="left">';
	put '<table width="100%"  border="0" align="left" cellpadding="3" cellspacing="0">';
	put '<tr>';
	put '<td><table width="100%"  border="0" cellspacing="0" cellpadding="0">';

	put '<tr>';
	put '<td><span class="BreadCrumb">'@;
	put '<a href="javascript:Link1()">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.linnk_reporting.1, noquote))"@;
	put '</a> &gt; ';
	put '<a href="javascript:Link2()">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.linnk_mon_model_input.1, noquote))"@;
	put '</a> &gt; ';
	put '<a href="javascript:Link3()">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.linnk_sel_var.1, noquote))"@;
	put '</a> &gt;'@;
/*	put' </span>';*/
	put '</span><span class="BreadCrumb">'@;
	put rep @;
	put'</span></td>';

	put '<td width="215"><div align="right"><img src="/SASCSReports/images/CrBanner.gif" width="259" height="29"></div></td>';
	put '</tr></table>';

	/* Heading ends */
	put '<H2 align="left"><table width="100%"  border="0" cellspacing="0" cellpadding="0"><tr><td>';
	put rep @;
	put '</td>';
	if kupcase(&print) eq 'Y' then do;
		put '<td width=65><div align="center"><span class="LabelFont">';
		put '<a href="javascript:printpage()">';
		put '<img src="/SASCSReports/images/Print.gif" border="0"></a>'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.report_print.1, noquote))"@;
		put '</div></td>';
	end;
	if kupcase(&save) eq 'Y' then do;
		put '<td width=50><div align="center"><span class="LabelFont">';
		put '<a href="javascript:enregistre()">';
		put '<img src="/SASCSReports/images/SaveFile.gif" border="0"></a>'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.report_save.1, noquote))"@;
		put '</div></td>';
	end;
	if kupcase(&help) eq 'Y' then do;
		put '<td width=50><div align="center"><span class="LabelFont">';
		put '<a href="javascript:Info()">';
		put '<img src="/SASCSReports/images/Info.gif" border="0"></a>'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.report_help.1, noquote))"@;
		put '</div></td>';
	end;
	put '</tr></table></H2>';
	put '</td></tr>';

	put '<tr>';
	put '<td align="left">';
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_model_type.1, noquote))"@;
	put "&model_type_desc"@;
	put '</p>';
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_product_type.1, noquote))"@;
	put "&prod_type_desc";
	put '</p> ';
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_period.1, noquote))"@;
/*I18N-scnhuh-20080427-1455*/	
/* del one line, add two lines. For displaying in NLS format */	
/*	put "&period" @;*/
		Period_WEBUI=put(input("&period",&CS_TIME_PER_FMT.),&NLDATE_FMT.);
		put Period_WEBUI @;
/*I18N-scnhuh-20080427-1455*/		
	put '</p> ';
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_model.1, noquote))"@;
	put "&model";
	put '</p> ';
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_variable.1, noquote))"@;
	put "&variable";
	put '</p> ';

	/* Target period and elapsed time*/
	put '<p class="LabelFont">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_outcome_period.1, noquote))"@;
	put ': '@;
	put "&outcome_period" @;
	put ' '@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_months.1, noquote))"@;
	put '  '@;
	put "&elapsed_time" @;
	put '  '@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_month_elapsed.1, noquote))"@;
	put '</p>';

	/* Number of months of Historic data */
	if kupcase("&model_type_code") eq kupcase(&PD_APP_MODEL.) then do;
		put '<p class="LabelFont">'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.pd_stored_proc_hist.label_historic_data.1, noquote))"@;
		if kupcase("&prod_type_code") eq kupcase(&MTG_PRD_TYPE.) then do;
			put "&PDA_MTG_HIST" @;
		end;
		if kupcase("&prod_type_code") eq kupcase(&LON_PRD_TYPE.) then do;
			put "&PDA_LON_HIST" @;
		end;
		if kupcase("&prod_type_code") eq kupcase(&COR_PRD_TYPE.) then do;
			put "&PDA_COR_HIST" @;
		end;
		if kupcase("&prod_type_code") eq kupcase(&CCD_PRD_TYPE.) then do;
			put "&PDA_CCD_HIST" @;
		end;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.pd_stored_proc_month.label_months.1, noquote))"@;
		put '</p>';
	end;

run;

data _null_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
	file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/
	set &out_lib..stats end=last;
	format mv &CS_DEC_FMT.;
	format mdlv &CS_DEC_FMT.;
	mv=measure_value;
	mdlv=mdl_measure_value;
	if _N_=1 then
		put '<table  border="0" cellspacing="0" cellpadding="0"> ';
/*************************VALUES SHOWN FOR ACTUAL AND MODELING***************************************************/
	put'<tr> ';
	put'<td><span class="LabelFont">';
	put measure_nm;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.label_actual_val.1, noquote))"@;
	put ':  '@;
	put mv;
	put'</span></td><td>&nbsp;</td> ';

/* To be used by calling a macro */
	put'<td width="25"><div align="center"><span class="LabelFont"><img src="/SASCSReports/images/';
	if color="%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.colour_white.1, noquote))" then do;
		put 'spacer.gif"';
	end;
	else do;
		put 'Sphere';
		put color;
		put'.gif"';
	end;

	put 'width="16" height="16" border="0"></a> </span></div></td> ';
	put'</tr> ';
	if last then do;
		put'</table>';
		put '<p class="LabelFont">&nbsp;</p>';
	end;
run;

/***Macro Draw table 
	1.Input data set
	2.column name of passed data set,this is defined above as col_name where name of column 
	  are provide this is  compulsary to be given by the user while defining col_names;
	3.column lables of the datset ,this is defined as col_lables above this feild might be 
	  empty for in that case  column lables should be there in the dataset	 	 
********************************/
 

data &out_lib..score_shift_ds; 
set &out_lib..score_shift_ds;
format scorecard_points;
scorecard_points=round(scorecard_points);
run;

%draw_table(ds=&out_lib..score_shift_ds,col_names=&colnames,col_labels=&collabels);
data _NULL_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
	file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/
	/****************************************************************************/
	put '<tr>';
	put '<td align="left">';
	put '<p>';
	put '<H2 align="left">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.graph_graph.1, noquote))"@;
	put '</H2>';
	put '</td></tr>';
	put	'<tr><td><table width="100%"  border="0" cellspacing="0" cellpadding="0">';
    put '<tr>';
	put '<td width="49%" valign="top"><fieldset> ';
	put '<legend><span class="style2">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_proportion.1, noquote))"@;
	put '<input name="radiobutton" type="radio" value="/SASCSReports/';
	put "&s_i1"@;
	put '.gif" onclick="swapIt(this.value)"> ';
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_attrib_proportion.1, noquote))"@;
	put '&nbsp;&nbsp;<input name="radiobutton" type="radio"  value="/SASCSReports/';
	put "&s_i2"@;
	put'.gif" onclick="swapIt(this.value)" checked> ';
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.shift_index.graph_event_proportion.1, noquote))"@;
	put '</span></legend> ';
	put '<p><img border="0" src="/SASCSReports/';
	put "&s_i2"@; 
	put '.gif" width="460" height="230"  name="Cumulative"></p>';
	put '</fieldset></td> ';
run;

%let count=1;
%macro over_time();
	data _NULL_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
		file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/
		rep="&statistic";
	/* Code for displaying overtime graph.
	This will get displayed only if filter is set to NO */
		set &out_lib..&in_stat_ot end=last;
		%do %while(&count le &cnt_stat);
			if &count eq _N_ then do;
				if _N_= 1 then do;
					put '<td width="2%">&nbsp;</td>';
			        put '<td width="49%" valign="top"><fieldset>';
			        put '<legend><span class="style2">';
					put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.graph_over_time.1, noquote))"@;
					put '</span></legend>';
					put '<p><select name="measure_name" onChange="javascript:display()">';
				end;
				put "<option value='/SASCSReports/&&graph_nm&count...gif'";
				if measure_nm =kupcase(put(rep,&format_name..)) then do;
					put 'selected';
				end;
				put '>';
				put measure_nm;
				put '</option>';
				if last then do;
					put '</select></p>';
				    put '<p><img name=my_img src="/SASCSReports/'@;
					put "&fst_graph";
					put '.gif" width="460" height="200"></p>';
				    put '</fieldset></td>';
			        put '</tr></table>';
				end;
			end;
			%let count=%eval(&count+1);
		%end;
	run;
%mend over_time;
%over_time();

data _NULL_;
/*I18N-scnhuh-20080422-1955*/
/*add &webout_options*/
	file _webout &webout_options;
/*I18N-scnhuh-20080422-1955*/

	put '<p>&nbsp;</p></td> ';
	put '</tr> ';
	put '</table> ';
	put '</td> ';
	put '</tr> ';
	put '<tr> ';
	put '<td height="32" bgcolor="#F1EACB">&nbsp;&nbsp;';
	put '<INPUT type=button class=mmhide_btn onClick="feed_table()" value="'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.info_val.button_view_feed.1, noquote))"@;
	put '">';
	put '</td> ';
	put '</tr> ';
	put '</table> ';
	put '</form> ';
	put '</body> ';
	put '</html> ';
run;
/******macro feed_table pass parameter  separated by #

		Input parameter 
		1.dt_st =Pass dataset name in string separated by #
		2.dt_nm=pass name u want to give to the table
		3.out_libout library  which is defined above
		        *****************************************************************/

	 /*I18N-scnjih-20080421-1517*/
/*%let dt_nm=Pool Feed;*/
%let dt_nm=%sysfunc(sasmsg(smd_ds.bismsg,Call_feed_table.sas.dt_nm.1, noquote));
	 /*I18N-scnjih-20080421-1517*/
%let dt_st=&out_simip;
%feed_table(ds=&dt_st,out_lib1=&out_lib,data_name=&dt_nm,name_html=&view_feed);

goptions reset=all;
options nomprint;
