/******************************************************************************************************
       Module:  build_pool_abt

     Function:  To build the data for the scheme in the background 

      Authors:  BIS Team
         Date:  14 May 2007
          SAS:  9.1
    Called-by:	build_scheme_data
        Calls:  macro build_all_abt

Datasets used:  1. pool.pool_scheme_defn: Retrieves the defn of existing pool schemes and saves the defn
					for new schemes in this dataset
				2. dim.analytical_model_dim: Retrieves the PD, LGD and CCF models from this
					dataset
				3. pool.build_abt_status: Stores the status of the building activity of the various ABTs
					to be built for the pool scheme
				4. The scored datasets _<PD_MODEL_RK>_PD, _<LGD_MODEL_RK>_LGD, _<CCF_MODEL_RK>_CCF
				5. The long run datasets _LNG_<PD_MODEL_RK>_PD, _LNG_<PD_MODEL_RK>_PD_LGDYRS, 
					_LNG_<LGD_MODEL_RK>_LGD, _LNG_<CCF_MODEL_RK>_CCF, _LNG_<CCF_MODEL_RK>_CCF_LGDYRS
				6. dim.default_event_dim: Retrieve the default information from this dataset
				7. dim.account_dim: Retrieve the account information from this dataset
				8. fact.account_snapshot: Retrieve the balance information from this dataset
				9. fact.recovery_fact: Retrieve the loss information from this dataset
 
        Logic:  1. Determine the name of the scored abts
				2. Determine the name and dates of the longrun abts
				3. If the scheme is being build for the first time, then determine all the ABTs to 
					be built. The ABTs to be built are stored in the temporary dataset Build_abt_status.
				4. Check the scheme type. According to scheme type, build the scoring ABTs. 
					- For Score ranges, PD ranges and Map PD to master scale based scheme types, 
						build the ABT for the PD model
					- For PD and LGD ranges based scheme types, build the ABTS for the PD and LGD models
					- For PD, LGD and CCF ranges based scheme types, build the ABTS for the PD, LGD 
						and CCF models
				5. For long run ABTs, insert one record for each year that the ABT needs to be built 
					in the dataset Build_abt_status.
				6. Build all the ABTs by running the ETL jobs to build the aggregate tables and then 
					join them to create the required ABT.
				7. Load the information into the scored and long run datasets. And update the actual 
					information for the long run tables at the end of that year.
				
   Parameters:  scheme_no 		: The unique identifier of an existing pool scheme
   				scheme_type		: Pool scheme type. For e.g., SCR, PDR, PLR, PLC, PMS
				pd_method 		: The assignment method set for a PD model. For e.g., AVG, LRM, LRA
				lgd_method 		: The assignment method set for a LGD model. For e.g., AVG, LRM, LRA
				ccf_method 		: The assignment method set for a CCF model. For e.g., AVG, LRM, LRA
				seg_method 		: The assignment method set for a Segmentation model. For e.g., LRM, LRA
				pd_model_key 	: The retained key for the selected PD model
				lgd_model_key 	: The retained key for the selected LGD model
				ccf_model_key 	: The retained key for the selected CCF model
				seg_model_key 	: The retained key for the selected Segmentation model
				obs_dt 			: The date of data capture
				action 			: Screen action code
   		
 ******************************************************************************************************/
options mlogic mprint;

/*This code is Internationalized*/
/* Global Input Parameters */
%global scheme_no scheme_type pd_model_key pd_method lgd_model_key lgd_method ccf_model_key 
	ccf_method obs_dt action;
	%put &obs_dt.;

%let lib=pool;

%let error_code=NONE;
/*In few stored process the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
/* Setting output library based on the debug flag */
%let out_lib=;
%debug_lib_path;

/*Proc printto will redirect the log to the directory
"Data/bisdata/log". */
%global log_name;

data _NULL_;	
name=put(time(),mmss8.1);	
name1=kscan(name,1,":.");	
name2=kscan(name,2,":.");	
name3=kscan(name,3,":.");	
name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
call symput('log_name',"Data\bisdata\log\"||"build_pool_abt_"||ktrim(kleft(name4))||".log");
run;
%put NOTE: Sending log messages to :&log_name;
proc printto log="&log_name";
run;

%let action=%sysfunc(ktrim(%sysfunc(kleft(&action))));

/*options mprint;*/
%global create_error_cd;
%let create_error_cd=0;
%macro chk_error(return_err);
	%if &return_err. > &create_error_cd. %then %do;
		%let create_error_cd = &return_err.;		
	%end;
%mend chk_error;
%macro build_data;
	%let pd_model_key=%sysfunc(ktrim(%sysfunc(kleft(&pd_model_key))));
	%let lgd_model_key=%sysfunc(ktrim(%sysfunc(kleft(&lgd_model_key))));
	%let ccf_model_key=%sysfunc(ktrim(%sysfunc(kleft(&ccf_model_key))));

	/* Get date of building the ABT for naming the scored ABT */
	data _null_;
		dt=&obs_dt;
		mth=put(month(dt),z2.);
		yr=year(dt);
		dt1=yr||mth;
		call symput('data_dt', dt1);
	run;	
	%let data_dt=%sysfunc(ktrim(%sysfunc(kleft(&data_dt))));

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	/* Determine the name of the scored abts */
	%let pd_ds=_PD_&data_dt.;
	%let scr_pd_ds=_&pd_model_key._PD_&data_dt.;
		
	%let lgd_ds=_LGD_&data_dt.;
	%let scr_lgd_ds=_&lgd_model_key._LGD_&data_dt.;

	%let ccf_ds=_CCF_&data_dt.;
	%let scr_ccf_ds=_&ccf_model_key._CCF_&data_dt.;

	/* Determine the name and dates of the longrun abts */
	%let pd_method_type=%quote(%'&pd_method%');
	%let lgd_method_type=%quote(%'&lgd_method%');
	%let ccf_method_type=%quote(%'&ccf_method%');
	%let longrun_pd_ds=;
	%let longrun_lgd_ds=;
	%let longrun_ccf_ds=;

	/* Determine the name of the long run abts */
	%let lng_pd_ds=_lng_PD;
	%let lng_lgd_ds=_lng_LGD;
	%let lng_ccf_ds=_lng_CCF;

	%let start_dt=;
	/* Determine the name of the long run tables */
	%if &lgd_method_type eq &LONG_RUN_MAX_TYPE or &lgd_method_type eq &LONG_RUN_AVG_TYPE %then %do;
		%let longrun_pd_ds=_lng_&pd_model_key._PD_LGDYRS;
		%let longrun_lgd_ds=_lng_&lgd_model_key._LGD;
		%let longrun_ccf_ds=_lng_&ccf_model_key._CCF_LGDYRS;
		%let start_dt=&obs_dt;

		data _null_;
			start_dt=intnx('year', &obs_dt, -(&RCV_PERIOD_MONTH_CNT), 'begin');
			call symput('start_dt', start_dt);
		run;

		%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
	%end;
	%else %do;
		%let longrun_pd_ds=_lng_&pd_model_key._PD;
		%let longrun_lgd_ds=_lng_&lgd_model_key._LGD;
		%let longrun_ccf_ds=_lng_&ccf_model_key._CCF;
		%let start_dt=&obs_dt;
	%end;

	/* Determine years of data to build */
	data &out_lib..longrun_dts;
		length dt 8.;
		stop;
	run;

	%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

	%do cnt=1 %to &LONG_RUN_YRS;
		data _null_;
			dt=intnx('year', &start_dt, -(&cnt), 'begin');
			call symput('abt_data_dt', dt);
		run;

		%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

		proc sql;
			insert into &out_lib..longrun_dts values(&abt_data_dt.); 
		quit;

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	%end;

	/* Take the years of data into macro variables */
	proc sql noprint;
		select dt into :lng_dt1-:lng_dt&LONG_RUN_YRS
		from &out_lib..longrun_dts;
	quit; 

	%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

	%let create_lng_pd=N;
	%let create_lng_lgd=N;
	%let create_lng_ccf=N;

	%let create_scr_pd=N;
	%let create_scr_lgd=N;
	%let create_scr_ccf=N;

	/* If action is restart */
	%if %str(&action) eq %str(R) %then %do;
		/* Action = Restart */
		%if %sysfunc(exist(&lib..build_abt_status)) eq 0 %then %do;
			proc sql noprint;
				select count(*) :no_scheme
				from &lib..build_abt_status
				where pool_scheme_no = &scheme_no;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

			%let no_scheme=%sysfunc(ktrim(%sysfunc(kleft(&no_scheme))));
			%if &no_scheme > 0 %then %do;
				proc sql noprint;
					select count(*) :no_failed
					from &lib..build_abt_status
					where pool_scheme_no = &scheme_no and status_desc ne "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.status_desc_completed.1, noquote))";
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				%let no_failed=%sysfunc(ktrim(%sysfunc(kleft(&no_failed))));
				%if &no_failed > 0 %then %let action=R;
				%else %let action=S;
			%end;
			%else %let action=B;
		%end;
	%end;
	/* If action is build */
	%if %str(&action) eq %str(B) %then %do;
		/* Action = Build new scheme */

		/* Determine all the scored ABTs and long run ABTs to be built by loading the
			table with the details */
		%if %sysfunc(exist(&lib..build_abt_status)) eq 0 %then %do; 
			/* If table does not exist, create it */
			data &lib..build_abt_status;
				length ABT_NO 8.;
				length POOL_SCHEME_NO 8.;
				length ABT_PHY_DS $40;
				length SCR_ABT_PHY_DS $40;
				length COLS_DS $40;
				length MODEL_CODE $100;
				length MODEL_KEY 8.;
				length MODEL_NM $40.;
				length ABT_DT 8.;
				format ABT_DT nldate10.;
				length MODEL_TYPE_CD $3;
				length METHOD_TYPE_CD $3;
				length STATUS_DESC $40;
				stop;
			run;

			%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
		%end;

		%let abt_cnt=0;

		/* Check scheme type. According to scheme type, build scoring ABTs
			For scheme types SCR, PDR, PMS, build for PD
			For scheme types PLR, build for PD and LGD 
			For scheme types PLC, build for PD, LGD and CCF 
		*/
		%let sch_type=%quote(%'&scheme_type%');

		/* For all cases build PD abt and score it */

		/* If the scored abt exists do not build, else build the scored abt */
		%put &lib..&scr_pd_ds.;
		%if %sysfunc(exist(&lib..&scr_pd_ds.)) eq 0 %then %do; 
			%let create_scr_pd=Y;			
			%let abt_cnt=%eval(&abt_cnt+1);
			proc sql;
				insert into &lib..build_abt_status
					(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
				VALUES (&abt_cnt, &scheme_no, "abtscr.&pd_ds", "abtscr.&scr_pd_ds", &pd_model_key, &obs_dt, 
						"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_pd.1, noquote))",
						"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))" 
						 '');
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;

		%if &sch_type eq &PD_LGD_RANGE_TYPE or 
			&sch_type eq &PD_LGD_CCF_RANGE_TYPE %then %do;
			/* Build LGD abt and score it */
			/* If the scored abt exists do not build, else build the scored abt */
			%if %sysfunc(exist(&lib..&scr_lgd_ds.)) eq 0 %then %do; 
				%let create_scr_lgd=Y;			
				%let abt_cnt=%eval(&abt_cnt+1);
				proc sql;
					insert into &lib..build_abt_status
						(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
					VALUES (&abt_cnt, &scheme_no, "abtscr.&lgd_ds", "abtscr.&scr_lgd_ds", &lgd_model_key, &obs_dt, 
							"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_lgd.1, noquote))", 
							"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))" 
							 '');
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		%if &sch_type eq &PD_LGD_CCF_RANGE_TYPE %then %do;
			/* Build CCF abt and score it */
			/* If the scored abt exists do not build, else build the scored abt */
			%if %sysfunc(exist(&lib..&scr_ccf_ds.)) eq 0 %then %do; 
				%let create_scr_ccf=Y;			
				%let abt_cnt=%eval(&abt_cnt+1);
				proc sql;
					insert into &lib..build_abt_status
						(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
					VALUES (&abt_cnt, &scheme_no, "abtscr.&ccf_ds", "abtscr.&scr_ccf_ds", &ccf_model_key, &obs_dt, 
							"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_ccf.1, noquote))", 
							"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))" 
							'');
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		/* Check PD, LGD, CCF method. If any of them are long run, build the long run table */
		/* PD Longrun */
		%put ****************PD Method : &pd_method_type******************;
		%if &pd_method_type eq &LONG_RUN_MAX_TYPE or 
			&pd_method_type eq &LONG_RUN_AVG_TYPE %then %do;

			/* If the long run table exists for the required years do not build, else build the long run table */
			%let match_yrs=0;
			%if %sysfunc(exist(longrun.&longrun_pd_ds.)) eq 1 %then %do; 
				proc sql noprint;
					select count(*) into :match_yrs
					from longrun.&longrun_pd_ds., &out_lib..longrun_dts
					where year_start_dt=dt;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
			%let match_yrs=%sysfunc(ktrim(%sysfunc(kleft(&match_yrs))));
			%if %sysfunc(exist(longrun.&longrun_pd_ds.)) eq 0 or &match_yrs ne &LONG_RUN_YRS %then %do;
				%let create_lng_pd=Y;
				%do cnt=1 %to &LONG_RUN_YRS;
					%let abt_cnt=%eval(&abt_cnt+1);
					proc sql;
						insert into &lib..build_abt_status
							(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
						VALUES (&abt_cnt, &scheme_no, "abtscr.&lng_pd_ds.&cnt.", "abtscr.&longrun_pd_ds.&cnt.", &pd_model_key, &&lng_dt&cnt., 
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_pd.1, noquote))",
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))",
								'');
					quit;

					%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
				%end;
			%end;
		%end;

		/* LGD longrun */
		%put ****************LGD Method : &lgd_method_type******************;
		%if &lgd_method_type eq &LONG_RUN_MAX_TYPE or 
			&lgd_method_type eq &LONG_RUN_AVG_TYPE %then %do;
			/* If the long run table exists for the required years do not build, else build the long run table */
			%let match_yrs=0;
			%if %sysfunc(exist(longrun.&longrun_lgd_ds.)) eq 1 %then %do; 
				proc sql noprint;
					select count(*) into :match_yrs
					from longrun.&longrun_lgd_ds., &out_lib..longrun_dts
					where year_start_dt=dt;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
			%let match_yrs=%sysfunc(ktrim(%sysfunc(kleft(&match_yrs))));
			%if %sysfunc(exist(longrun.&longrun_lgd_ds.)) eq 0 or &match_yrs ne &LONG_RUN_YRS %then %do;
				%let create_lng_lgd=Y;
				%do cnt=1 %to &LONG_RUN_YRS;
					%let abt_cnt=%eval(&abt_cnt+1);
					proc sql;
						insert into &lib..build_abt_status
							(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
						VALUES (&abt_cnt, &scheme_no, "abtscr.&lng_lgd_ds.&cnt.", "abtscr.&longrun_lgd_ds.&cnt.", &lgd_model_key, &&lng_dt&cnt., 
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_lgd.1, noquote))",
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))",
								'');
					quit;

					%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
				%end;
			%end;
		%end;

		/* CCF longrun */
		%put ****************CCF Method : &ccf_method_type******************;
		%if &ccf_method_type eq &LONG_RUN_MAX_TYPE or 
			&ccf_method_type eq &LONG_RUN_AVG_TYPE %then %do;
			/* If the long run table exists for the required years do not build, else build the long run table */
			%let match_yrs=0;
			%if %sysfunc(exist(longrun.&longrun_ccf_ds.)) eq 1 %then %do; 
				proc sql noprint;
					select count(*) into :match_yrs
					from longrun.&longrun_ccf_ds., &out_lib..longrun_dts
					where year_start_dt=dt;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
			%let match_yrs=%sysfunc(ktrim(%sysfunc(kleft(&match_yrs))));
			%if %sysfunc(exist(longrun.&longrun_ccf_ds.)) eq 0 or &match_yrs ne &LONG_RUN_YRS %then %do;
				%let create_lng_ccf=Y;
				%do cnt=1 %to &LONG_RUN_YRS;
					%let abt_cnt=%eval(&abt_cnt+1);
					proc sql;
						insert into &lib..build_abt_status
							(ABT_NO, POOL_SCHEME_NO, ABT_PHY_DS, SCR_ABT_PHY_DS, MODEL_KEY, ABT_DT, MODEL_TYPE_CD, METHOD_TYPE_CD, STATUS_DESC)
						VALUES (&abt_cnt, &scheme_no, "abtscr.&lng_ccf_ds.&cnt.", "abtscr.&longrun_ccf_ds.&cnt.", &ccf_model_key, &&lng_dt&cnt., 
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_ccf.1, noquote))",
								"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))",
								'');
					quit;

					%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
				%end;
			%end;
		%end;

		/* Update table with model_names */
		proc sql;
			update &lib..build_abt_status set MODEL_NM=(select model_nm from 
				dim.analytical_model_dim where 
				analytical_model_dim.model_rk=build_abt_status.model_key and 
				analytical_model_dim.valid_end_dttm=&valid_high_dttm) 
			where pool_scheme_no=&scheme_no;
		quit;	

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
	%end;

	%if %str(&action) eq %str(R) or %str(&action) eq %str(B) %then %do;
		/* Action = Restart or Build */
		/* Builds and scores all the ABTs listed in the dataset Build_abt_status.
		   Parameters:  repository	: The metadata repository to get model information from
						scheme_no	: The scheme identifier for which the ABTs have to be built
						in_ds		: The dataset containing the list of ABTs to be built
						work_lib	: The library which should be used for temporary calculations
						action		: B - Build Data
									  R - Restart from falied ABT
		*/
		%build_all_abt(repository=Foundation, scheme_no=&scheme_no, in_ds=&lib..build_abt_status, 
			work_lib=&out_lib, action=&action);

		%let action=S;
	%end;

	%if %str(&action) eq %str(S) %then %do;
		/* Action = Score */
		/* Build the scored tables. The scored tables are named as follows:
			_<PD_MODEL_KEY>_PD_<YYYYMM>
			_<LGD_MODEL_KEY>_LGD_<YYYYMM>
			_<CCF_MODEL_KEY>_CCF_<YYYYMM>
		*/
		%if %str(&create_scr_pd) eq %str(Y) %then %do;
			%let tab=;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
					strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_pd.1, noquote))" and 
						  METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))";

				%if %str(&tab) ne %str() %then %do;
					data &lib..&scr_pd_ds.;
						set &tab(keep=ACCOUNT_RK SCORECARD_POINTS P_TGT_VAR1 
							rename=(P_TGT_VAR1=ESTIMATED_PD SCORECARD_POINTS=SCORE_POINTS));
					run;
				%end;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;
		
		%if %str(&create_scr_lgd) eq %str(Y) %then %do;
			%let tab=;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
					  strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_lgd.1, noquote))" and 
						METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))";

				%if %str(&tab) ne %str() %then %do;
					data &lib..&scr_lgd_ds.;
						set &tab(keep=ACCOUNT_RK P_TGT_VAR rename=(P_TGT_VAR=ESTIMATED_LGD));
					run;
				%end;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;

		%if %str(&create_scr_ccf) eq %str(Y) %then %do;
			%let tab=;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
						strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_ccf.1, noquote))" and 
							  METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_scr.1, noquote))";

				%if %str(&tab) ne %str() %then %do;
					data &lib..&scr_ccf_ds.;
						set &tab(keep=ACCOUNT_RK P_TGT_VAR rename=(P_TGT_VAR=ESTIMATED_CCF));
					run;
				%end;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
		%end;

		/* In case of longrun, load into long run table. The longrun tables are named as follows:
			_LNG_<PD_MODEL_KEY>_PD
			_LNG_<PD_MODEL_KEY>_PD_LGD_YRS
			_LNG_<LGD_MODEL_KEY>_LGD
			_LNG_<CCF_MODEL_KEY>_CCF
			_LNG_<CCF_MODEL_KEY>_CCF_LGDYRS
		*/

		/* There is a temp dataset with the year start date and account information which will 
		be created as required if create_lng_pd / create_lng_lgd / create_lng_ccf is Y. If it
		exists at this stage, delete it */
		proc sql;
			drop table &out_lib..temp_actual;
			create table &out_lib..temp_actual 
				(
				   YEAR_START_DT num format=NLDATE10.,
				   ACCOUNT_RK num
				);
		quit;

		%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

		/* If long run abts for pd were created, then create long run table for pd */
		%if %str(&create_lng_pd) eq %str(Y) %then %do;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab1-:tab&LONG_RUN_YRS.
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
					strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_pd.1, noquote))" and 
						METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))";

				%if &lgd_method_type eq &LONG_RUN_MAX_TYPE or 
					&lgd_method_type eq &LONG_RUN_AVG_TYPE %then %do;
					create table longrun.&longrun_pd_ds.
					  (
					   YEAR_START_DT num format=NLDATE10.,
					   ACCOUNT_RK num,
					   ESTIMATED_PD num,
					   ACTUAL_GOOD_BAD num, 
					   EXPOSURE_AT_DEFAULT num, 
					   ACTUAL_LOSS num, 
					   SCORE_POINTS num,
					   BALANCE_AT_START num, 
					   BALANCE_AT_DEFAULT num, 
					   LIMIT_AT_START num,
					   LIMIT_AT_DEFAULT num 
					  );
				%end;
				%else %do;
					create table longrun.&longrun_pd_ds.
					  (
					   YEAR_START_DT num format=NLDATE10.,
					   ACCOUNT_RK num,
					   ESTIMATED_PD num,
					   ACTUAL_GOOD_BAD num, 
					   SCORE_POINTS num,
					   BALANCE_AT_START num, 
					   BALANCE_AT_DEFAULT num, 
					   LIMIT_AT_START num, 
					   LIMIT_AT_DEFAULT num 
					  );
				%end;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

			%do cnt=1 %to &LONG_RUN_YRS;
				proc sql;
					create table &out_lib..temp_pd as
					select ACCOUNT_RK, &&lng_dt&cnt. as YEAR_START_DT, 
					p_tgt_var1 as ESTIMATED_PD, SCORECARD_POINTS as SCORE_POINTS
					from &&tab&cnt.
					order by YEAR_START_DT, ACCOUNT_RK;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				/* Load data into long tun table */
				proc append data=&out_lib..temp_pd base=longrun.&longrun_pd_ds.;
				quit;
				
				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

				/* Create temp data of year start date and account to get actuals information */
				proc sql;
					insert into &out_lib..temp_actual
					select YEAR_START_DT, ACCOUNT_RK
					from &out_lib..temp_pd ;
				quit;

				proc sort data=&out_lib..temp_actual;
					by YEAR_START_DT  ACCOUNT_RK;
				run;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		/* If long run abts for lgd were created, then create long run table for lgd */
		%if %str(&create_lng_lgd) eq %str(Y) %then %do;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab1-:tab&LONG_RUN_YRS.
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
						strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_lgd.1, noquote))" and 
						METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))";

				create table longrun.&longrun_lgd_ds.
				  (
				   ACCOUNT_RK num,
				   YEAR_START_DT num format=NLDATE10.,
				   ESTIMATED_LGD num,
				   EXPOSURE_AT_DEFAULT num, 
				   ACTUAL_LOSS num 
				  );
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

			%do cnt=1 %to &LONG_RUN_YRS;
				proc sql;
					create table &out_lib..temp_lgd as
					select ACCOUNT_RK, &&lng_dt&cnt. as YEAR_START_DT, p_tgt_var as ESTIMATED_LGD
					from &&tab&cnt.
					order by YEAR_START_DT, ACCOUNT_RK;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				/* Load data into long tun table */
				proc append data=&out_lib..temp_lgd base=longrun.&longrun_lgd_ds.;
				quit;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		/* If long run abts for ccf were created, then create long run table for ccf */
		%if %str(&create_lng_ccf) eq %str(Y) %then %do;
			proc sql noprint;
				select SCR_ABT_PHY_DS into :tab1-:tab&LONG_RUN_YRS.
				from &lib..build_abt_status 
				where POOL_SCHEME_NO=&scheme_no and 
						strip(MODEL_TYPE_CD)="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.model_type_cd_ccf.1, noquote))" and 
							  METHOD_TYPE_CD="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.method_type_cd_lng.1, noquote))" ;

				%if &lgd_method_type eq &LONG_RUN_MAX_TYPE or 
					&lgd_method_type eq &LONG_RUN_AVG_TYPE %then %do;
					create table longrun.&longrun_pd_ds.
					  (
					   YEAR_START_DT num format=NLDATE10.,
					   ACCOUNT_RK num,
					   ESTIMATED_CCF num
					  );
				%end;
				%else %do;
					create table longrun.&longrun_ccf_ds.
					  (
					   YEAR_START_DT num format=NLDATE10.,
					   ACCOUNT_RK num,
					   ESTIMATED_CCF num,
					   BALANCE_AT_START num,
					   BALANCE_AT_DEFAULT num, 
					   LIMIT_AT_START num,
					   LIMIT_AT_DEFAULT num 
					  );
				%end;
			quit;

			%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

			%do cnt=1 %to &LONG_RUN_YRS;
				proc sql;
					create table &out_lib..temp_ccf as
					select ACCOUNT_RK, &&lng_dt&cnt. as YEAR_START_DT, p_tgt_var as ESTIMATED_CCF
					from &&tab&cnt.
					order by YEAR_START_DT, ACCOUNT_RK;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				/* Load data into long tun table */
				proc append data=&out_lib..temp_ccf base=longrun.&longrun_ccf_ds.;
				quit;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		%if %str(&create_lng_pd) eq %str(Y) or %str(&create_lng_lgd) eq %str(Y) or
			%str(&create_lng_ccf) eq %str(Y)%then %do;
			/* Get actuals information at end of the year */
			%do cnt=1 %to &LONG_RUN_YRS;
				/* Update actuals at the end of the year */
/*			%let dt_start = datepart;*/
				data _null_;
					dt_start=&&lng_dt&cnt.;
					dt_end=intnx('year', dt_start, 0, 'end');
					dttm_start=dhms(dt_start, 0,0,0);
					dttm_end=dhms(dt_end, 23, 59, 59);
					dt_3_month=intnx('month', dt_start, -3, 'begin');
					dttm_3_month=dhms(dt_3_month, 0,0,0);
					call symput('year_start_dt', dt_start);
					call symput('year_end_dt', dt_end);
					call symput('year_start_dttm', dttm_start);
					call symput('year_end_dttm', dttm_end);
					call symput('month_3_dt', dt_3_month);
					call symput('month_3_dttm', dttm_3_month);
				run;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

				proc sql noprint;
					/* Get accounts only for the selected year */
					create table &out_lib..temp_actual_sel as
					select YEAR_START_DT, ACCOUNT_RK
					from &out_lib..temp_actual 
					where year_start_dt = &year_start_dt;
				
					/* Get Default Date */
				
					create table &out_lib..temp_actual_def as
					select YEAR_START_DT, temp_actual_sel.ACCOUNT_RK, 
						DEFAULT_DT, DEFAULT_EVENT_SK, EXPOSURE_AT_DEFAULT_AMT AS EXPOSURE_AT_DEFAULT,
						EXPOSURE_AT_DEFAULT_AMT AS BALANCE_AT_DEFAULT
					from &out_lib..temp_actual_sel inner join dim.default_event_dim
					on (temp_actual_sel.account_rk = default_event_dim.account_rk and
						default_event_dim.VALID_END_DTTM > &year_start_dttm and 
						default_event_dim.VALID_START_DTTM < &year_end_dttm
						and DEFAULT_STATUS_CD=&DEFAULT_STATUS_CD and 
						DEFAULT_DT between &year_start_dt and &year_end_dt)
					order by YEAR_START_DT, temp_actual_sel.ACCOUNT_RK, DEFAULT_DT DESC,
					DEFAULT_EVENT_SK DESC;
				quit;
				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				/* Get Exposure at default on default date */
				data table &out_lib..temp_actual_def1;
					set &out_lib..temp_actual_def;
					by YEAR_START_DT ACCOUNT_RK DEFAULT_DT DEFAULT_EVENT_SK;
					if FIRST.DEFAULT_EVENT_SK then output;
				run;
				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */

					/* Get limit at start of year */
				proc sql noprint;
					create table &out_lib..temp_actual_acc as
					select YEAR_START_DT, temp_actual_def1.ACCOUNT_RK, DEFAULT_DT, 
						EXPOSURE_AT_DEFAULT, BALANCE_AT_DEFAULT,  
						CURRENT_LIMIT_AMT as LIMIT_AT_START
					from &out_lib..temp_actual_def1 inner join dim.account_dim
					on (temp_actual_def1.account_rk = account_dim.account_rk and
						&year_start_dttm between account_dim.VALID_START_DTTM and 
						account_dim.VALID_END_DTTM);
				

					/* Get limit at default date */
				
					create table &out_lib..temp_actual_acc1 as
					select YEAR_START_DT, temp_actual_acc.ACCOUNT_RK, DEFAULT_DT, 
						EXPOSURE_AT_DEFAULT, BALANCE_AT_DEFAULT, 
						LIMIT_AT_START, CURRENT_LIMIT_AMT as LIMIT_AT_DEFAULT
					from &out_lib..temp_actual_acc inner join dim.account_dim
					on (temp_actual_acc.account_rk = account_dim.account_rk and
						dhms(DEFAULT_DT, 23, 59, 59) between account_dim.VALID_START_DTTM and 
						account_dim.VALID_END_DTTM);
				

					/* Get time_sks for the year */
				
					select time_sk into :year_time separated by ","
					from dim.time_dim
					where cal_month_last_dt between &year_start_dt and &year_end_dt;

					/* Get time_sks for last three months at start of year */
					select time_sk into :bal_time separated by ","
					from dim.time_dim
					where cal_month_last_dt between &month_3_dt and &year_start_dt;
					
					/* Get account sks for three month at start of year */
					
					create table &out_lib..temp_actual_3mth_acs as
					select YEAR_START_DT, temp_actual_acc1.ACCOUNT_RK, ACCOUNT_SK
					from &out_lib..temp_actual_acc1 inner join dim.account_dim
					on (temp_actual_acc1.account_rk = account_dim.account_rk and 
						account_dim.VALID_END_DTTM > &month_3_dttm and 
						account_dim.VALID_START_DTTM < &year_start_dttm);

					/* Get balance at start of year */
					create table &out_lib..temp_actual_snap as
					select YEAR_START_DT, temp_actual_3mth_acs.ACCOUNT_RK, 
						avg(BALANCE_AMT) as BALANCE_AT_START 
					from &out_lib..temp_actual_3mth_acs inner join fact.account_snapshot
					on (temp_actual_3mth_acs.account_sk = account_snapshot.account_sk and
						time_sk in (&bal_time))
					group by YEAR_START_DT, temp_actual_3mth_acs.ACCOUNT_RK;
					

					/* Get account sks for the year */
					
					create table &out_lib..temp_actual_year_acs as
					select YEAR_START_DT, temp_actual_acc1.ACCOUNT_RK, ACCOUNT_SK
					from &out_lib..temp_actual_acc1 inner join dim.account_dim
					on (temp_actual_acc1.account_rk = account_dim.account_rk and 
						account_dim.VALID_END_DTTM > &year_start_dttm and 
						account_dim.VALID_START_DTTM < &year_end_dttm);
					
					/* Get actual good bad during the year */
					create table &out_lib..temp_actual_gb as
					select YEAR_START_DT, temp_actual_year_acs.ACCOUNT_RK, 
						max(DAYS_PAYMENT_PAST_DUE_CNT) as DAYS_PAYMENT_PAST_DUE_CNT 
					from &out_lib..temp_actual_year_acs inner join fact.account_snapshot
					on (temp_actual_year_acs.account_sk = account_snapshot.account_sk and
						time_sk in (&year_time))
					group by YEAR_START_DT, temp_actual_year_acs.ACCOUNT_RK;

					/* Join balance at start and actual good bad with other data */
					
					create table &out_lib..temp_actual_snap1 as
					select temp_actual_acc1.YEAR_START_DT, temp_actual_acc1.ACCOUNT_RK, 
						temp_actual_acc1.DEFAULT_DT, temp_actual_acc1.EXPOSURE_AT_DEFAULT, 
						temp_actual_acc1.BALANCE_AT_DEFAULT, temp_actual_acc1.LIMIT_AT_START, 
						temp_actual_acc1.LIMIT_AT_DEFAULT, BALANCE_AT_START, 
						case when DAYS_PAYMENT_PAST_DUE_CNT>&ACTUAL_DLQ_DAYS then 1
						else 0 end as ACTUAL_GOOD_BAD,
						intnx('month', DEFAULT_DT, &RCV_PERIOD_MONTH_CNT, 'same') as RCV_END_DT, 
						dhms(DEFAULT_DT, 0, 0, 0) as DEFAULT_DTTM
					from &out_lib..temp_actual_acc1 inner join &out_lib..temp_actual_snap
					on (temp_actual_acc1.account_rk = temp_actual_snap.account_rk and
						temp_actual_acc1.year_start_dt = temp_actual_snap.year_start_dt)
					inner join &out_lib..temp_actual_gb
					on (temp_actual_acc1.account_rk = temp_actual_gb.account_rk and
						temp_actual_acc1.year_start_dt = temp_actual_gb.year_start_dt);
					

					/*Finding the time_sk for default_dt */
					
					create table &out_lib..temp_actual_rcv2 as
					select YEAR_START_DT, ACCOUNT_RK, DEFAULT_DT, DEFAULT_DTTM, 
						RCV_END_DT, dhms(RCV_END_DT, 0, 0, 0) as RCV_END_DTTM, 
						TIME_SK as DFT_TIME_SK
					from &out_lib..temp_actual_snap1 inner join dim.time_dim
					on (intnx('month', temp_actual_snap1.default_dt, 0, 'end') = 
						time_dim.cal_month_last_dt);
						
					/*Finding the time_sk for recovery end time*/
					
					create table &out_lib..temp_actual_rcv3 as
					select YEAR_START_DT, ACCOUNT_RK, DEFAULT_DT, DEFAULT_DTTM, 
						RCV_END_DT, RCV_END_DTTM, DFT_TIME_SK, 
						TIME_SK as RCV_END_TIME_SK
					from &out_lib..temp_actual_rcv2 inner join dim.time_dim
					on (intnx('month', temp_actual_rcv2.rcv_end_dt, 0, 'end') = 
						time_dim.cal_month_last_dt);
					
					/* Get accounts for the recovery period */
					
					create table &out_lib..temp_actual_rcv_acs as
					select YEAR_START_DT, temp_actual_rcv3.ACCOUNT_RK, ACCOUNT_SK, 
						DEFAULT_DT, RCV_END_DT, DEFAULT_DTTM, RCV_END_DTTM, 
						DFT_TIME_SK, RCV_END_TIME_SK
					from &out_lib..temp_actual_rcv3 inner join dim.account_dim
					on (temp_actual_rcv3.account_rk = account_dim.account_rk and 
						account_dim.VALID_END_DTTM > DEFAULT_DTTM and 
						account_dim.VALID_START_DTTM <= RCV_END_DTTM);
					
					/* Sum the value of recovery cost amount and value at recovery
						amount for every account */
					
					create table &out_lib..temp_actual_rcv4 as
					select YEAR_START_DT, ACCOUNT_RK,  
						sum(RECOVERY_COST_AMT) as RECOVERY_COST_AMT, 
						sum(VALUE_AT_RECOVERY_AMT) as VALUE_AT_RECOVERY_AMT
					from &out_lib..temp_actual_rcv_acs inner join fact.recovery_fact
					on (temp_actual_rcv_acs.account_sk = recovery_fact.account_sk and
						time_sk between dft_time_sk and rcv_end_time_sk)
					group by YEAR_START_DT, ACCOUNT_RK;
					
					/* Get actual loss at the end of the recovery period. */
					
					create table &out_lib..temp_actual_rcv as
					select temp_actual_snap1.YEAR_START_DT, temp_actual_snap1.ACCOUNT_RK, 
						EXPOSURE_AT_DEFAULT, BALANCE_AT_DEFAULT, 
						LIMIT_AT_START, LIMIT_AT_DEFAULT, BALANCE_AT_START, ACTUAL_GOOD_BAD, 
						(EXPOSURE_AT_DEFAULT + RECOVERY_COST_AMT - VALUE_AT_RECOVERY_AMT) AS ACTUAL_LOSS
					from &out_lib..temp_actual_snap1 left join &out_lib..temp_actual_rcv4
					on (temp_actual_snap1.account_rk = temp_actual_rcv4.account_rk and 
						temp_actual_snap1.year_start_dt = temp_actual_rcv4.year_start_dt)
					order by YEAR_START_DT, temp_actual_snap1.ACCOUNT_RK;
				quit;

				%chk_error(&sqlrc.); /*Call the macro to check for errors in the previous step */

				data &out_lib..temp_actual;
					merge &out_lib..temp_actual (in=a) &out_lib..temp_actual_rcv;
					by YEAR_START_DT ACCOUNT_RK;
					if a;
				run;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;

		/* Update the long run tables */
		%if %str(&create_lng_pd) eq %str(Y) %then %do;
			%if &lgd_method_type eq &LONG_RUN_MAX_TYPE or 
				&lgd_method_type eq &LONG_RUN_AVG_TYPE %then %do;
				proc sort data=longrun.&longrun_pd_ds.;
				by YEAR_START_DT ACCOUNT_RK;
				run;

				data longrun.&longrun_pd_ds.;
					merge longrun.&longrun_pd_ds. &out_lib..temp_actual;
					by YEAR_START_DT ACCOUNT_RK;
				run;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
			%else %do;
				proc sort data=longrun.&longrun_pd_ds.;
				by YEAR_START_DT ACCOUNT_RK;
				run;

				data longrun.&longrun_pd_ds.;
					merge longrun.&longrun_pd_ds. &out_lib..temp_actual(drop=EXPOSURE_AT_DEFAULT ACTUAL_LOSS);
					by YEAR_START_DT ACCOUNT_RK;
				run;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;
		%if %str(&create_lng_lgd) eq %str(Y) %then %do;
			data longrun.&longrun_lgd_ds.;
				merge longrun.&longrun_lgd_ds. &out_lib..temp_actual(drop=ACTUAL_GOOD_BAD
					BALANCE_AT_START BALANCE_AT_DEFAULT LIMIT_AT_START LIMIT_AT_DEFAULT);
				by YEAR_START_DT ACCOUNT_RK;
			run;

			%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
		%end;
		%if %str(&create_lng_ccf) eq %str(Y) %then %do;
			%if &lgd_method_type eq &AVG_EST_TYPE %then %do;
				data longrun.&longrun_pd_ds.;
					merge longrun.&longrun_pd_ds. &out_lib..temp_actual(drop=ACTUAL_GOOD_BAD 
						EXPOSURE_AT_DEFAULT ACTUAL_LOSS);
					by YEAR_START_DT ACCOUNT_RK;
				run;

				%chk_error(&syserr.); /*Call the macro to check for errors in the previous step */
			%end;
		%end;
	%end;

	%if &create_error_cd > 4 %then %do;
		/* Insert failed status into the table */
		%trylock(member=&lib..pool_scheme_defn, timeout=100000);
		proc sql;
			update &lib..pool_scheme_defn set 
				STATUS_DESC="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.status_desc_failed.1, noquote))"
			where POOL_SCHEME_NO=&scheme_no and OBS_DT=&obs_dt;
		quit;
		lock  &lib..pool_scheme_defn clear;
	%end;
	%else %do;
		/* Insert success status into the table */
		%trylock(member=&lib..pool_scheme_defn, timeout=100000);
		proc sql;
			update &lib..pool_scheme_defn set STATUS_DESC="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_pool_abt.status_desc_completed.1, noquote))"
			where POOL_SCHEME_NO=&scheme_no and OBS_DT=&obs_dt;
		quit;
		lock  &lib..pool_scheme_defn clear;
	%end;
%mend build_data;
%build_data;
Options nomlogic nosymbolgen nomprint;
/*Proc printto will reset the log redirection */
proc printto;
run;
%put NOTE: Resetting log to _webout&webout_options;
/*End of proc printto */
