/******************************************************************************************************
       Module:  build_scheme_data

     Function:  To define the pool scheme and build the data for the scheme in the background 

      Authors:  BIS Team
         Date:  14 May 2007
          SAS:  9.1
    Called-by:	pool_main
        Calls:  build_pool_abt

Datasets used:  1. pool.pool_scheme_defn: Retrieves the defn of existing pool schemes and saves the defn
					for new schemes in this dataset
				2. dim.analytical_model_dim: Retrieves the PD, LGD and CCF models from this
					dataset
 
        Logic:  1. The existing pool scheme definitions and its statuses are picked from the
					dataset pool.pool_scheme_defn and displayed on screen.
				2. The screen provides the user with the ability to define a new pool scheme by 
					clicking on the New Scheme button.
				3. For a new scheme, the user may set the name of the pool scheme, type of the scheme,
					date of data capture, the PD, LGD and CCF models and their assignment methods.
				4. If scheme type is based on Score ranges, then it is necessary to provide a PD model.
				5. If scheme type is based on PD ranges, then it is necessary to provide a PD model.
				6. If scheme type is based on Master scale, then it is necessary to provide a PD model.
				7. If scheme type is based on PD, LGD ranges, then it is necessary to provide PD, LGD models.
				8. If scheme type is based on PD, LGD, CCF ranges, then it is necessary to provide 
					PD, LGD, CCF models.
				9. For each model, Average of estimated values is the default assignment method. The
					other supported methods are Average of Long Run values and Maximum of Long Run values.
				10. If all information is available, the user may save the defined pool scheme and start
					the process of building the data by clicking the button Build Data.
				11. For an existing scheme, the user may restart building data for a failed pool scheme
					from the point of failure.
				12. The available actions are: 
						N: None. 
						A: Add Scheme
						D: Scheme Details
						B: Build Scheme
						R: Restart Scheme 
				
   Parameters:  scheme_no 		: The unique identifier of an existing pool scheme
				scheme_nm 		: The name of a new or existing pool scheme
   				scheme_type		: Pool scheme type. For e.g., SCR, PDR, PLR, PLC, PMS
				pd_method 		: The assignment method set for a PD model. For e.g., AVG, LRM, LRA
				lgd_method 		: The assignment method set for a LGD model. For e.g., AVG, LRM, LRA
				ccf_method 		: The assignment method set for a CCF model. For e.g., AVG, LRM, LRA
				seg_method 		: The assignment method set for a Segmentation model. For e.g., LRM, LRA
				pd_model_key 	: The retained key for the selected PD model
				lgd_model_key 	: The retained key for the selected LGD model
				ccf_model_key 	: The retained key for the selected CCF model
				seg_model_key 	: The retained key for the selected Segmentation model
				pd_model_nm 	: The name for the selected PD model
				lgd_model_nm 	: The name for the selected LGD model
				ccf_model_nm 	: The name for the selected CCF model
				seg_model_nm 	: The name for the selected Segmentation model
				month_no 		: The month of data capture
				year_no			: The year of data capture
				action 			: Screen action code
   		
******************************************************************************************************/

options mlogic mprint;
/*This code is Internationalized*/
data _NULL_;	
name=put(time(),mmss8.1);	
name1=kscan(name,1,":.");	
name2=kscan(name,2,":.");	
name3=kscan(name,3,":.");	
name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));	
call symput('log_name',"Data\bisdata\log\"||"build_scheme_data_"||ktrim(kleft(name4))||".log");
run;
proc printto log="&log_name";
run;
/* Global Input Parameters */

%global scheme_no scheme_nm scheme_type pd_method lgd_method ccf_method 
	pd_model_key pd_model_nm lgd_model_key lgd_model_nm ccf_model_key ccf_model_nm month_no 
	action year_no seg_method seg_model_key seg_model_nm;
/*In few stored process the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
%let lib=pool; 
%let out_lib=debug;
/* Setting default values for any required parameters */
data _null_;
	if "&pd_method"="" then call symput('pd_method',&AVG_EST_TYPE);
	if "&lgd_method"="" then call symput('lgd_method',&AVG_EST_TYPE);
	if "&ccf_method"="" then call symput('ccf_method',&AVG_EST_TYPE);
	if "&seg_method"="" then call symput('seg_method',&LONG_RUN_MAX_TYPE);
/*	if "&model_key"="" then call symput('model_key','.');*/
	if "&pd_model_key"="" then call symput('pd_model_key','.');
	if "&lgd_model_key"="" then call symput('lgd_model_key','.');
	if "&ccf_model_key"="" then call symput('ccf_model_key','.');
	if "&seg_model_key"="" then call symput('seg_model_key','.');
	if "&action"="" then call symput('action','N');
	if "&action"="N" then do;
		call symput('scheme_type','');
		call symput('pd_model_key','.');
		call symput('pd_model_nm','');
		call symput('lgd_model_key','.');
		call symput('lgd_model_nm','');
		call symput('ccf_model_key','.');
		call symput('ccf_model_nm','');
		call symput('seg_model_key','.');
		call symput('seg_model_nm','');
		call symput('month_no','');
		call symput('year_no','');
	end;
run;
/**/
%macro check_macro_variable;  
/* Trimming extra spaces for all input paremeters */
%if "&scheme_no" ne "" %then 
	%let scheme_no=%str(%sysfunc(ktrim(%sysfunc(kleft(&scheme_no)))));
%if "&scheme_nm" ne "" %then
	%let scheme_nm=%str(%sysfunc(ktrim(%sysfunc(kleft(&scheme_nm)))));
/*following line add AND "&scheme_type" ne " "   by scnhuh*/	
%if "&scheme_type" ne "" AND "&scheme_type" ne " " %then
	%let scheme_type=%str(%sysfunc(ktrim(%sysfunc(kleft(&scheme_type)))));
/*%let model_key=%str(%trim(%left(&model_key)));*/
/*%let model_nm=%str(&pd_model_nm);*/
%if "&pd_method" ne "" %then
	%let pd_method=%str(%sysfunc(ktrim(%sysfunc(kleft(&pd_method)))));
%if "&lgd_method" ne "" %then
	%let lgd_method=%str(%sysfunc(ktrim(%sysfunc(kleft(&lgd_method)))));
%if "&ccf_method" ne "" %then
	%let ccf_method=%str(%sysfunc(ktrim(%sysfunc(kleft(&ccf_method)))));
%if "&pd_model_key" ne "" %then
	%let pd_model_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&pd_model_key)))));
%if "&pd_model_nm" ne "" %then
	%let pd_model_nm=%str(&pd_model_nm);
%if "&lgd_model_key" ne "" %then
	%let lgd_model_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&lgd_model_key)))));
%if "&lgd_model_nm" ne "" %then
	%let lgd_model_nm=%str(&lgd_model_nm);
%if "&ccf_model_key" ne "" %then
	%let ccf_model_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&ccf_model_key)))));
%if "&ccf_model_nm" ne "" %then
	%let ccf_model_nm=%str(&ccf_model_nm);
/*following line add AND "&month_no" ne " "   by scnhuh*/	
%if "&month_no" ne "" AND "&month_no" ne " " %then
	%let month_no=%str(%sysfunc(ktrim(%sysfunc(kleft(&month_no)))));
/*following line add AND "&year_no" ne " "   by scnhuh*/	
%if "&year_no" ne "" AND "&year_no" ne " " %then
	%let year_no=%str(%sysfunc(ktrim(%sysfunc(kleft(&year_no)))));
%if "&seg_method" ne "" %then
	%let seg_method=%str(%sysfunc(ktrim(%sysfunc(kleft(&seg_method)))));
%if "&seg_model_key" ne "" %then
	%let seg_model_key=%str(%sysfunc(ktrim(%sysfunc(kleft(&seg_model_key)))));
%if "&seg_model_nm" ne "" %then
	%let seg_model_nm=%str(&seg_model_nm);
%if "&action" ne "" %then
	%let action=%str(&action);
/**/

%mend;
%check_macro_variable;

/* Links specific for the Score Range screens */
%let SELECT_SCHEME='SBIP://Foundation/BIS/pool/build_scheme_data';
%let BUILD_POOL_ABT='SBIP://Foundation/BIS/pool/build_pool_abt';

/* Setting output library based on the debug flag */
%let out_lib=;
%debug_lib_path;

data &out_lib..pd_model &out_lib..lgd_model &out_lib..ccf_model
	&out_lib..seg_model;
	set dim.analytical_model_dim(where=(model_sk > 0 and valid_end_dttm=&valid_high_dttm));
/*	output &out_lib..score_range_model;*/
	if MODEL_TYPE_CD in (&PD_ONG_MODEL) then output &out_lib..pd_model;
	if MODEL_TYPE_CD in (&LGD_MODEL) then output &out_lib..lgd_model;
	if MODEL_TYPE_CD in (&CCF_MODEL) then output &out_lib..ccf_model;
	if MODEL_TYPE_CD in (&SEG_MODEL) then output &out_lib..seg_model;
run;

data &out_lib..schemes;
	length scheme_type_cd $3.;
	scheme_type_cd=&SCORE_RANGE_TYPE;
	output;
	scheme_type_cd=&PD_RANGE_TYPE;
	output;
	scheme_type_cd=&PD_LGD_RANGE_TYPE;
	output;
	scheme_type_cd=&PD_LGD_CCF_RANGE_TYPE;
	output;
	scheme_type_cd=&MAP_PD_MASTER_TYPE;
	output;
/*	scheme_type_cd=&SEGMENT_TYPE;*/
/*	output;*/
run;

data &out_lib..scheme_type;
	length scheme_type_nm $50.;
	set &out_lib..schemes;
	scheme_type_nm=put(scheme_type_cd, $poolschm.);
run;

proc sql;
	create table &out_lib..scheme_details as
	select pool_scheme_no, pool_scheme_name, pool_scheme_type_cd, 
		obs_dt, status_desc
	from &lib..pool_scheme_defn
	order by pool_scheme_no;
quit;	

/* Date for Year for Data Capture*/
proc sql;
	create table &out_lib..year as
	select distinct cal_year_no as year
	from dim.time_dim;
quit;

/* Date for Month for Data Capture*/
data &out_lib..month;
	length month $10.;
	do month_no=1 to 12;
		month=put(month_no, $MTHFMT.);
		output;
	end;
run;

%macro add_scheme;
/*Code to remove the doubble quotes or single quotes if present*/
			%let scheme_nm_c1=%index(&scheme_nm,%str(%'));
			%let scheme_nm_c2=%index(&scheme_nm,%str(%"));
			data _null_;
			%if %str(&scheme_nm_c1) ne %str(0) %then %do;
				x=compress(&scheme_nm,"'");
				call symput('scheme_nm',x);
			%end;
			%else %if %str(&scheme_nm_c2) ne %str(0) %then %do;
				x=compress(&scheme_nm,'"');
				call symput('scheme_nm',x);
			%end;
			
			run;
/**************************************/	
	%if %str(&action) = %str(N) %then %do;
		/* No Action */
		%let stat_cnt=0;
		proc sql noprint;
			select sum(case when STATUS_DESC in ("%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.status_desc.1, noquote))") then 1 else 0 end) 
				format=8. into :stat_cnt
			from &lib..pool_scheme_defn;
		quit;

		data _null_;
			file _webout&webout_options;
			a=input("&stat_cnt", 8.);
			put '		<tr><td>&nbsp;</td></tr>';
			put '		<tr> ';
			put '			<td height="32" bgcolor="#F1EACB">&nbsp; ';
			put '				<input type="button" name="Submit5" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.button_new_desc.1, noquote))"@;
			put '" class="mmhide_btn" onClick="javascript:newScheme()" '@;
			if a > 0 then 
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.start_cnt_disable.1, noquote))"@;
			put '/>';
			put '            	<input type="button" name="Submit5" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.button_restart.1, noquote))"@;
			put '" class="mmhide_btn" onClick="javascript:restart()" ';
			if a > 0 then 
					put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.start_cnt_disable.1, noquote))"@;
			put '/>';
			put '			</td> ';
			put '		</tr> ';
		run;
	%end;
	%else %if %str(&action) = %str(A) or %str(&action) = %str(D) %then %do;
		/* Action = New Scheme */
		data _null_;
			file _webout&webout_options;
			put '		<tr><td>&nbsp;</td></tr>';
			put '		<tr><td>';
			put '			<table width="100%" border="0" cellspacing="0" cellpadding="5">';
			put '					<H2 align="left">&nbsp;'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.schm_detail.1, noquote))"@;
			put '</H2>';
			put '				<tr>';
			put '					<td class="LabelFont">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.schm_nm.1, noquote))"@;
			put '</td>';
			put '					<td class="LabelFont">';
			put '						<input type="text" name="scheme_nm" style="width: 90mm" value="'@; 
			put "&scheme_nm"@; 
			put '" >';
/*			put '						<INPUT type=button class=mmhide_btn onClick="javascript:Select()" value="Select">';*/
			put '					</td>';
			put '				<tr>';
			put '					<td class="LabelFont">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.schm_type.1, noquote))"@;
			put '</td>';
			put '					<td class="LabelFont">';
			put '                			<select name="schemetype" onChange="javascript:selectSchType(this.value)">';
			/*I18N-scnjih-20080421-1320*/
/*			put '                  				<option>Select Scheme Type'@;*/
			put '     <option>'@; 
			put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.1, noquote))"@;
			
			/*I18N-scnjih-20080421-1320*/
			put '</option>';
		run;

		data _null_;
			file _webout&webout_options;
			set &out_lib..scheme_type;
			put '                  			<option value="'@;
			put scheme_type_cd@;
			put '" '@;
			if strip(scheme_type_cd)="&scheme_type." then put 'selected'@;
			put '>'@;
			put scheme_type_nm@;
			put '</option>';
		run;

		data _null_;
			file _webout&webout_options;
			put '						</select>';
			put '					</td>';
			put '				</tr>';
/*			put '				<tr>';*/
/*			put '					<td class="LabelFont">Model Name: </td>';*/
/*			put '					<td class="LabelFont">';*/
/*			put '                			<select name="model" onChange="javascript:setMdl(this.value)">';*/
/*			put '                  				<option>Select Model</option>';*/
		run;

/*		data _null_;*/
/*			file _webout&webout_options;*/
/*			set &out_lib..score_range_model;*/
/*			put '                  				<option value="'@;*/
/*			put MODEL_RK@; */
/*			put ':'@;*/
/*			put MODEL_NM@;*/
/*			put '" '@;*/
/*			if input(strip(put(MODEL_RK,8.)),8.)="&model_key." then put 'selected'@;*/
/*			put '>'@;*/
/*			put MODEL_NM@;*/
/*			put '</option>';*/
/*		run;*/

		data _null_;
			file _webout&webout_options;
/*			put '						</select>';*/
/*			put '					</td>';*/
/*			put '				</tr>';*/
			put '				<tr>';
			put '					<td width="100" class="LabelFont">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.data_capture.1, noquote))"@;
			put '</td>';
			put '              		<td><span class="LabelFont">';
			put '                		<select name="year" onChange="javascript:selectYear(this.value)">';
			/*I18N-scnjih-20080421-1320*/
/*			put '                  			<option>Year</option>';*/
			put '                  			<option>'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.2, noquote))"@;
			put '</option>';
			/*I18N-scnjih-20080421-1320*/
		run;

		data _null_;
			file _webout&webout_options;
			set &out_lib..year;
			put '                  			<option value="'@;
			put year@;
			put '" '@;
			if input(strip(put(year,8.)),8.)="&year_no." then put 'selected'@;
			put '>'@;
			put year@;
			put '</option>';
		run;

		data _null_;
			file _webout&webout_options;
			put '                		</select>';
			put '                		<select name="month" onChange="javascript:selectMonth(this.value)">';
			/*I18N-scnjih-20080421-1320*/
/*			put '                  			<option value="">Month</option>';*/
			put '                  			<option value="">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.3, noquote))"@;
			put '</option>';
			/*I18N-scnjih-20080421-1320*/
		run;

		data _null_;
			file _webout&webout_options;
			set &out_lib..month;
			put '                  			<option value="'@;
			put _N_@;
			put '" '@;
			if input(strip(put(_N_,8.)),8.)="&month_no." then put 'selected'@;
			put '>'@;
			put month@;
			put '</option>';
		run;

		data _null_;
/*I18N-scnhuh-20080418-1515*/
/*change webout to _webout and add &webout_options*/		
			/*file webout;*/
			file _webout&webout_options;
/*I18N-scnhuh-20080418-1515*/
			put '                		</select>';
			put '              		</span></td>';
			put '            		</tr>';
		run;
	%end;
	%else %if %str(&action) = %str(B) %then %do;
		/* Action = Build Data */
		%let error_code=NONE;

		/*Proc printto will redirect the log to the directory
		"Data/bisdata/log". */
/*		%global log_name;*/
/*		data _NULL_;*/
/*		name=put(time(),mmss8.1);*/
/*		name1=kscan(name,1,":.");*/
/*		name2=kscan(name,2,":.");*/
/*		name3=kscan(name,3,":."); */
/*		name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));*/
/*		call symput('log_name',"&LOG_PATH"||"build_scheme_data_"||ktrim(kleft(name4))||".log");*/
/**/
/*		run;*/
/**/
/*		%put NOTE: Sending log messages to :&log_name;*/
/**/
/*		proc printto log="&log_name";*/
/*		quit;*/

		%let obsdt=%sysfunc(mdy(&month_no,1,&year_no));
		%let sch_no=.;
		proc sql noprint;
			select pool_scheme_no into :sch_no 
			from pool.pool_scheme_defn 
			where kupcase(ktrim(kleft(pool_scheme_name)))=kupcase(ktrim(kleft("&scheme_nm"))); 
		quit;

		%if %str(&sch_no)=%str(.) %then %do;
			proc sql noprint;
				select case when max(pool_scheme_no)=. then 0 else max(pool_scheme_no) end 
					into :scheme_no 
				from &lib..pool_scheme_defn;
			quit;

			%let scheme_no=%eval(&scheme_no+1);
			%if %quote(%'&scheme_type%') ne &SEGMENT_TYPE %then %do;
		 		proc sql noprint;
					insert into &lib..pool_scheme_defn
					(POOL_SCHEME_NO, POOL_SCHEME_NAME, POOL_SCHEME_TYPE_CD, PD_METHOD_TYPE_CD, 
					PD_MODEL_RK, LGD_METHOD_TYPE_CD, LGD_MODEL_RK, CCF_METHOD_TYPE_CD, CCF_MODEL_RK, 
					OBS_DT, STATUS_DESC, CREATION_DT, CREATED_BY, KS_VALUE, 
					STABILITY_VALUE) 
					VALUES(&scheme_no, "&scheme_nm", "&scheme_type", "&pd_method", 
					&pd_model_key, "&lgd_method", &lgd_model_key, "&ccf_method", &ccf_model_key, 
					&obsdt, 
					"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.status_desc.1, noquote))",
					., "", ., .);
				quit;
			%end;
			%else %do;
				proc sql noprint;
					insert into &lib..pool_scheme_defn
					(POOL_SCHEME_NO, POOL_SCHEME_NAME, POOL_SCHEME_TYPE_CD, SEG_METHOD_TYPE_CD, 
					SEG_MODEL_RK, OBS_DT, STATUS_DESC, CREATION_DT, CREATED_BY, KS_VALUE, 
					STABILITY_VALUE) 
					VALUES(&scheme_no, "&scheme_nm", "&scheme_type", "&seg_method", 
					&seg_model_key, &obsdt, 
					"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.status_desc.1, noquote))",
					., "", ., .);
				quit;
			%end;
			%if &sqlrc ne 0 %then
			%do;
			%let error_code = ERROR;
			%end;

			/*Proc printto will reset the log redirection */
/*			proc printto;*/
/*			quit; */

			%put NOTE: Resetting log to _webout&webout_options;
			/*End of proc printto */

			/* Show the relevant messages on the screen */
			data _null_;
				file _webout&webout_options;
				put '<script language="JavaScript" type="text/JavaScript"> ';
				put '<!-- ';
			%if %str(&error_code) eq %str(ERROR) %then %do;
				put '	alert("'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_failed_save_schm_p1.1, noquote))"@;
				put "&log_name"@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_failed_save_schm_p2.1, noquote))"@;
				put '")';
				put "	document.form1._program.value=&SELECT_SCHEME";
				put '	document.form1.action.value="A"';
				put '	document.form1.submit();';
			%end;
			%else %do;
				/* Start the process of building in background */
				put "	document.form1.obs_dt.value=&obsdt";
				put "	document.form1.scheme_no.value=&scheme_no";
				put '	document.form1.action.value="B"';
				put "	document.form1._program.value=&BUILD_POOL_ABT";
				put '	document.form1.submit();';
				/* Refresh the screen with the required information */
				put '	alert("'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_schm_dtl_saved.1, noquote))"@;
				put '")';
				put '	Cancel();';
			%end;
				put '//-->';
				put	'</script> ';
			run;
		%end;
		%else %do;
			data _null_;
				file _webout&webout_options;
				put '<script language="JavaScript" type="text/JavaScript"> ';
				put '<!-- ';
				put '	alert("'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_schm_same_nm_exist.1, noquote))"@;
				put '")';
				put '	newScheme()';
				put '//-->';
				put	'</script> ';
			run;
		%end;
	%end;
	%if %str(&action) = %str(R) %then %do;
		/* Action = Restart */
		%let error_code=NONE;

		/*Proc printto will redirect the log to the directory
		"Data/bisdata/log". */
/*		%global log_name;*/
/*		data _NULL_;*/
/*		name=put(time(),mmss8.1);*/
/*		name1=kscan(name,1,":.");*/
/*		name2=kscan(name,2,":.");*/
/*		name3=kscan(name,3,":."); */
/*		name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));*/
/*		call symput('log_name',"Data/bisdata/log/pool_restart_"||ktrim(kleft(name4))||".log");*/
/*		run;*/
/**/
/*		%put NOTE: Sending log messages to :&log_name;*/
/**/
/*		proc printto log="&log_name";*/
/*		quit;*/

		proc sql noprint;
			update &lib..pool_scheme_defn set STATUS_DESC ="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.status_desc.1, noquote))"
			where POOL_SCHEME_NO=&scheme_no;

			select obs_dt, pool_scheme_type_cd, pd_model_rk, pd_method_type_cd, 
				lgd_model_rk, lgd_method_type_cd, ccf_model_rk, ccf_method_type_cd
				into :obsdt, :scheme_type, :pd_model_key, :pd_method, 
				:lgd_model_key, :lgd_method, :ccf_model_key, :ccf_method
			from &lib..pool_scheme_defn
			where POOL_SCHEME_NO=&scheme_no;
		quit;
		%if &sqlrc ne 0 %then
		%do;
		%let error_code = ERROR;
		%end;

		%let lgd_model_key=%sysfunc(ktrim(%sysfunc(kleft(&lgd_model_key))));
		/*Proc printto will reset the log redirection */
/*		proc printto;*/
/*		quit; */

		%put NOTE: Resetting log to _webout&webout_options;
		/*End of proc printto */

		/* Show the relevant messages on the screen */
		data _null_;
			file _webout&webout_options;
			put '<script language="JavaScript" type="text/JavaScript"> ';
			put '<!-- ';
		%if %str(&error_code) eq %str(ERROR) %then %do;
			put '	alert("'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_failed_strt_schm_p1.1, noquote))"@;
			put "&log_name"@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_failed_strt_schm_p2.1, noquote))"@;
			put '")';
			put "	document.form1._program.value=&SELECT_SCHEME";
			put '	document.form1.action.value="N"';
			put '	document.form1.submit();';
		%end;
		%else %do;
			/* Start the process of building in background */
			put "	document.form1.obs_dt.value=&obsdt";
			put "	document.form1.scheme_no.value=&scheme_no";
			put "	document.form1.scheme_type.value='&scheme_type'";
			put "	document.form1.pd_model_key.value='&pd_model_key'";
			put "	document.form1.pd_method.value='&pd_method'";
			put "	document.form1.lgd_model_key.value='&lgd_model_key'";
			put "	document.form1.lgd_method.value='&lgd_method'";
			put "	document.form1.ccf_model_key.value='&ccf_model_key'";
			put "	document.form1.ccf_method.value='&ccf_method'";
			put '	document.form1.action.value="R"';
			put "	document.form1._program.value=&BUILD_POOL_ABT";
			put '	document.form1.submit();';
			/* Refresh the screen with the required information */
			put '	alert("'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_schm_restarted.1, noquote))"@;
			put '")';
			put '	Cancel();';
		%end;
			put '//-->';
			put	'</script> ';
		run;
	%end;
	%if %str(&action) = %str(A) %then %do;
		/* Cancel button in case of Add Scheme only */
		data _null_;
			file _webout&webout_options;
			put '			</table>';
			put '		<tr> ';
			put '			<td height="32" bgcolor="#F1EACB">&nbsp; ';
			put '            	<input type="button" name="Submit" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.button_cancel.1, noquote))"@;
			put '" class="mmhide_btn" onClick="javascript:Cancel()" />';
			put '			</td> ';
			put '		</tr> ';
		run;
	%end;
	%if %str(&action) = %str(D) %then %do;
		/* Show Scheme details also */
		/* If scheme type is not segmentation, show the methods and models for PD,
			LGD and CCF.
		   If scheme type is segmentation, show the method and model for segmentation alone
		*/
		%if %quote(%'&scheme_type%') ne &SEGMENT_TYPE %then %do;
			/* In all cases except segmentation */
			data _null_;
				file _webout&webout_options;
				put '				<tr>';
				put '<table border="0" cellspacing="0" cellpadding="10">';
				put '<tr>';
				put '<td>';
			    put '<table border="0"  width="100%" cellpadding="0" cellspacing="0">';
				put '<tr>';
				put '<td valign="top">';
				put '<fieldset>';
				put '<legend><span class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_pd_to_pool.1, noquote))"@;
				put '</span></legend>';
				put '<p class="LabelFont">';
				desc_max=put(&LONG_RUN_MAX_TYPE, $poolasgn.);
				desc_avg=put(&LONG_RUN_AVG_TYPE, $poolasgn.);
				desc_est=put(&AVG_EST_TYPE, $poolasgn.);
				put '<input type="radio" name="pd" value="'@;
					put &LONG_RUN_MAX_TYPE@;
					put '" onClick="javascript:setPDMethod(this.value)" '@;
					if "&pd_method" = &LONG_RUN_MAX_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_max@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="pd" value="'@;
					put &LONG_RUN_AVG_TYPE@;
					put '" onClick="javascript:setPDMethod(this.value)" '@;
					if "&pd_method" = &LONG_RUN_AVG_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_avg@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="pd" value="'@;
					put &AVG_EST_TYPE@;
					put '" onClick="javascript:setPDMethod(this.value)" '@;
					if "&pd_method" = &AVG_EST_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_est@;
					put '</p>';
				put '</fieldset>';
				put '</td>';
				put '</tr>';
				put '</table>';
				put '</td>';

				put '<td>';
				put '<table border="0"  width="100%" cellpadding="0" cellspacing="0">';
				put '<tr>';
				put '<td valign="top">';
				put '<fieldset>';
				put '<legend><span class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_lgd_to_pool.1, noquote))"@;
				put '</span></legend>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="lgd" value="'@;
					put &LONG_RUN_MAX_TYPE@;
					put '" onClick="javascript:setLGDMethod(this.value)" '@;
					if "&lgd_method" = &LONG_RUN_MAX_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_max@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="lgd" value="'@;
					put &LONG_RUN_AVG_TYPE@;
					put '" onClick="javascript:setLGDMethod(this.value)" '@;
					if "&lgd_method" = &LONG_RUN_AVG_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_avg@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="lgd" value="'@;
					put &AVG_EST_TYPE@;
					put '" onClick="javascript:setLGDMethod(this.value)" '@;
					if "&lgd_method" = &AVG_EST_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_est@;
					put '</p>';
				put '</fieldset>';
				put '</td>';
				put '</tr>';
				put '</table>';
				put '</td>';
				
				put '<td>';
				put '<table border="0"  width="100%" cellpadding="0" cellspacing="0">';
				put '<tr>';
				put '<td valign="top">';
				put '<fieldset>';
				put '<legend><span class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_ccf_to_pool.1, noquote))"@;
				put '</span></legend>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="ccf" value="'@;
					put &LONG_RUN_MAX_TYPE@;
					put '" onClick="javascript:setCCFMethod(this.value)" '@;
					if "&ccf_method" = &LONG_RUN_MAX_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_max@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="ccf" value="'@;
					put &LONG_RUN_AVG_TYPE@;
					put '" onClick="javascript:setCCFMethod(this.value)" '@;
					if "&ccf_method" = &LONG_RUN_AVG_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_avg@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="ccf" value="'@;
					put &AVG_EST_TYPE@;
					put '" onClick="javascript:setCCFMethod(this.value)" '@;
					if "&ccf_method" = &AVG_EST_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_est@;
					put '</p>';
				put '</fieldset>';
				put '</td>';
				put '</tr>';
				put '</table>';
				put '</td>';
				put '</tr>';

				/*Here we are populating the drop down box for PD_MODEL type */
				put'<tr>';
				put'	<td>';
				/*Drop Down start here */
				put'<p class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_pd_model_nm.1, noquote))"@;
				put'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</p>';
				put'<select name=pdmodel style="width: 71mm" onChange="javascript:setPDMdl()">';
				 /*I18N-scnjih-20080421-1320*/
/*				put '<option value="">None</option>';*/
				put '<option value="">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.4, noquote))"@;
				put '</option>';
				 /*I18N-scnjih-20080421-1320*/
			run;

			data _null_;
				file _webout&webout_options;
				set &out_lib..pd_model;
				put '<option value="'@;
				put MODEL_RK@; 
				put ':'@;
				put MODEL_NM@;
				put '" '@;
				if input(strip(put(MODEL_RK,8.)),8.)="&pd_model_key." then put 'selected'@;
				put '>'@;
				put MODEL_NM@;
				put '</option>';
			run;

			data _null_;
				file _webout&webout_options;
				put'</select>';
				put'</td>';

				/*Here we are populating the drop down box for LGD_MODEL type */
				put'	<td>';
				/*Drop Down start here */
				put'<p class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_lgd_model_nm.1, noquote))"@;
				put'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</p>';
				put'<select name=lgdmodel style="width: 71mm" onChange="javascript:setLGDMdl()">';
				/*I18N-scnjih-20080421-1320*/
/*				put '<option value="">None</option>';*/
				put '<option value="">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.5, noquote))"@;
				put '</option>';
				 /*I18N-scnjih-20080421-1320*/
				run;

			data _null_;
				file _webout&webout_options;
				set &out_lib..lgd_model;
				put '<option value="'@;
				put MODEL_RK@; 
				put ':'@;
				put MODEL_NM@;
				put '" '@;
				if input(strip(put(MODEL_RK,8.)),8.)="&lgd_model_key." then put 'selected'@;
				put '>'@;
				put MODEL_NM@;
				put '</option>';
			run;

			data _null_;
				file _webout&webout_options;
				put'</select>';
				put'</td>';

				/*Here we are populating the drop down box for CCF_MODEL type */
				put'	<td>';
				/*Drop Down start here */
				put'<p class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_ccf_model_nm.1, noquote))"@;
				put'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</p>';
				put'<select name=ccfmodel style="width: 71mm" onChange="javascript:setCCFMdl()">';
				 /*I18N-scnjih-20080421-1320*/
				/*put '<option value="">None</option>';*/
				put '<option value="">'@;
				put  "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.6, noquote))"@;
				put '</option>';
				 /*I18N-scnjih-20080421-1320*/
			run;

			data _null_;
				file _webout&webout_options;
				set &out_lib..ccf_model;
				put '<option value="'@;
				put MODEL_RK@; 
				put ':'@;
				put MODEL_NM@;
				put '" '@;
				if input(strip(put(MODEL_RK,8.)),8.)="&ccf_model_key." then put 'selected'@;
				put '>'@;
				put MODEL_NM@;
				put '</option>';
			run;
		%end;
		%else %do;
			/* In case of segmentation */
			data _null_;
				file _webout&webout_options;
				put '                		</select>';
				put '              		</span></td>';
				put '            	</tr>';
				put '				<tr>';
				put '<table border="0" cellspacing="0" cellpadding="10">';
				put '<tr>';
				put '<td>';
			    put '<table border="0"  width="100%" cellpadding="0" cellspacing="0">';
				put '<tr>';
				put '<td valign="top">';
				put '<fieldset>';
				put '<legend><span class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_assign_value_pool.1, noquote))"@;
				put '</span></legend>';
				put '<p class="LabelFont">';
				desc_max=put(&LONG_RUN_MAX_TYPE, $poolasgn.);
				desc_avg=put(&LONG_RUN_AVG_TYPE, $poolasgn.);
				desc_est=put(&AVG_EST_TYPE, $poolasgn.);
				put '<input type="radio" name="pd" value="'@;
					put &LONG_RUN_MAX_TYPE@;
					put '" onClick="javascript:setPDMethod(this.value)" '@;
					if "&pd_method" = &LONG_RUN_MAX_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_max@;
					put '</p>';
				put '<p class="LabelFont">';
				put '<input type="radio" name="pd" value="'@;
					put &LONG_RUN_AVG_TYPE@;
					put '" onClick="javascript:setPDMethod(this.value)" '@;
					if "&pd_method" = &LONG_RUN_AVG_TYPE then 
						put 'checked=checked'@;
					put '>'@;
					put desc_avg@;
					put '</p>';
				put '</fieldset>';
				put '</td>';
				put '</tr>';
				put '</table>';
				put '</td>';
				put '</tr>';

				/*Here we are populating the drop down box for SEG_MODEL type */
				put'<tr>';
				put'	<td>';
				/*Drop Down start here */
				put'<p class="LabelFont">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_seg_model_nm.1, noquote))"@;
				put'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</p>';
				put'<select name=segmodel style="width: 71mm" onChange="javascript:setSegMdl()">';
				 /*I18N-scnjih-20080421-1320*/
				/*	put '<option value="">None</option>';*/
				put '<option value="">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_data.sas.7, noquote))"@;
				put '</option>';
				 /*I18N-scnjih-20080421-1320*/
			run;

			data _null_;
				file _webout&webout_options;
				set &out_lib..seg_model;
				put '<option value="'@;
				put MODEL_RK@; 
				put ':'@;
				put MODEL_NM@;
				put '" '@;
				if input(strip(put(MODEL_RK,8.)),8.)="&seg_model_key." then put 'selected'@;
				put '>'@;
				put MODEL_NM@;
				put '</option>';
			run;

			data _null_;
				file _webout&webout_options;
				put'</select>';
				put'</td>';
			run;
		%end;

		data _null_;
			file _webout&webout_options;
			put '			</table>';
			put '		<tr> ';
			put '			<td height="32" bgcolor="#F1EACB">&nbsp; ';
			put '				<input type="button" name="Submit" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.button_build_data.1, noquote))"@;
			put '" class="mmhide_btn" onClick="javascript:buildData()" />';
			put '            	<input type="button" name="Submit" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.button_cancel.1, noquote))"@;
			put '" class="mmhide_btn" onClick="javascript:Cancel()" />';
			put '			</td> ';
			put '		</tr> ';
		run;
	%end;
%mend add_scheme;

/* Start HTML Screen */
data _null_;
	file _webout&webout_options;
	put '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" ';
	put '"http://www.w3.org/TR/html4/loose.dtd"> ';
	put '<html> ';
	put '<head> ';
/*I18N-scnhuh-20080418-1023*/
/*from iso-8859-1 to utf-8*/
	put '<meta http-equiv="Content-Type" content="text/html; charset=utf-8"> ';
/*I18N-scnhuh-20080418-1023*/	
	put '<title>'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.titel_pooling.1, noquote))"@;
	put '</title> ';
	put '<link href="/SASCSPooling/styles/default.css" rel="stylesheet" type="text/css"> ';
	put '<style type="text/css">';
	put '<!--';
	put 'body {';
	put '	background-color: #FAF8ED;';
	put '}';
	put '-->';
	put '</style>';

	put '<script language="JavaScript" type="text/JavaScript"> ';
	put '<!-- ';

	/* For all dropdowns and model_types, the value holds the information such as 
	prod_type_cd:prod_type_desc. Hence depending on the selected index in the dropdown, 
	the code and descriptions have to extracted. This function does the job of extracting 
	the codes and descriptions. */
	put 'function getCodeDesc(mdl_val,mdl_key,mdl_name) {';
	put '	var len=mdl_val.length';
	put '	var indx=mdl_val.indexOf(":")';
	put '	var key_end=indx-1';
	put '	var nm_st=indx+1';
	put '	var nm_len=len-nm_st';
	put '	var key=mdl_val.substr(0,key_end)';
	put '	var nm=mdl_val.substr(nm_st,nm_len)';
	put '	mdl_key.value=key';
	put '	mdl_name.value=nm';
	put '}';
	/**/

	put 'function selectMonth(val) {';
	put '	document.form1.month_no.value=val';
	put '}';

	put 'function selectYear(val) {';
	put '	document.form1.year_no.value=val';
	put '}';

	put 'function selectSchType(val) {';
	put '	if((val != '''') && (val != '' '')) {';
	put "		var seg=&SEGMENT_TYPE";
	put '		var new_sc=val.replace(/ /, "")';
	put '		var sc_type=document.form1.scheme_type.value.replace(/ /, "")';
	put '		if( ((sc_type == '''') || (sc_type == '' '')) || '@;
	put '((sc_type != seg) && (new_sc == seg)) || '@;
	put '((sc_type == seg) && (new_sc != seg)) ) {';
	put '			document.form1.scheme_type.value=val';
	put "			document.form1._program.value=&SELECT_SCHEME";
	put '			document.form1.action.value="D"';
	put '			document.form1.submit();';
	put '		}';
	put '		else document.form1.scheme_type.value=val';
	put '	}';
	put '}';

/*	put 'function setMdl(val) {';*/
/*	put '	var indx=document.form1.model.selectedIndex';*/
/*	put '	if (indx!=0) {';*/
/*	put '		getCodeDesc(document.form1.model.options[indx].value, document.form1.model_key, document.form1.model_nm)';*/
/*	put '	}';*/
/*	put '	else {';*/
/*	put '		document.form1.model_key.value=""';*/
/*	put '		document.form1.model_nm.value=""';*/
/*	put '	}';*/
/*	put '}';*/

	put 'function setPDMdl() {';
	put '	var indx=document.form1.pdmodel.selectedIndex';
	put '	if (indx!=0) {';
	put '		getCodeDesc(document.form1.pdmodel.options[indx].value, document.form1.pd_model_key, document.form1.pd_model_nm)';
	put '	}';
	put '	else {';
	put '		document.form1.pd_model_key.value=""';
	put '		document.form1.pd_model_nm.value=""';
	put '	}';
	put '}';

	put 'function setLGDMdl() {';
	put '	var indx=document.form1.lgdmodel.selectedIndex';
	put '	if (indx!=0) {';
	put '		getCodeDesc(document.form1.lgdmodel.options[indx].value, document.form1.lgd_model_key, document.form1.lgd_model_nm)';
	put '	}';
	put '	else {';
	put '		document.form1.lgd_model_key.value=""';
	put '		document.form1.lgd_model_nm.value=""';
	put '	}';
	put '}';

	put 'function setCCFMdl() {';
	put '	var indx=document.form1.ccfmodel.selectedIndex';
	put '	if (indx!=0) {';
	put '		getCodeDesc(document.form1.ccfmodel.options[indx].value, document.form1.ccf_model_key, document.form1.ccf_model_nm)';
	put '	}';
	put '	else {';
	put '		document.form1.ccf_model_key.value=""';
	put '		document.form1.ccf_model_nm.value=""';
	put '	}';
	put '}';

	put 'function setSegMdl() {';
	put '	var indx=document.form1.segmodel.selectedIndex';
	put '	if (indx!=0) {';
	put '		getCodeDesc(document.form1.segmodel.options[indx].value, document.form1.seg_model_key, document.form1.seg_model_nm)';
	put '	}';
	put '	else {';
	put '		document.form1.seg_model_key.value=""';
	put '		document.form1.seg_model_nm.value=""';
	put '	}';
	put '}';

	put 'function setPDMethod(type) {';
	put '	document.form1.pd_method.value=type';
	put '}';

	put 'function setLGDMethod(type) {';
	put '	document.form1.lgd_method.value=type';
	put '}';

	put 'function setCCFMethod(type) {';
	put '	document.form1.ccf_method.value=type';
	put '}';

	put 'function setScheme(check, val) {';
	put '	if(check.checked == true) {';
	put '		document.form1.scheme_no.value=val';
	put '		getCodeDesc(val, document.form1.scheme_no, document.form1.seg_model_nm)';
	put '	}';
	put '	else';
	put '		document.form1.scheme_no.value=''.''';
	put '}';

	put 'function newScheme() {';
	put "	document.form1._program.value=&SELECT_SCHEME";
	put '	document.form1.action.value="A"';
	put '	document.form1.submit();';
	put '}';

	put 'function buildData() {';
	put '	if((document.form1.scheme_nm.value == '''') || (document.form1.scheme_nm.value == '' '') || '@;
/*	put '(document.form1.model_key.value == '''') || (document.form1.model_key.value == ''.'') || '@;*/
	put '(document.form1.month_no.value == '''') || (document.form1.month_no.value == '' '') || '@;
	put '(document.form1.year_no.value == '''') || (document.form1.year_no.value == '' '')) ';
	put '		alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_sel_schm_to_build.1, noquote))"@;
	put '")';
	put '	else {';
	put '		var sc_type=document.form1.scheme_type.value.replace(/ /,"")';
	put '		if( ((sc_type == '@;
	put "&SCORE_RANGE_TYPE"@;
	put ') || (sc_type == '@;
	put "&PD_RANGE_TYPE"@;
	put ') || (sc_type == '@;
	put "&MAP_PD_MASTER_TYPE"@;
	put ')) && ((document.form1.pd_model_key.value == '''') || (document.form1.pd_model_key.value == ''.'')) )';
	put '			alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_enter_pd_model_to_scr.1, noquote))"@;
	put'")';
	put '		else if( (sc_type == '@;
	put "&PD_LGD_RANGE_TYPE"@;
	put	') && ( (document.form1.pd_model_key.value == '''') || (document.form1.pd_model_key.value == ''.'') || '@;
	put '(document.form1.lgd_model_key.value == '''') || (document.form1.lgd_model_key.value == ''.'') ) )';
	put '			alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_enter_pd_lgd_model_to_scr.1, noquote))"@;
	put '")';
	put '		else if( (sc_type  == '@;
	put "&PD_LGD_CCF_RANGE_TYPE"@;
	put	') && ( (document.form1.pd_model_key.value == '''') || (document.form1.pd_model_key.value == ''.'') || '@;
	put '(document.form1.lgd_model_key.value == '''') || (document.form1.lgd_model_key.value == ''.'') || '@;
	put '(document.form1.ccf_model_key.value == '''') || (document.form1.ccf_model_key.value == ''.'') ) )';
	put '			alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_enter_pd_lgd_ccf_model_to_scr.1, noquote))"@;
	put '")';
	put '		else if( (sc_type  == '@;
	put "&SEGMENT_TYPE"@;
	put ')&& ( (document.form1.seg_model_key.value == '''') || (document.form1.seg_model_key.value == ''.'') ) )';
	put '			alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_seg_model_to_scr.1, noquote))"@;
	put '")';
	put '		else {';
	put "			document.form1._program.value=&SELECT_SCHEME";
	put '			document.form1.action.value="B"';
	put '			document.form1.submit();';
	put '		}';
	put '	}';
	put '}';

	put 'function Cancel() {';
	put "	document.form1._program.value=&SELECT_SCHEME";
	put '	document.form1.action.value="N"';
	put '	document.form1.submit();';
	put '}';

	put 'function restart() {';
	put '	if((document.form1.scheme_no.value == '''') || (document.form1.scheme_no.value == '' '')){';
	put '		alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.alert_pls_schm_restart.1, noquote))"@;
	put '")';
	put '	}';
	put '	else{';
	put "		document.form1._program.value=&SELECT_SCHEME";
	put '		document.form1.action.value="R"';
	put '		document.form1.submit();';
	put '	}';
	put '}';

	put '//-->';
	put	'</script> ';
	put '</head>';

	put '<body>';
	put '<form name="form1" method="post" ACTION="/SASStoredProcess/do"> ';

	/* Hidden textboxes to hold the values for the program name and the output paremeters */
	put '<input type="hidden" name="_program" VALUE="">';
	scheme_no=symget("scheme_no");
	x='<input type="hidden" name="scheme_no" value="'||strip(scheme_no)||'">';
	put x;
	scheme_type=symget("scheme_type");
	x='<input type="hidden" name="scheme_type" value="'||strip(scheme_type)||'">';
	put x;
/*	model_key=symget("model_key");*/
/*	x='<input type="hidden" name="model_key" value="'||strip(model_key)||'">';*/
/*	put x;*/
/*	model_nm=symget("model_nm");*/
/*	x='<input type="hidden" name="model_nm" value="'||strip(model_nm)||'">';*/
/*	put x;*/
	pd_model_key=symget("pd_model_key");
	x='<input type="hidden" name="pd_model_key" value="'||strip(pd_model_key)||'">';
	put x;
	pd_model_nm=symget("pd_model_nm");
	x='<input type="hidden" name="pd_model_nm" value="'||strip(pd_model_nm)||'">';
	put x;
	lgd_model_key=symget("lgd_model_key");
	x='<input type="hidden" name="lgd_model_key" value="'||strip(lgd_model_key)||'">';
	put x;
	lgd_model_nm=symget("lgd_model_nm");
	x='<input type="hidden" name="lgd_model_nm" value="'||strip(lgd_model_nm)||'">';
	put x;
	ccf_model_key=symget("ccf_model_key");
	x='<input type="hidden" name="ccf_model_key" value="'||strip(ccf_model_key)||'">';
	put x;
	ccf_model_nm=symget("ccf_model_nm");
	x='<input type="hidden" name="ccf_model_nm" value="'||strip(ccf_model_nm)||'">';
	put x;
	seg_model_key=symget("seg_model_key");
	x='<input type="hidden" name="seg_model_key" value="'||strip(seg_model_key)||'">';
	put x;
	seg_model_nm=symget("seg_model_nm");
	x='<input type="hidden" name="seg_model_nm" value="'||strip(seg_model_nm)||'">';
	put x;
	pd_method=symget("pd_method");
	x='<input type="hidden" name="pd_method" value="'||strip(pd_method)||'">';
	put x;
	lgd_method=symget("lgd_method");
	x='<input type="hidden" name="lgd_method" value="'||strip(lgd_method)||'">';
	put x;
	ccf_method=symget("ccf_method");
	x='<input type="hidden" name="ccf_method" value="'||strip(ccf_method)||'">';
	put x;
	seg_method=symget("seg_method");
	x='<input type="hidden" name="seg_method" value="'||strip(seg_method)||'">';
	put x;
	month_no=symget("month_no");
	x='<input type="hidden" name="month_no" value="'||strip(month_no)||'">';
	put x;
	year_no=symget("year_no");
	x='<input type="hidden" name="year_no" value="'||strip(year_no)||'">';
	put x;
	obs_dt=symget("obs_dt");
	x='<input type="hidden" name="obs_dt" value="'||strip(obs_dt)||'">';
	put x;
	put '<input type="hidden" name="action" value="N">';
	/**/
run;

data _null_;
	file _webout&webout_options;
	put '<table width="100%"  border="0" cellpadding="0" cellspacing="0" bgcolor="#FAF8ED"> ';
	put '	<tr> ';
	put '		<td align="left"> ';
	put '		<table width="100%"  border="0" align="left" cellpadding="3" cellspacing="0"> ';
	put '			<tr> ';
	put '				<td>';
	put '				<table width="100%"  border="0" cellspacing="0" cellpadding="0"> ';
	put '					<tr> ';

	/* Navigational Links */
	put '						<td><span class="BreadCrumb">&nbsp;</td>';
	put '						<td align="right"><img src="/SASCSPooling/images/CrBanner.gif" width="259" height="29"></td> ';
	put '					</tr>';
	put '				</table>';  
	put '				</td>';
	put '			</tr>';
	put '			<tr>';
	put '				<td>';

	/* Main Body */
	put '					<H2 align="left">';
	put '						<table width="100%" border="0" cellspacing="0" cellpadding="0">';
	put '							<tr><td>'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_def_pool_schm.1, noquote))"@;
	put '</td>';
	put '							</tr>';
	put '						</table>';
	put '					</H2>';
	put '				</td>';
	put '			</tr>';

	put '	   <tr><td>&nbsp;</td></tr>';
	put '      <tr>';
	put '        <td><table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#7F9DB9" class="tableborder">';
	put '          <tr>';
	put '            <td valign="top"><table width="100%" border="0" cellpadding="3" cellspacing="0">';
	put '                <tr bgcolor="#FFFFFF">';
	put '                  <td class="HZtableborder" width=2%>&nbsp; </td>';
	put '                  <td class="HZtableborder">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_schm_nm.1, noquote))"@;
	put '</td>';
	put '                  <td class="HZtableborder">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_schm_type.1, noquote))"@;
	put '</td>';
	put '                  <td class="HZtableborder">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_dt_of_abt.1, noquote))"@;
	put '</td>';
	put '                  <td class="HZtableborder">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.column_status.1, noquote))"@;
	put '</td>';
	put '                </tr>';
run;

data _NULL_;
	file _webout&webout_options;
	set &out_lib..scheme_details;
	if  mod(_N_,2)=0 then
		put '                <tr bgcolor="#F4F4F2">';
	else 
		put '                <tr bgcolor="#FFFFFF">';
	put '				   <td width=2% align="center">';
	put '						<input type="radio" name="action_radio" value="'@;
	put pool_scheme_no@;
	put ':'@;
	put status_desc@;
	put '" OnClick="javascript:setScheme(this, this.value)"'@;
	put '></td>';
	put '                  <td class="verttebleborder">' @;
	put pool_scheme_name@; 
	put '</td>'@;
	schm=put(pool_scheme_type_cd, $poolschm.);
	put '                  <td class="verttebleborder">' @;
	put schm@; 
	put '</td>'@;
/*I18N-scnhuh-20080428-1818*/	
/*change format from ddyymm10. to nldate.*/
	word_dt=put(obs_dt,nldate.);
/*I18N-scnhuh-20080428-1818*/		
  	put '                  <td class="verttebleborder">'@;
	put word_dt@; 
	put '</td>';
  	put '                  <td class="verttebleborder">'@;
	if status_desc="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_data.status_desc.1, noquote))" then 
	put '<img src="/SASCSPooling/images/MessageProcess.gif" width="16" height="16" />';
	put status_desc@; 
	put '</td>';
	put '                </tr>';
run;
	
data _NULL_;
	file _webout&webout_options;
	put '            </table></td>';
	put '          </tr>';
	put '        </table>';
run;

%add_scheme;

data _NULL_;
	file _webout&webout_options;
	put '	</tr>';
	put '</table>';
	put '</body>';
	put '</html>';
run;
Options nomlogic nosymbolgen nomprint;
proc printto ;
run;
