/******************************************************************************************************
       Module:  build_scheme_rating_pool

     Function:  To define the pools for the selected pool scheme

      Authors:  BIS Team
         Date:  14 May 2007
          SAS:  9.1
    Called-by:	pool_main
        Calls:

Datasets used:  1. pool.pool_scheme_defn: Retrieves the defn of existing pool schemes and saves the defn
					for new schemes in this dataset
				2. dim.analytical_model_dim: Retrieves the PD, LGD and CCF models from this
					dataset
				3. pool.build_abt_status: Stores the status of the building activity of the various ABTs
					to be built for the pool scheme
				4. The scored datasets _<PD_MODEL_RK>_PD, _<LGD_MODEL_RK>_LGD, _<CCF_MODEL_RK>_CCF
				5. The long run datasets _LNG_<PD_MODEL_RK>_PD, _LNG_<PD_MODEL_RK>_PD_LGDYRS,
					_LNG_<LGD_MODEL_RK>_LGD, _LNG_<CCF_MODEL_RK>_CCF, _LNG_<CCF_MODEL_RK>_CCF_LGDYRS
				6. pool.pool_defn: Retrieves the defn of existing pools for the pool scheme and saves
					the defn for new pools in this dataset

        Logic:  1. The user may select for the list of completed schemes which are of PD ranges based
				2. If an existing pool defn exists for the selected scheme, retrieve it from
					pool.pool_defn and display it on screen
				3. If an existing pool defn does not exist, create pools with the ranges based
					on the selected master scale.
				4. Other action available on the screen is editing the description of a pool.
				5. Once a pool is defined, it may be saved by clicking the Save button.
				6. The no of accounts are deterined by pooling the scored PD dataset.
				7. The ssigned PD, LGD, CCF values for the pools are determined by the assignment
					methods AVG, LRA, LRM and the scored or long run datasets.
				8. The available actions are:
					N: None.
					R: Retrieve scheme. If the scheme_nm and scheme_no exist in the pool tables, a retrieval
						is done.
					M: Master Scale
					P: New Master Scale
					D: Edit Pool Description
					V: Save


   Parameters:  scheme_no 		: The unique identifier of an existing pool scheme
   				scheme_type		: Pool scheme type. For e.g., SCR, PDR, PLR, PLC, PMS
				action 			: Screen action code
				ms_sale_no		: Specifies the key of the selected master scale
				pool_desc		: Specifies the description to be set for the specified pool
				pool_no_desc	: Specifies the pool to which the pool description should be set
				ks_value		: Specifies the calculated KS value for the scheme
				stab_value		: Specifies the calculated pool stability index value for the scheme

 ******************************************************************************************************/
/*This code is Internationalized*/
options mlogic mprint;
data _NULL_;
	name=put(time(),mmss8.1);
	name1=kscan(name,1,":.");
	name2=kscan(name,2,":.");
	name3=kscan(name,3,":.");
	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));
	call symput('log_name',"Data\bisdata\log\"||"build_scheme_rating_pool_"||ktrim(kleft(name4))||".log");
run;
proc printto log="&log_name";
run;
/* Global Input Parameters */
%global scheme_no scheme_type ms_scale_no action pool_desc pool_no_desc ks_value stab_value;
/*In few stored process the exact location of change is not mentioned. Just file is known so 
  it is identified with the tag of hot fix as below*/
/*CS4.3 HF2 */
%let lib=pool;

/* Trimming extra spaces for all input paremeters */
%macro check_macro_variable;
%if "&scheme_no" ne "" %then
	%let scheme_no=%str(%sysfunc(ktrim(%sysfunc(kleft(&scheme_no)))));
%if "&scheme_type" ne "" %then
	%let scheme_type=%str(%sysfunc(ktrim(%sysfunc(kleft(&scheme_type)))));
%if "&action" ne "" %then
	%let action=%str(%sysfunc(ktrim(%sysfunc(kleft(&action)))));
%if "&ms_scale_no" ne "" %then
	%let ms_scale_no=%str(%sysfunc(ktrim(%sysfunc(kleft(&ms_scale_no)))));
%if "&pool_desc" ne "" %then
	%let pool_desc=%str(%sysfunc(ktrim(%sysfunc(kleft(&pool_desc)))));
%if "&ks_value" ne "" %then
	%let ks_value=%str(%sysfunc(ktrim(%sysfunc(kleft(&ks_value)))));
%if "&stab_value" ne "" %then
	%let stab_value=%str(%sysfunc(ktrim(%sysfunc(kleft(&stab_value)))));
%let pool_desc=&pool_desc;
/*Code to remove the doubble quotes or single quotes if present*/
			%let pool_desc_c1=%index(&pool_desc,%str(%'));
			%let pool_desc_c2=%index(&pool_desc,%str(%"));
			data _null_;
				%if %str(&pool_desc_c1) ne %str(0) %then %do;
					x=compress(&pool_desc,"'");
					call symput('pool_desc',x);
				%end;
				%else %if %str(&pool_desc_c2) ne %str(0) %then %do;
					x=compress(&pool_desc,'"');
					call symput('pool_desc',x);
				%end;
			run;
%mend;
%check_macro_variable;
/* Links specific for the PD Range screens */
%let SELECT_POOL_SCHEME='SBIP://Foundation/BIS/pool/build_scheme_rating_pool';
%let VIEW_POOL_SCHEME=SBIP://Foundation/BIS/pool/view_scheme;

/* Setting output library based on the debug flag */
%let out_lib=;
%debug_lib_path;

/*%let in_rate_range=%trim(%left(_&scheme_no._rate_range_in));*/

	%let out_rate_range=%sysfunc(ktrim(%sysfunc(kleft(_&scheme_no._rate_range_out)))) ;
	%let pool_defn=%sysfunc(ktrim(%sysfunc(kleft(_&scheme_no._pool_defn)))) ;
	%let out_data_prep=%sysfunc(ktrim(%sysfunc(kleft(&out_lib.._&scheme_no._data_prep)))) ;
	%let out_graph_ds=%sysfunc(ktrim(%sysfunc(kleft(&out_lib.._&scheme_no._ks_graph)))) ;
	%let out_pool_stab=%sysfunc(ktrim(%sysfunc(kleft(&out_lib.._&scheme_no._psi))))  ;

/* Setting default values for any required parameters */
data _null_;
	if "&action"="" then call symput('action', 'N');
	if "&scheme_no"="" then call symput('scheme_no', '.');
	if "&ms_scale_no"="" then call symput('ms_scale_no', '.');
	if "&ks_value"="" then call symput('ks_value','.');
	if "&stab_value"="" then call symput('stab_value','.');
run;

/* Display screen with table and pool stability index in all cases except when action = N */
%macro display_screen;
	%if "&action" ne "N" %then %do;
		proc sql noprint;
			select count(*) into :cnt from &out_lib..ms_scale_details
		quit;

		%if %sysfunc(ktrim(%sysfunc(kleft(&cnt)))) > 0 %then %do;
			data _null_;
				file _webout&webout_options;
				put '					<tr>';
					 /*I18N-scnjih-20080421-1501*/
/*				put '						<td class="LabelFont" width="15%">Master Scale Name: </td>';*/
				put '						<td class="LabelFont" width="15%">'@;
				put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.14, noquote))"@;
				put '</td>';
					 /*I18N-scnjih-20080421-1501*/
				put '						<td class="LabelFont">';
				put '							<p class="LabelFont">';
				put '							<select name="msname" style="width: 90mm" onChange="javascript:setMS()">';
					 /*I18N-scnjih-20080421-1039*/
/*				put '								<option>Select Scheme</option>';*/
				put '								<option>'@;
				put  "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.2, noquote))"@;
				put '</option>';
				 /*I18N-scnjih-20080421-1039*/
			run;

			data _null_;
				file _webout&webout_options;
				set &out_lib..ms_scale_details;
				put '								<option value="'@;
				put master_scale_no@;
				put '" '@;
				if master_scale_no=&ms_scale_no then put 'selected'@;
				put '>'@;
				put master_scale_name@;
				put '</option>';
			run;

			data _null_;
				file _webout&webout_options;
				put '							</select>';
				put '							</p>';
				put '						</td>';
				put '					</tr>';
				put '				</table>';
				put '			</tr>';
			run;

			%let action=R;
		%end;
		%else %do;
			%let action=P;
		%end;
	%end;
	%put *******************&action***************&ms_scale_no*********;
	%if "&action" ne "N" and &ms_scale_no ne . %then %do;
		data _null_;
			file _webout&webout_options;
			put '			<tr>';
			put '				<td>';

			/* If action ne N */
				/*table*/
			put '					<table border="0" width="100%" cellpadding="" cellspacing="0" class="tableborder">';
			put '						<tr bgcolor="#7F9DB9">';
			put '							<td width="3%" align="center" class="HZtableborder" rowspan=2>';
				/*<!-- slide down content goes here -->*/
			put '								<div id="dhtmlgoodies_slidedown">';
			put '								<div id="dhtmlgoodies_control"><a href="#" onclick="slidedown_showHide();return false;"><img src="/SASCSPooling/images/ActionMenu.gif" style="border:0"/></a>
			</div>';
			put '								<div id="dhtmlgoodies_contentBox">';
			put '									<div id="dhtmlgoodies_content">';
			put '									<table width="100%" border="1" class="tableborder" cellpadding="3" cellspacing="0">';
			put '										<tr><td>';
			put '										<table width="100%" border="0" cellpadding="3" cellspacing="0">';
			put '										<tr>';
			put '											<td width=30% align="left">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.table_name.1, noquote))"@;
			put '</td>';
			put '											<td align="left">';
			put '												<input type="text" name="edit_desc" size=15 maxlength=70 value=""'@;
			put '"></td>';
			put '										</tr>';
			put '										</table>';
			put '										<table width="100%" border="0" cellpadding="3" cellspacing="0">';
			put '										<tr>';
			put '											<td align="left">&nbsp;';
			put '												<INPUT type=button class=mmhide_btn onClick="javascript:OK()" value="   '@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.button_ok.1, noquote))"@;
			put'   ">';
			put '												<INPUT type=button class=mmhide_btn onclick="slidedown_showHide();return false;" value=" '@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.button_cancel.1, noquote))"@;
			put ' "></td>';
			put '										</tr>';
			put '										</table>';
			put '										<tr><td>';
			put '									</table>';
			put '									</div>';
			put '								</div>';
			put '								</div>';
				/*		<!-- End slide down content -->*/
/*I am here*/
			put '							</td>';
					 /*I18N-scnjih-20080421-1039*/
/*			put '							<td width="5%" align="right" class="HZtableborder" rowspan=2>Pool Seq. No</td>';*/
/*			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>Start PD Range</td>';*/
/*			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>End PD Range</td>';*/
/*			put '							<td align="left" class="HZtableborder" rowspan=2>Pool Description</td>';*/
/*			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>No of Accounts</td>';*/
/*			put '							<td width="10%" align="left" class="HZtableborder" rowspan=2>Rating Grade</td>';*/
/*			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>Calculated PD</td>';*/
/*			put '							<td class="HZtableborder" align="center" colspan=3>Assigned</td>';*/
/*			put '						</tr>';*/
/*			put '						<tr>';*/
/*			put '							<td width="7%" align="right" class="HZtableborder">PD</td>';*/
/*			put '							<td width="7%" align="right" class="HZtableborder">LGD</td>';*/
/*			put '							<td width="7%" align="right" class="HZtableborder">CCF</td>';*/
			put '							<td width="5%" align="right" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.3, noquote))"@;
			put '</td>';
			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.4, noquote))"@;
			put '</td>';
			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.5, noquote))"@;
			put '</td>';
			put '							<td align="left" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.6, noquote))"@;
			put '</td>';
			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.7, noquote))"@;
			put '</td>';
			put '							<td width="10%" align="left" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.8, noquote))"@;
			put '</td>';
			put '							<td width="10%" align="right" class="HZtableborder" rowspan=2>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.9, noquote))"@;
			put '</td>';
			put '							<td class="HZtableborder" align="center" colspan=3>'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.10, noquote))"@;
			put '</td>';
			put '						</tr>';
			put '						<tr>';
			put '							<td width="7%" align="right" class="HZtableborder">'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.11, noquote))"@;
			put '</td>';
			put '							<td width="7%" align="right" class="HZtableborder">'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.12, noquote))"@;
			put '</td>';
			put '							<td width="7%" align="right" class="HZtableborder">'@;
			put "%sysfunc( sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.13, noquote))"@;
			put '</td>';
					 /*I18N-scnjih-20080421-1039*/
			put '						</tr>';
		run;

		data _null_;
			file _webout&webout_options;
			set &lib..&pool_defn.;
			if  mod(_N_,2)=0 then
				put '	<tr bgcolor="#F4F4F2" class="Font12px">';
			else
				put '	<tr bgcolor="#FFFFFF" class="Font12px">';
			put '		<td class="verttebleborder" align="center">&nbsp;';
			put '			<input type="radio" name="c" value="'@;
			put pool_seq_no@;
			put '" OnClick="javascript:SelPool(this)"'@;
			put '><br></td>';

			put '		<td class="verttebleborder" align="right">'@;
			put pool_seq_no @;
			put '</td>';

			put '		<td class="verttebleborder" align="right">'@;
			put from_pd@;
			put '</td>';

		  	put '		<td class="verttebleborder" align="right">'@;
			put to_pd @;
			put '</td>';

			put '		<td class="verttebleborder">'@;
			put pool_name@;
			put'</td>';

			put '		<td class="verttebleborder" align="right">'@;
			put No_of_Accounts@;
			put'</td>';

			put '		<td class="verttebleborder" align="left">'@;
			put ms_grade_name@;
			put'</td>';

			calculated_pd=put(calculated_pd, &CS_DEC_FMT);
			put '		<td class="verttebleborder" align="right">'@;
			put calculated_pd@;
			put'</td>';

			assigned_pd=put(assigned_pd, &CS_DEC_FMT);
			put '		<td class="verttebleborder" align="right">'@;
			put assigned_pd@;
			put '</td>';
			assigned_lgd=put(assigned_lgd, &CS_DEC_FMT);
			put '		<td class="verttebleborder" align="right">'@;
			put assigned_lgd@;
			put '</td>';
			assigned_ccf=put(assigned_ccf, &CS_DEC_FMT);
			put '		<td class="verttebleborder" align="right">'@;
			put assigned_ccf@;
			put '</td>';
			put '	</tr>';
		run;

		data _null_;
			file _webout&webout_options;
			put '					</table>';
			put '					</td>';
			put '				</tr>';

			put '			<tr>';
			put '				<td>';
			put '					<table width="60%" border="0" cellpadding="5" cellspacing="0">';
			put '						<tr>';
			put '							<td class="LabelFont" width="35%">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.column_pool_stab_indx.1, noquote))"@;
			put '</td>';
			put '							<td class="LabelFont">'@;
			stab=ktrim(kleft("&stab_value"));
			put '<input type="text" name="stab_value" readonly style="width: 30mm; background-color:#FAF8ED; border:0" value="'@;
			put stab@;
			put '"/>'@;
			put '</td>';
			put '						</tr>';
			put '						<tr>';
			put '							<td class="LabelFont">'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.column_ks_value.1, noquote))"@;
			put '</td>';
			put '							<td class="LabelFont">'@;
			ks=ktrim(kleft("&ks_value"));
			put '<input type="text" name="ks_value" readonly style="width: 30mm; background-color:#FAF8ED; border:0" value="'@;
			put ks@;
			put '"/>'@;
			put '</td>';
			put '						</tr>';
			put '					</table>';
			put '				</td>';
			put '			</tr>';
			put '		</table>';
			put '		</td>';
			put '	</tr>';
			put '	<tr><td>&nbsp;</td></tr>';
			put '	<tr> ';
			put '		<td height="32" bgcolor="#F1EACB">&nbsp;&nbsp;';
			put '			<INPUT type=button class=mmhide_btn onClick="Save()" value="'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.button_save.1, noquote))"@;
			put '" align="left">';
			put '		</td>';
			put '	</tr>';
		run;
	%end;
%mend display_screen;

/************ Macros for Pool Calculation and definition ************/
/* Save Pool Information */
%macro save_pool;
	%let error_code=NONE;

	/*Proc printto will redirect the log to the directory
	"Data/bisdata/error". */
/*	%global log_name;*/
/*	data _NULL_;*/
/*	name=put(time(),mmss8.1);*/
/**/
/**/
/**/
/*	name1=kscan(kname,1,":.");*/
/*	name2=kscan(kname,2,":.");*/
/*	name3=kscan(kname,3,":.");*/
/**/
/**/
/**/
/*	name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));*/
/*	call symput('log_name',"&LOG_PATH"||"build_scheme_rating_pool_"||ktrim(kleft(name4))||".log");*/
/*	run;*/
/**/
/*	%put NOTE: Sending log messages to :&log_name;*/
/**/
/*	proc printto log="&log_name";*/
/*	quit;*/

	/* Save scheme information in pool.pool_scheme_defn */
	proc sql;
		update pool.pool_scheme_defn set
			ks_value=&ks_value, stability_value=&stab_value, master_scale_no=&ms_scale_no
		where pool_scheme_no=&scheme_no;

		delete from pool.pool_defn where pool_scheme_no=&scheme_no;
	quit;

	%if &sqlrc > 4 %then %do;
		%let error_code = ERROR;
	%end;
	%else %do;
		data &out_lib..pool_temp(drop=from_pd to_pd);
			set pool.&pool_defn.(keep=pool_seq_no pool_name from_pd to_pd no_of_accounts
				assigned_pd assigned_lgd assigned_ccf score_points ms_grade_name
				calculated_pd rename=(calculated_pd=ms_calc_pd_no));
			pool_scheme_no=&scheme_no;
			score_seq_no=.;
			from_score_no=.;
			to_score_no=.;
			pd_seq_no=pool_seq_no;
			from_pd_no=from_pd;
			to_pd_no=to_pd;
			lgd_seq_no=.;
			from_lgd_no=.;
			to_lgd_no=.;
			ccf_seq_no=.;
			from_ccf_no=.;
			to_ccf_no=.;
		run;

		/* Save pool information in pool.pool_defn */
		proc append data=&out_lib..pool_temp base=pool.pool_defn force;
		quit;
	%end;

	/*Proc printto will reset the log redirection */
/*	proc printto;*/
/*	quit;*/

/*	%put NOTE: Resetting log to _webout&webout_options;*/
	/*End of proc printto */

	/* Show the relevant messages on the screen */
	data _null_;
		file _webout&webout_options;
		put '<script language="JavaScript" type="text/JavaScript"> ';
		put '<!-- ';
	%if %str(&error_code) eq %str(ERROR) %then %do;
		put '	alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.alert_pool_not_saved_p1.1, noquote))"@;
		put "&log_name"@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.alert_pool_not_saved_p2.1, noquote))"@;
		put '")';
	%end;
	%else %do;
		put '	alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.alert_pool_saved.1, noquote))"@;
		put '")';
		put '	document.form1.action.value = "R";';
		put "	document.form1._program.value=&SELECT_POOL_SCHEME";
		put '	document.form1.submit()';
	%end;
		put '//-->';
		put	'</script> ';
	run;
%mend;
/* Build all pool information */
%macro build_pool;
	proc sort data=&out_lib..&out_rate_range. out=work.temp SORTSEQ=EBCDIC;
		by descending from_pd;
	quit;

	proc sql;
		create table &out_lib..&out_rate_range. as
		select monotonic() as pool_seq_no, from_pd, to_pd, pd, ms_grade_name
		from work.temp;
	quit;

	/* Calculate the no. of accounts and assigned pd, lgd, ccf in case of AVG estimated */
	proc sql noprint;
		create table &out_lib..temp_avg as
			select *
			from &lib..&scored_ds_nm., &out_lib..&out_rate_range.
			where &scored_ds_nm..estimated_pd > &out_rate_range..from_pd and
				&scored_ds_nm..estimated_pd <= &out_rate_range..to_pd;

		create table &out_lib..temp2_avg as
			select pool_seq_no, count(account_rk) as No_of_Accounts,
				avg(estimated_pd) as calculated_pd,
				avg(estimated_lgd) as assigned_lgd,
				avg(estimated_ccf) as assigned_ccf,
				max(score_points) as score_points
			from &out_lib..temp_avg
			group by pool_seq_no;

		create table &lib..&pool_defn. as
			select &out_rate_range..pool_seq_no, &out_rate_range..from_pd,
				&out_rate_range..to_pd, calculated_pd,
				put(&out_rate_range..from_pd, 7.2)||"%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.column_range_pd.1, noquote))"||put(&out_rate_range..to_pd, 7.2) as pool_name,
				case when no_of_accounts = . then 0 else no_of_accounts end as no_of_accounts,
				&out_rate_range..pd as assigned_pd, assigned_lgd, assigned_ccf, score_points,
				ms_grade_name
			from &out_lib..&out_rate_range. left outer join &out_lib..temp2_avg
			on (&out_rate_range..pool_seq_no=temp2_avg.pool_seq_no);
	quit;

	%put ************&pd_method********&lgd_method**********&ccf_method********;
	/* If pd method is Long run max or avg, calculate the assigned pd */
	%let pd_mtd=%quote(%'&pd_method%');
	%if %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
		%sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) %then %do;
		%put ************LR: PD********;
		proc sql;
			create table &out_lib..temp_lr_pd AS
			select year_start_dt, pool_name, pool_seq_no,
				sum (actual_good_bad) as NO_OF_BADS,
				count (*) as NO_OF_ACCOUNTS,
				calculated NO_OF_BADS / calculated NO_OF_ACCOUNTS as ESTIMATED_POOL_PD
			from longrun.&long_run_ds_nm., &out_lib..&out_pd_range.
			where estimated_pd > from_pd and estimated_pd <= to_pd
			group by year_start_dt, pool_name, pool_seq_no;

			%if %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) %then %do;
				create table &out_lib..temp2_lr_pd AS
				select pool_name, pool_seq_no,
					max(estimated_pool_pd) as assigned_pd
				from &out_lib..temp_lr_pd
				group by pool_name, pool_seq_no;
			%end;
			%else %do;
				create table &out_lib..temp2_lr_pd AS
				select pool_name, pool_seq_no,
					avg(estimated_pool_pd) as assigned_pd
				from &out_lib..temp_lr_pd
				group by pool_name, pool_seq_no;
			%end;

			create table &lib..&pool_defn. as
			select monotonic() as pool_seq_no, &pool_defn..from_pd,
				&pool_defn..to_pd, &pool_defn..pool_name, &pool_defn..no_of_accounts,
				temp2_lr_pd.assigned_pd, &pool_defn..assigned_lgd, &pool_defn..assigned_ccf,
				&pool_defn..score_points
			from &lib..&pool_defn. left join &out_lib..temp2_lr_pd
			on (&pool_defn..pool_seq_no=temp2_lr_pd.pool_seq_no);
		quit;
	%end;

	/* If lgd method is Long run max or avg, calculate the assigned lgd */
	%let lgd_mtd=%quote(%'&lgd_method%');
	%if %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
		%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) %then %do;
		%put ********LR: LGD**********;
		proc sql;
			create table &out_lib..temp_lr_lgd AS
			select year_start_dt, pool_name, pool_seq_no,
				sum (ACTUAL_LOSS) as ACTUAL_LOSS,
				sum (EXPOSURE_AT_DEFAULT) as EXPOSURE_AT_DEFAULT,
				calculated ACTUAL_LOSS / calculated EXPOSURE_AT_DEFAULT as ESTIMATED_POOL_LGD
			from longrun.&long_run_ds_nm., &out_lib..&out_pd_range.
			where estimated_pd > from_pd and estimated_pd <= to_pd
			group by year_start_dt, pool_name, pool_seq_no;

			%if %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) %then %do;
				create table &out_lib..temp2_lr_lgd AS
				select pool_name, pool_seq_no,
					max(estimated_pool_lgd) as assigned_lgd
				from &out_lib..temp_lr_lgd
				group by pool_name, pool_seq_no;
			%end;
			%else %do;
				create table &out_lib..temp2_lr_lgd AS
				select pool_name, pool_seq_no,
					avg(estimated_pool_lgd) as assigned_lgd
				from &out_lib..temp_lr_lgd
				group by pool_name, pool_seq_no;
			%end;

			create table &lib..&pool_defn. as
			select monotonic() as pool_seq_no, &pool_defn..from_pd,
				&pool_defn..to_pd, &pool_defn..pool_name, &pool_defn..no_of_accounts,
				&pool_defn..assigned_pd, temp2_lr_lgd.assigned_lgd, &pool_defn..assigned_ccf,
				&pool_defn..score_points
			from &lib..&pool_defn. left outer join &out_lib..temp2_lr_lgd
			on (&pool_defn..pool_seq_no=temp2_lr_lgd.pool_seq_no);
		quit;
	%end;

	/* If ccf method is Long run max or avg, calculate the assigned ccf */
	%let ccf_mtd=%quote(%'&ccf_method%');
	%if %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
		%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) %then %do;
		%put **********LR: CCF********;
/* Actual CCF calculation
	(BALANCE_AMT * (PR_CURRENT_LIMIT_AMT / CURRENT_LIMIT_AMT)) as Balance_Default
	(Balance_Default - PR_BALANCE_AMT) / (CURRENT_LIMIT_AMT - PR_BALANCE_AMT) as
*/
		proc sql;
			create table &out_lib..temp_lr_act_ccf as
			select BALANCE_AT_DEFAULT * (LIMIT_AT_START / LIMIT_AT_DEFAULT) as
				APPR_BALANCE_DEFAULT, (calc APPR_BALANCE_DEFAULT - BALANCE_AT_START) /
				(LIMIT_AT_DEFAULT - BALANCE_AT_START) as ACTUAL_CCF
			from longrun.&long_run_ds_nm.;

			create table &out_lib..temp_lr_ccf AS
			select year_start_dt, pool_name, pool_seq_no,
				sum (ACTUAL_CCF) as ACTUAL_CCF,
				count (*) as NO_OF_ACCOUNTS,
				calculated ACTUAL_CCF / calculated NO_OF_ACCOUNTS as ESTIMATED_POOL_CCF
			from &out_lib..temp_lr_act_ccf, &out_lib..&out_pd_range.
			where estimated_pd > from_pd and estimated_pd <= to_pd
			group by year_start_dt, pool_name, pool_seq_no;

			%if %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) %then %do;
				create table &out_lib..temp2_lr_ccf AS
				select pool_name, pool_seq_no,
					max(estimated_pool_ccf) as assigned_ccf
				from &out_lib..temp_lr_ccf
				group by pool_name, pool_seq_no;
			%end;
			%else %do;
				create table &out_lib..temp2_lr_ccf AS
				select pool_name, pool_seq_no,
					avg(estimated_pool_ccf) as assigned_ccf
				from &out_lib..temp_lr_ccf
				group by pool_name, pool_seq_no;
			%end;

			create table &lib..&pool_defn. as
			select monotonic() as pool_seq_no, &pool_defn..from_pd,
				&pool_defn..to_pd, &pool_defn..pool_name, &pool_defn..no_of_accounts,
				&pool_defn..assigned_pd, &pool_defn..assigned_lgd, temp2_lr_ccf.assigned_ccf,
				&pool_defn..score_points
			from &lib..&pool_defn. left outer join &out_lib..temp2_lr_ccf
			on (&pool_defn..pool_seq_no=temp2_lr_ccf.pool_seq_no);
		quit;
	%end;
%mend build_pool;
/* Macro to calculete the value of Pool Stability Index for the pool definition */
%macro calculate_pool_stab;
	/* The pool_stability_index macro creates the data with the value for Pool Stability
		Index. The following parameters are passed to this macro:
		input_ds		: Name of the input data set
		input_pool_ds	:
		output_ds		: Name of the data set which stores data for pool_stability_index
		scheme_type		: Type of Pooling scheme
	*/
	proc sql noprint;
		select  count(*)  into: count_recs
		from &lib..&pool_defn.
		where ASSIGNED_PD < 1 and ASSIGNED_PD >0;
	quit;
	/*checking the all default , No default before calculating the statistics.*/
	%let count_recs=&count_recs;
/*	%let var_ASSIGNED_PD=&var_ASSIGNED_PD.;*/
	%put ********count_recs=&count_recs;
	%if (&count_recs eq 0)%then %do;		
		data _null_;

			file _webout&webout_options;
			put '<script language="JavaScript" type="text/JavaScript"> ';
			put '<!-- ';
			put '	alert("'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.pooling.alert_default_no_default.1, noquote))"@;
			put '")';
			put '	document.form1.action.value = "R";';
			put "	document.form1._program.value=&SELECT_POOL_SCHEME";
			put '	document.form1.submit()';
			put '//-->';
			put	'</script> ';
		run;	
	%end;
	%else %do;
		%let scheme_type1=%nrquote(%'&scheme_type%');
		%pool_stability_index(input_ds=&out_lib..temp_avg, input_pool_ds=&lib..&pool_defn.,
			output_ds=&out_pool_stab., scheme_type_cd=&scheme_type1);

		proc sql noprint;
			/*The select stmt below is for display purpose only*/
			select pool_stability_index format=&CS_DEC_FMT. into :stab_value from &out_pool_stab.;
		quit;
	%end;
%mend calculate_pool_stab;
/* Macro to calculete the value of KS for the pool definition */
%macro calculate_ks;
	proc sql;
		create table &out_lib..ks_ds as
		select monotonic() as pool_seq_no, &pool_defn..from_pd,
			&pool_defn..to_pd, &pool_defn..pool_name,
			&pool_defn..no_of_accounts as No_Of_ACC_Actual,
			&pool_defn..assigned_pd as PD, &pool_defn..assigned_lgd,
			&pool_defn..assigned_ccf, score_points, . as No_of_Actual_bads
		from &lib..&pool_defn.;
	quit;

		/* The cs_rep_data_prep macro prepares the data which is required as the input to
			calculate KS.
			The following parameters are passed to this macro:
			input_ds	: Name of the input data set
			output_ds	: Name of the data set which stores data for graph of ks
			Dist_type	: This mentions which distribution the ks has to be calculated on
							E.g., ACT will calculate the KS on Actual data and
								  EST will calculate the KS on Modeling predictions.
		*/
		%cs_rep_data_prep(input_ds=&out_lib..ks_ds,output_ds=&out_data_prep,Dist_type="EST");
			data check_default;
				set &out_data_prep;
				var_tot_acc = TP+FP+TN+FN;
				var_no_bads = FN+TN;
			run;
/*checking the all default , No default before calculating the statistics.*/
	%let err_code_var=;
	%check_alldefaults_nodefaults(check_default,var_tot_acc,var_no_bads, err_code_var);
	%if (&err_code_var ne 'NONE')%then %do;
		data _null_;
			file _webout&webout_options;
			put '<script language="JavaScript" type="text/JavaScript"> ';
			put '<!-- ';
			put '	alert("'@;
			put "%sysfunc(sasmsg(smd_ds.bismsg,crs.pooling.alert_default_no_default.1, noquote))"@;
			put '")';
			put '	document.form1.action.value = "R";';
			put "	document.form1._program.value=&SELECT_POOL_SCHEME";
			put '	document.form1.submit()';
			put '//-->';
			put	'</script> ';
		run;	
	%end;
	%else %do;
		/* The KS_Stat macro creates the data set for plotting the KS graph.
			The following parameters are passed to this macro:
			input_ds		: Name of the input data set
			output_graph_ds	: Name of the data set which stores data for graph of ks
		*/
			%ks_stat(input_ds=&out_data_prep,output_graph_ds=&out_graph_ds);
	
		proc sql noprint;
	/*		select max(ks) into :max_ks from &out_graph_ds;*/
			/*The select stmt below is for display purpose only*/
			select max(ks) format=&CS_DEC_FMT. into :ks_value from &out_graph_ds;
		quit;
	%end;
%mend calculate_ks;
/* Perform action according to screen click */
%macro screenaction;
	/* Start: In case of all screens we need scheme details and master scale details */
	proc sql;
		create table &out_lib..scheme_details as
		select *
		from &lib..pool_scheme_defn
		where pool_scheme_type_cd in (&MAP_PD_MASTER_TYPE) and status_desc="%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.status_desc.1, noquote))";
	quit;

	proc sql;
		create table &out_lib..ms_scale_details as
		select distinct master_scale_no, master_scale_name
		from &lib..master_scale_defn;
	quit;

	/* Empty pool definition for first screen */
	%if %sysfunc(kupcase("&action")) eq "N" %then %do;
		proc sql;
			create table &lib..&pool_defn. (
				pool_seq_no numeric(8),
				from_pd numeric(8),
				to_pd numeric(8),
				pool_name char(100),
				no_of_accounts numeric(8),
				assigned_pd numeric(8),
				assigned_lgd numeric(8),
				assigned_ccf numeric(8),
				score_points numeric(8)
			);
		quit;
	%end;
	/* Edit Description */
	%else %if %sysfunc(kupcase("&action")) eq "D" %then %do;
		proc sql;
			update &lib..&pool_defn. set pool_name="%bquote(&pool_desc)" /*add %bquote() by scnhuh*/
			
			where pool_seq_no=&pool_no_desc;
		quit;
	%end;
	/* Save Pool Information */
	%else %if %sysfunc(kupcase("&action")) eq "V" %then %do;
		%save_pool;
	%end;
	%else %do;
		/* In all cases except for None, get the pool scheme information */
		proc sql;
/*I18N-scnhuh-20080420-1138*/
/*change sselect to select*/		
			select obs_dt, pd_method_type_cd, pd_model_rk, lgd_method_type_cd, lgd_model_rk,
				ccf_method_type_cd, ccf_model_rk, pool_scheme_type_cd into
				:obsdt, :pd_method, :pd_model_key, :lgd_method, :lgd_model_key,
				:ccf_method, :ccf_model_key, :scheme_type
			from &lib..pool_scheme_defn
			where pool_scheme_no=&scheme_no;
		quit;
/*I18N-scnhuh-20080420-1138*/
		

/*		%let obsdt=%trim(%left(&obsdt));*/
/*		%let model_key=%trim(%left(&model_key));*/
		%if "&pd_model_key" ne "" %then
		%let pd_model_key=%sysfunc(ktrim(%sysfunc(kleft(&pd_model_key))));
		%if "&lgd_model_key" ne "" %then
		%let lgd_model_key=%sysfunc(ktrim(%sysfunc(kleft(&lgd_model_key))));
		%if "&ccf_model_key" ne "" %then
		%let ccf_model_key=%sysfunc(ktrim(%sysfunc(kleft(&ccf_model_key))));

		/* In all cases except for None, get the names of the scored and long run datasets */
		data _null_;
			dt=&obsdt;
			mth=put(month(dt),z2.);
			yr=year(dt);
			dt1=yr||mth;
			call symput('data_dt', dt1);
		run;
		%if "&data_dt" ne "" %then
		%let data_dt=%sysfunc(ktrim(%sysfunc(kleft(&data_dt))));

		/* Scored Dataset */
		%let scr_pd_ds=_&pd_model_key._PD_&data_dt.;
		%let scr_lgd_ds=_&lgd_model_key._LGD_&data_dt.;
		%let scr_ccf_ds=_&ccf_model_key._CCF_&data_dt.;

		/* PD Dataset has to be available */
		proc sort data=&lib..&scr_pd_ds. out=work.&scr_pd_ds. SORTSEQ=EBCDIC;
			by account_rk;
		quit;

		data &lib..scr_&data_dt.;
			set work.&scr_pd_ds.;
			estimated_lgd=.;
			estimated_ccf=.;
		run;

		/* If LGD dataset is available, merge */
		%if %sysfunc(exist(&lib..&scr_lgd_ds.)) %then %do;
			proc sort data=&lib..&scr_lgd_ds. out=work.&scr_lgd_ds. SORTSEQ=EBCDIC;
				by account_rk;
			quit;

			data &lib..scr_&data_dt.;
				merge &lib..scr_&data_dt. work.&scr_lgd_ds.;
				by account_rk;
			run;
		%end;
		/* If CCF dataset is available, merge */
		%if %sysfunc(exist(&lib..&scr_ccf_ds.)) %then %do;
			proc sort data=&lib..&scr_ccf_ds. out=work.&scr_ccf_ds. SORTSEQ=EBCDIC;
				by account_rk;
			quit;

			data &lib..scr_&data_dt.;
				merge &lib..scr_&data_dt. work.&scr_ccf_ds.;
				by account_rk;
			run;
		%end;

		/* The structure of the scored dataset will be:
		----------------------------------------------------------------------------------
			Account Key, Score Points, Estimated PD, Estimated LGD, Estimated CCF
		----------------------------------------------------------------------------------
			First three columns will always have data. Estimated LGD and Estimated CCF
			will have data based on whether the user has provided LGD and CCF models.
		*/
		%let scored_ds_nm=scr_&data_dt.;

		/* Long Run Dataset:
		Datasets to be considered with the various cases for method types.
		----------------------------------------------------------------------------------
		PD		LGD		CCF							For Assignment
								----------------------------------------------------------
								PD					LGD					CCF
		----------------------------------------------------------------------------------
		E		E		E		_<M>_PD				_<M>_LGD			_<M>_CCF
		E		E		L		_<M>_PD				_<M>_LGD			_LNG_<M>_PD
		E		L		E		_<M>_PD				_LNG_<M>_PD_LGDYRS	_<M>_CCF
		L		E		E		_LNG_<M>_PD			_<M>_LGD			_<M>_CCF
		E		L		L		_<M>_PD				_LNG_<M>_PD_LGDYRS	_LNG_<M>_PD_LGDYRS
		L		E		L		_LNG_<M>_PD			_<M>_LGD			_LNG_<M>_PD
		L		L		E		_LNG_<M>_PD_LGDYRS	_LNG_<M>_PD_LGDYRS	_<M>_CCF
		L		L		L		_LNG_<M>_PD_LGDYRS	_LNG_<M>_PD_LGDYRS	_LNG_<M>_PD_LGDYRS
		----------------------------------------------------------------------------------
		Note: <M> stands for the PD model Key
		*/
		/* Long Run Dataset */
		%let long_run_ds_nm=;
		%let longrun_pd_ds=_lng_&pd_model_key._PD;
		%let longrun_pd_lgdyrs_ds=_lng_&pd_model_key._PD_LGDYRS;
		%let pd_mtd=%quote(%'&pd_method%');
		%let lgd_mtd=%quote(%'&lgd_method%');
		%let ccf_mtd=%quote(%'&ccf_method%');
			/* AVG: PD, LGD
			   LRM: CCF */
		%if %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			( %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) %then %do;
			%let long_run_ds_nm=&longrun_pd_ds;
		%end;
			/* AVG: PD, CCF
			   LRM: LGD */
		%else %if %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			( %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) %then %do;
			%let long_run_ds_nm=&longrun_pd_lgdyrs_ds;
		%end;
			/* AVG: LGD, CCF
			   LRM: PD */
		%else %if ( %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) %then %do;
			%let long_run_ds_nm=&longrun_pd_ds;
		%end;
			/* AVG: PD
			   LRM: LGD, CCF */
		%if %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			( %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			( %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) %then %do;
			%let long_run_ds_nm=&longrun_pd_lgdyrs_ds;
		%end;
			/* AVG: LGD
			   LRM: PD, CCF */
		%else %if ( %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) and
			( %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) %then %do;
			%let long_run_ds_nm=&longrun_pd_ds;
		%end;
			/* AVG: CCF
			   LRM: PD, LGD */
		%else %if ( %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			( %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&AVG_EST_TYPE.)) %then %do;
			%let long_run_ds_nm=&longrun_pd_lgdyrs_ds;
		%end;
			/* AVG: -
			   LRM: PD, LGD, CCF */
		%else %if ( %sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&pd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			( %sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&lgd_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) and
			( %sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_MAX_TYPE.)) or
			%sysfunc(kupcase(&ccf_mtd)) eq %sysfunc(kupcase(&LONG_RUN_AVG_TYPE.)) ) %then %do;
			%let long_run_ds_nm=&longrun_pd_lgdyrs_ds;
		%end;

		%put ************&scored_ds_nm*********&long_run_ds_nm*********;

		/* Retrieve Pool Information */
		%if %sysfunc(kupcase("&action")) ne "N" %then %do;
			%if %sysfunc(kupcase("&action")) ne "P" %then %do;
				proc sql noprint;
					select ks_value, stability_value, master_scale_no into
					:ks_value, :stab_value, :ms_scale_no
					from &lib..pool_scheme_defn
					where pool_scheme_no=&scheme_no;
				quit;
			%end;

			proc sql noprint;
				select count(*) into :cnt_pool
				from &lib..pool_defn
				where pool_scheme_no=&scheme_no;
			quit;

			%if &ms_scale_no ne . %then %do;
				/* If pool definitions exist for the scheme, get pool definitions */
				%if &cnt_pool ne 0 and %sysfunc(kupcase("&action")) ne "P" %then %do;
					proc sql;
						create table &lib..&pool_defn. as
						select pool_seq_no, from_pd_no as from_pd, to_pd_no as to_pd,
							pool_name, no_of_accounts, assigned_pd, assigned_lgd, assigned_ccf,
							score_points, ms_grade_name, ms_calc_pd_no as calculated_pd
						from &lib..pool_defn
						where pool_scheme_no=&scheme_no;
					quit;
				%end;
				/* If no pool definitions exist for the scheme, by default action should be
				to create pools using the Range Interval method using 100 as the interval */
				%else %do;
					/* Get the selected Master Scale */
					proc sql;
						create table &out_lib..sel_master_scale as
						select ms_rating_grade_cd, ms_grade_name,
							ms_grade_pd_no
						from &lib..master_scale_defn
						where master_scale_no=&ms_scale_no
						order by ms_grade_pd_no;
					quit;

					/* Calculate the ranges */
					proc sql noprint;
						select count(*) into :no_rec from &out_lib..sel_master_scale;
					quit;
					%let no_rec=%sysfunc(ktrim(%sysfunc(kleft(&no_rec))));

					proc transpose data=&out_lib..sel_master_scale out=work.trans;
						var ms_grade_pd_no;
					quit;

					data &out_lib..&out_rate_range.(keep=from_pd to_pd pd);
						retain from_pd 0;
						set work.trans;
						array val{&no_rec} col1-col&no_rec.;
						do i=1 to &no_rec.;
							pd=val(i);
							if i=&no_rec. then do;
								to_pd=1;
								output;
							end;
							else do;
								to_pd=(val(i)+val(i+1))/2;
								output;
								from_pd=to_pd;
							end;
						end;
					run;

					proc sql;
						create table &out_lib..&out_rate_range. as
						select from_pd, to_pd, pd, ms_rating_grade_cd, ms_grade_name
						from &out_lib..&out_rate_range., &out_lib..sel_master_scale
						where &out_rate_range..pd=sel_master_scale.ms_grade_pd_no;
					quit;

					%build_pool;
					%calculate_ks;
					%calculate_pool_stab;
				%end;
			%end;
		%end;
	%end;
%mend screenaction;
/************ END of Macros for Pool Calculation and definition ************/

data _null_;
	file _webout&webout_options;
	put'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" ';
	put'"http://www.w3.org/TR/html4/loose.dtd"> ';
	put'<html> ';
	put'<head> ';
/*I18N-scnhuh-20080418-1023*/
/*from iso-8859-1 to utf-8*/
	put'<meta http-equiv="Content-Type" content="text/html; charset=utf-8"> ';
/*I18N-scnhuh-20080418-1023*/	
	put'<title>'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.title_pooling_scr2.1, noquote))"@;
	put'</title> ';
	put'<link href="/SASCSPooling/styles/default.css"rel="stylesheet" type="text/css"> ';
	put '<style type="text/css">';
	put '<!--';
	put 'body {';
	put '	background-color: #FAF8ED;';
	put '}';
	put '-->';
	put '</style>';

	put '	<style type="text/css"> ';
	put '	#dhtmlgoodies_contentBox{ ';
	put '		border:1px solid #7F9DB9; ';
	put '		height:0px; ';
	put '		visibility:hidden; ';
	put '		position:absolute; ';
	put '		background-color:#FAF8ED; ';
	put '		overflow:hidden; ';
	put '		padding:2px; ';
	put '		width:250px; ';
	put '	} ';
	put '	#dhtmlgoodies_content{ ';
	put '		position:relative;		 ';
	put '		font-family: Trebuchet MS ';
	put '		width:100%; ';
/*	put '		font-size:0.8em; ';*/
	put '	} ';
	put '	#dhtmlgoodies_slidedown{ ';
	put '		position:relative; ';
	put '		width:20px; ';
	put '	} ';
	put '	</style> ';

	put '<script language="JavaScript" src="/SASCSPooling/script/action_menu.js"></script>';
	put '<script language="JavaScript" type="text/JavaScript"> ';
	put '<!-- ';

	/* When a scheme is selected, set the values in the required text boxes */
	put 'function setScheme(){';
	put '	index = document.form1.schemename.selectedIndex;';
	put '	if (index==0) {';
	put '		document.form1.scheme_no.value = ''''';
	put '		document.form1.action.value = "N";';
	put "		document.form1._program.value=&SELECT_POOL_SCHEME";
	put '		document.form1.submit()';
	put '	}';
	put '	else {';
	put '		document.form1.scheme_no.value = document.form1.schemename.options[index].value;';
	put '		if ((document.form1.ms_scale_no.value!='''') && (document.form1.ms_scale_no.value!=''.'')) {';
	put '			document.form1.ms_scale_no.value = ''''';
	put '		}';
	put '		document.form1.action.value = "M";';
	put "		document.form1._program.value=&SELECT_POOL_SCHEME";
	put '		document.form1.submit()';
	put '	}';
	put '}';

	put 'function setMS(){';
	put '	index = document.form1.msname.selectedIndex;';
	put '	if (index==0) {';
	put '		document.form1.ms_scale_no.value = ''''';
	put '		document.form1.action.value = "N";';
	put "		document.form1._program.value=&SELECT_POOL_SCHEME";
	put '		document.form1.submit()';
	put '	}';
	put '	else {';
	put '		document.form1.ms_scale_no.value = document.form1.msname.options[index].value;';
	put '		document.form1.action.value = "P";';
	put "		document.form1._program.value=&SELECT_POOL_SCHEME";
	put '		document.form1.submit()';
	put '	}';
	put '}';

	put 'function viewScheme(){';
	put '	index = document.form1.schemename.selectedIndex;';
	put '	if (index==0) {';
	/*I18N-scnjih-20080429-0941*/
	/*put '		alert("Select the scheme to view.")';*/
		put '		alert("'@;
		put "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.15, noquote))"@;
		put '")';	
	/*I18N-scnjih-20080429-0941*/
	put '	}';
	put '	else {';
	put '		document.form1.scheme_no.value = document.form1.schemename.options[index].value;';
	link="http://&_SRVNAME:&_SRVPORT/SASStoredProcess/do?"||'&_program='||"&VIEW_POOL_SCHEME"||'&scheme_no='||"&scheme_no";
	put '		window.open("'@;
	put link@;
	put '");';
	put '	}';
	put '}';

	put 'function SelPool(obj) {';
	put '	if(obj.checked==true) {';
	put '		document.form1.pool_no_desc.value=obj.value';
	put '	}';
	put '}';

	put 'function EditDesc() {';
	put '	if((document.form1.pool_no_desc.value=='''') || (document.form1.pool_no_desc.value=='' '')){';
	put '		alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.alert_select_pool_edit.1, noquote))"@;
	put '")';
	put '		slidedown_showHide();';
	put '	}';
	put '	else if((document.form1.edit_desc.value=='''') || (document.form1.edit_desc.value=='' '')){';
	put '		alert("'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.alert_enter_desc.1, noquote))"@;
	put '")';
	put '	}';
	put '	else {';
	put '		document.form1.pool_desc.value=document.form1.edit_desc.value';
	put "		document.form1._program.value=&SELECT_POOL_SCHEME";
	put '		document.form1.action.value="D"';
	put '		document.form1.submit()';
	put '	}';
	put '}';

	put 'function OK() {';
	put '	EditDesc()';
	put '}';

	put 'function Save() {';
	put "	document.form1._program.value=&SELECT_POOL_SCHEME";
	put '	document.form1.action.value="V"';
	put '	document.form1.submit()';
	put '}';

	put '//-->';
	put	'</script> ';
	put '</head>';

	put '<body>';
	put '<form name="form1" method="post" ACTION="/SASStoredProcess/do"> ';

	/* Hidden textboxes to hold the values for the program name and the output paremeters */
	put '<input type="hidden" name="_program" VALUE="">';
	scheme_type=symget("scheme_type");
	x='<input type="hidden" name="scheme_type" value="'||ktrim(kleft(scheme_type))||'">';
	put x;
	scheme_no=symget("scheme_no");
	x='<input type="hidden" name="scheme_no" value="'||ktrim(kleft(scheme_no))||'">';
	put x;
	ms_scale_no=symget("ms_scale_no");
	x='<input type="hidden" name="ms_scale_no" value="'||ktrim(kleft(ms_scale_no))||'">';
	put x;
	action=symget("action");
	x='<input type="hidden" name="action" value="'||ktrim(kleft(action))||'">';
	put x;
/*	put '<input type="hidden" name="action" value="">';*/
	put '<input type="hidden" name="pool_desc" value="">';
	put '<input type="hidden" name="pool_no_desc" value="">';
run;

/* Calculate the data according to the screen events */
%screenaction;

data _null_;
	file _webout&webout_options;
	put '<table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#FAF8ED"> ';
	put '	<tr> ';
	put '		<td align="left"> ';
	put '		<table width="100%" border="0" align="left" cellpadding="0" cellspacing="0"> ';
	put '			<tr> ';
	put '				<td><table width="100%" border="0" cellspacing="0" cellpadding="0"> ';
	put '					<tr> ';

	/* Navigational Links */
	put '						<td><span class="BreadCrumb">&nbsp;</td>';
	put '						<td align="right"><img src="/SASCSPooling/images/CrBanner.gif" width="259" height="29"></td> ';
	put '					</tr>';
	put '				</table>';                                     /****Banner insert image********/
	/**/

	/* Main Body */
	put '				<H2 align="left"><table width="100%" border="0" cellspacing="0" cellpadding="0">';
	put '					<tr><td>'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.table_nm_map_pd_mstr_scl.1, noquote))"@;
	put '</td>';
	put '				</tr></table></H2>';
	put '	   		<tr><td>&nbsp;</td></tr>';
	put '      		<tr>';
	put '        		<td><table width="100%" border="0" cellpadding="3" cellspacing="0">';
	put '					<tr>';
	put '						<td class="LabelFont" width="15%">'@;
	put "%sysfunc(sasmsg(smd_ds.bismsg,crs.build_scheme_rating_pool.table_nm_schm_nm.1, noquote))"@;
	put '</td>';
	put '						<td class="LabelFont">';
	put '							<p class="LabelFont">';
	put '							<select name="schemename" style="width: 90mm" onChange="javascript:setScheme()">';
/*I18N-scnjih-20080421-1039*/
/*	put '								<option>Select Scheme</option>';*/
	put '								<option>'@;
	put  "%sysfunc(sasmsg(smd_ds.bismsg,build_scheme_rating_pool.sas.2, noquote))"@;
	put '</option>';
/*I18N-scnjih-20080421-1039*/
run;

data _null_;
	file _webout&webout_options;
	set &out_lib..scheme_details;
	put '								<option value="'@;
	put pool_scheme_no@;
	put '" '@;
	if pool_scheme_no=&scheme_no then put 'selected'@;
	put '>'@;
	put pool_scheme_name@;
	put '</option>';
run;

data _null_;
	file _webout&webout_options;
	put '							</select>';
	 /*I18N-scnjih-20080421-1039*/
/*	put '								<a href="javascript:viewScheme()">View Details</a>';*/
	put '								<a href="javascript:viewScheme();">' @;
	put "%sysfunc( sasmsg(smd_ds.bismsg, build_scheme_rating_pool.sas.1, noquote))" @;
	put '</a>';
	 /*I18N-scnjih-20080421-1039*/
	put '							</p>';
	put '						</td>';
	put '					</tr>';
run;

%display_screen;

data _null_;
	file _webout&webout_options;
	put '				</table>';
	put '				</td>';
	put '			</tr>';
	put '			<tr><td>&nbsp;</td></tr>';
	put '</table>';
	put '</form>';
	put '</body>';
	put '</html>';
run;
options nomprint;
proc printto;
run;

